         PRINT NOGEN
         BEGIN NAME=QEA0,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEA0                                       *
* RELATED MODULES. QEA1-QEA6, QEAA-QEAF                       *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF TEST DRIVER COMMAND PARSER            *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT RECEIVES ALL "ZTEST MPIF" COMMANDS *
*             FROM SEGMENT CVZZ.  THE COMMAND IS PARSED AND   *
*             AN APPROPRIATE SEGMENT IS CALLED TO PROCESS THE *
*             REQUEST.                                        *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   DEPENDENCY.... NONE                                       *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBW000 WORK AREA WILL BE SET UP WITH  *
*                       VALUES TO BE PASSED TO THE OTHER      *
*                       SEGMENTS.  THESE VALUES CORRESPOND TO *
*                       THE PARAMETERS SPECIFIED IN THE       *
*                       COMMAND.  SEE BELOW FOR DETAILS.      *
*                                                             *
* REGISTER-CONVENTIONS..R1  - SD0RV  BASE REGISTER            *
*                       R4  - MS0UT  BASE REGISTER            *
*                       R6  - DCTBPK BASE REGISTER            *
*                       R14 - GLOBAL AREA 1 WORK REGISTER     *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
* RESTRICTIONS... - MPIF MUST BE INSTALLED                    *
*                 - THE DRIVER SHOULD BE ABLE TO RUN IN ANY   *
*                   SUBSYSTEM-USER                            *
*                 - THE SYSTEM MUST BE AT 1052 STATE OR       *
*                   HIGHER FOR ALL FUNCTIONS EXCEPT 'SETUP'   *
*                 - THE DRIVER CANNOT RUN WHEN IPL IS OFF A   *
*                   LOADER GENERAL FILE                       *
*                 - DRIVER MUST BE EXECUTING IN THE MAIN      *
*                   I-STREAM, EXCEPT 'RESTART', 'SEND',       *
*                   'FORGET', AND 'HELP'                      *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * SEE BELOW            *
* DATA LEVELS.. * D0 - INPUT MESSAGE   * NONE                 *
* REGISTERS.... * NONE                 * NONE                 *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  CMMOPE  (SYSTEM ERROR OPTION WORD)       *
*                    CMMCTKM (KEYPOINT M)                     *
*                    MS0UT   (SUBSYSTEM-USER ATTRIBUTE TABLE) *
*                    @ISSDRV (DRIVER GLOBALS)                 *
*                        - @SYSLEV  : LEVEL OF SYSTEM TEST    *
*                                     ACTIVITY                *
*                        - @SYSINT  : FREQUENCY OF SYSTEM     *
*                                     TEST ACTIVITY           *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL..ENTNC QEA1 - "ZTEST MPIF START" ENTERED        *
*                    - EBW000 = X'00'                         *
*                                                             *
*              ENTNC QEA1 - "ZTEST MPIF STOP" ENTERED         *
*                    - EBW000 = X'03'                         *
*                                                             *
*              ENTNC QEA2 - "ZTEST MPIF RESTART" ENTERED      *
*                    - EBW000-EBW007 = USER'S NAME            *
*                    - EBW008 = EBCDIC PATH CLASS OF USER     *
*                    - EBW009 = BINARY PATH CLASS OF USER     *
*                    - EBW010 = DEVICE TYPE OF THE USER       *
*                                                             *
*              ENTNC QEA3 - "ZTEST MPIF SEND" ENTERED         *
*                    - EBW000-EBW007 = USER'S NAME            *
*                    - EBW008 = SIZE OF MESSAGES TO BE SENT,  *
*                               AND THE BLOCK TYPE            *
*                    - EBW010 = NUMBER OF MESSAGES TO BE SENT *
*                    - EBW020-EBW023 = X'00000000', ENTRY NOT *
*                                      BY CREMC FROM QEA3     *
*                                                             *
*              ENTNC QEA4 - "ZTEST MPIF FORGET" ENTERED       *
*                    - EBW000-EBW007 = USER'S NAME            *
*                                                             *
*              ENTNC QEAE - "ZTEST MPIF SETUP" ENTERED        *
*                    - EBW000-EBW001 = X'0000', ENTRY NOT BY  *
*                                      CRETC FROM QEAE        *
*                                                             *
*              ENTNC QEAF - "ZTEST MPIF STATUS" ENTERED       *
*                    - EBW000 = X'00', 'ALL' NOT SPECIFIED    *
*                    - EBW000 = X'FF', 'ALL' SPECIFIED        *
*                                                             *
*              EXITC      - "ZTEST MPIF HELP" ENTERED         *
*                                                             *
*     -ERROR... EXITC - MPIF NOT INSTALLED                    *
*                     - RUNNING ON A LOADER GENERAL FILE IPL  *
*                     - A BAD PARAMETER WAS ENTERED           *
*                     - THE SYSTEM IS IN RESTART              *
*                     - SYSTEM NOT RUNNING ON MAIN I-STREAM   *
*                                                             *
***************************************************************
         EJECT
*********************************************************************
* DO SOME INITIAL PROCESSING.  DEFINE DSECTS, AND ESTABLISH
* ADDRESSABILITY TO THE DRIVER GLOBAL AREA.  THEN, CHECK TO SEE IF
* MPIF IS INSTALLED, OR IF THIS IPL WAS FROM A LOADER GENERAL FILE.
* THE DRIVER CANNOT RUN UNLESS MPIF IS INSTALLED, AND THE IPL IS FROM
* A REAL-TIME MOD.
*********************************************************************
         SPACE  2
         DCTBPK REG=R6                 INPUT PARAMETER LIST DSECT
         DCTMUS REG=R15                NEEDED TO PICK UP EQUATES ONLY
         MK0CK  REG=R3                 KEYPOINT 'M' DSECT
         MS0UT  REG=R4                 SUBSYSTEM-USER ATTRIBUTE TABLE
         SPACE  2
         SD0RV  REG=R1                 @ISSDRV DSECT
         GLOBZ  REGR=R14               GET GLOBAL 1 ADDRESS AND DSECT
         L      R1,@ISSDRV             SET SD0RV BASE REGISTER
         SPACE  2
         SYSTC  SBMPIF,IFOFF=QEA03000  MAKE SURE MPIF IS INSTALLED
         SPACE  2
         CINFC  R,CMMCTKM,F,REG=R3     GET CORE ADDRESS OF CTKM
         TM     MK0SST,MKIPLGF         WAS IPL FROM GENERAL FILE?
         BNZ    QEA03010               YES - GO ISSUE AN ERROR MESSAGE
         EJECT
*********************************************************************
* ENTRY IS FROM CVZZ, THE MESSAGE SHOULD BE IN THE FORMAT:
*
*                 ZMPIF ACTION PARAMETERS
*
* WHERE ACTION IS SETUP, START, STOP, STATUS, RESTART, SEND, FORGET,
* OR HELP.  THE PARAMETERS WILL VARY ACCORDING TO THE SPECIFIED
* ACTION.
*********************************************************************
         SPACE 2
         STM   R0,R7,EBX000               SAVE REGISTERS ACROSS BPKDC
         BPKDC EXECUTE=YES,BPKD=MAINBPK   PARSE THE I/P MESSAGE
         LM    R0,R7,EBX000               RESTORE REGISTERS
         L     R6,CE1CR0                  GET BASE OF INPUT PARAM. LIST
         L     R7,ACTION                  INDEX VALUE OF GIVEN ACTION
         B     QEA00100(R7)               BRANCH TO TABLE + INDEX VALUE
         SPACE 2
QEA00100 DS    0H                      BRANCH TABLE FOR EACH ACTION
         B     QEA00900                INVALID INDEX GENERATED BY BPKDC
         B     QEA00200                ACTION IS 'SETUP'
         B     QEA00300                ACTION IS 'START'
         B     QEA00400                ACTION IS 'STOP'
         B     QEA00500                ACTION IS 'STATUS'
         B     QEA00600                ACTION IS 'RESTART'
         B     QEA00700                ACTION IS 'SEND'
         B     QEA00800                ACTION IS 'FORGET'
         B     QEA00900                ACTION IS 'HELP'
         EJECT
*********************************************************************
* "ZTEST MPIF SETUP" WAS ENTERED.  QEAE WILL BE CALLED TO DEFINE
* MPIF PARAMETERS USING "ZMPIF" COMMANDS.
*
* NOTE: EBROUT AND CE1CPD ARE NOT SAVED IN THE GLOBAL AREA LIKE THEY
* ARE FOR OTHER DRIVER FUNCTIONS.  THIS IS BECAUSE SETUP IS ALLOWED
* TO RUN IN RESTART, AND GLOBALS MAY NOT BE LOADED YET.
*********************************************************************
         SPACE 2
QEA00200 DS    0H
         CLC   CE1ISN,=X'0001'         RUNNING ON THE MAIN I-STREAM?
         BNE   QEA03110                NO - GO PRINT AN ERROR MESSAGE
         SPACE 2
         XC    EBW000(2),EBW000        QEAE ENTRY FROM QEA0
         ENTNC QEAE                    GO SET UP THE MPIF COMPLEX
         EJECT
*********************************************************************
* "ZTEST MPIF START" WAS ENTERED.  PLACE THE LEVEL AND INTERVAL
* PARAMETERS IN THE DRIVER GLOBAL, AND CALL QEA1 TO PROCESS THE
* REQUEST.
*********************************************************************
         SPACE 2
QEA00300 DS    0H
         CLC   CE1ISN,=X'0001'         RUNNING ON THE MAIN I-STREAM?
         BNE   QEA03110                NO - GO PRINT AN ERROR MESSAGE
         SPACE 2
         BAS   R2,QEA02000             GO DO SOME FRONT-END PROCESSING
         SPACE 2
         L     R7,LEVEL                GET ADDRESS OF LEVEL PARAMETER
         LTR   R7,R7                   WAS LEVEL PARAMETER SPECIFIED?
         BZ    QEA00310                NO - GO GIVE THE DEFAULT VALUE
         MVC   @SYSLEV(2),2(R7)        SAVE LEVEL IN @ISSDRV GLOBAL
         B     QEA00320
         SPACE 2
QEA00310 DS    0H
         LA    R15,LEVDEF              LOAD THE DEFAULT LEVEL INTO R15
         STH   R15,@SYSLEV             STORE DEFAULT LEVEL INTO GLOBAL
         SPACE 2
QEA00320 DS    0H
         L     R7,INTERVAL             GET ADDRESS OF INTERVAL PARAM.
         LTR   R7,R7                   WAS THE PARAMETER SPECIFIED?
         BZ    QEA00330                NO - GO GIVE THE DEFAULT VALUE
         CLC   2(2,R7),=X'0000'        WAS INTERVAL = 0 SPECIFIED?
         BE    QEA03020                YES - GO ISSUE ERROR MESSAGE
         MVC   @SYSINT(2),2(R7)        SAVE INTERVAL IN @ISSDRV GLOBAL
         B     QEA00340
         SPACE 2
QEA00330 DS    0H
         LA    R15,INTDEF              LOAD THE DEFAULT INTERVAL
         STH   R15,@SYSINT             STORE DEFAULT INTO GLOBAL
         SPACE 2
QEA00340 DS    0H
         XC    EBW000(1),EBW000        QEA1 ENTRY DUE TO START MESSAGE
         ENTNC QEA1                    SYSTEM TEST ACTIVITY SEGMENT
         EJECT
*********************************************************************
* "ZTEST MPIF STOP" WAS ENTERED.  IF 'ALL' IS SPECIFIED ALL
* SUBSYSTEM-USERS WILL BE STOPPED.  THIS WILL BE DONE BY INVOKING
* QEA1 MULTIPLE TIMES.  @EBROUT AND @CE1CPD WILL ALSO BE SET UP FOR
* ALL SSU.  IF THIS IS NOT DONE, AND THIS IS FIRST MPIF DRIVER
* MESSAGE, QEA1 WILL BE CALLED WITH THESE FIELDS UNINITIALIZED.  ANY
* WTOPC MACROS WILL RESULT IN AN OPR-346 (ROUTING ERROR).
*********************************************************************
         SPACE 2
QEA00400 DS    0H
         CLC   CE1ISN,=X'0001'         RUNNING ON THE MAIN I-STREAM?
         BNE   QEA03110                NO - GO PRINT AN ERROR MESSAGE
         SPACE 2
         BAS   R2,QEA02000             GO DO SOME FRONT-END PROCESSING
         SPACE 2
         MVI   EBW000,X'03'            QEA1 ENTRY DUE TO STOP MESSAGE
         L     R7,SUBSYS               GET ADDRESS OF SUBSYS PARAMETER
         LTR   R7,R7                   WAS SUBSYS PARAMETER SPECIFIED?
         BNZ   QEA00410                YES - GO PROCESS IT
         ENTNC QEA1                    NO  - ONLY STOP THIS SSU
         SPACE 2
QEA00410 DS    0H
         CLC   0(4,R7),=X'03C1D3D3'    WAS SUBSYS = 'ALL' SPECIFIED?
         BNE   QEA03030                NO  - GO ISSUE ERROR MESSAGE
         SPACE 2
         CINFC R,CMMSSUT,F,REG=R4,BSS=YES  GET SSU TABLE ADDRESS
         L     R0,MU0CNT                   LOAD NUMBER OF SSU IN SYSTEM
         SPACE 2
QEA00420 DS    0H
         LH    R14,MU0SSU              SSU ID CONTAINED IN SSU TABLE
         CEBIC SSU                     CHANGE SSU ID OF THIS ECB
         SPACE 2
         GLOBZ REGR=R14                GLOBAL 1 ADDRESS FOR THIS SSU
         L     R1,@ISSDRV              SD0RV BASE REGISTER FOR THIS SSU
         MVC   @EBROUT(3),EBROUT       SAVE EBROUT FOR MESSAGE ROUTING
         MVC   @CE1CPD(1),CE1CPD       SAVE CE1CPD FOR MESSAGE ROUTING
         SPACE 2
         LA    R14,1                   ONLY PASS 1 BYTE OF INFORMATION
         LA    R15,EBW000              PASS EBW000 PARAMETER TO QEA1
         CREMC QEA1                    CREATE AN ECB TO STOP THIS SSU
         SPACE 2
         AHI   R4,MU0LEN               POINT TO NEXT SLOT IN SSU TABLE  64ML3221
         BCT   R0,QEA00420             BRANCH IF MORE SSU TO STOP
         EXITC                         THIS ECB IS DONE, CAN EXIT
         EJECT
*********************************************************************
* "ZTEST MPIF STATUS" WAS ENTERED.  IF 'ALL' IS SPECIFIED SUMMARY
* STATUS FOR ALL SUBSYSTEM-USERS WILL BE DISPLAYED.  OTHERWISE,
* DETAIL STATUS FOR THIS SSU WILL BE DISPLAYED.
*********************************************************************
         SPACE 2
QEA00500 DS    0H
         CLC   CE1ISN,=X'0001'         RUNNING ON THE MAIN I-STREAM?
         BNE   QEA03110                NO - GO PRINT AN ERROR MESSAGE
         SPACE 2
         BAS   R2,QEA02000             GO DO SOME FRONT-END PROCESSING
         SPACE 2
         L     R7,SUBSYS               GET ADDRESS OF SUBSYS PARAMETER
         LTR   R7,R7                   WAS SUBSYS PARAMETER SPECIFIED?
         BZ    QEA00510                NO  - STATUS FOR THIS SSU ONLY
         CLC   0(4,R7),=X'03C1D3D3'    WAS SUBSYS = 'ALL' SPECIFIED?
         BNE   QEA03040                NO  - GO ISSUE ERROR MESSAGE
         MVI   EBW000,X'FF'            INDICATE 'ALL' WAS SPECIFIED
         ENTNC QEAF                    STATUS DISPLAY SEGMENT
         SPACE 2
QEA00510 DS    0H
         MVI   EBW000,X'00'            INDICATE 'ALL' NOT SPECIFIED
         ENTNC QEAF                    STATUS DISPLAY SEGMENT
         EJECT
*********************************************************************
* "ZTEST MPIF RESTART" WAS ENTERED.  THE PARAMETERS FROM THE COMMAND
* WILL BE PARSED, ERROR CHECKED, AND PLACED IN THE EBW000 WORK AREA.
* QEA2 WILL THEN BE CALLED TO PROCESS THE REQUEST.
*********************************************************************
         SPACE 2
QEA00600 DS    0H
         BAS   R2,QEA02000             GO DO SOME FRONT-END PROCESSING
         BAS   R2,QEA01000             GO MOVE USER NAME INTO ECB
         SPACE 2
         L     R7,CLASS                GET ADDRESS OF CLASS PARAMETER
         LTR   R7,R7                   WAS CLASS PARAMETER SPECIFIED?
         BZ    QEA00610                NO - GO GIVE DEFAULT PATH CLASS
         MVC   EBW008(1),1(R7)         MOVE EBCDIC PATH CLASS TO ECB
         B     QEA00620                GO CONVERT PATH CLASS TO BINARY
QEA00610 DS    0H
         MVI   EBW008,C'B'             MOVE DEFAULT PATH CLASS INTO ECB
         SPACE 2
QEA00620 DS    0H
         SR    R0,R0                   CLEAR REGISTER
         IC    R0,EBW008               INSERT EBCDIC PATH CLASS
         S     R0,=X'000000C0'         SUBTRACT X'C0' TO CREATE A BASE
         STC   R0,EBW009               SAVE PATH CLASS DISPLACEMENT
         TR    EBW009,CLSTRNS          TRANSLATE DISPLACEMENT TO BINARY
         SPACE 2
         L     R7,DEVICE               GET ADDRESS OF DEVICE PARAMETER
         LTR   R7,R7                   WAS DEVICE PARAMETER SPECIFIED?
         BNZ   QEA00630                YES - CONTINUE MESSAGE PARSING
         MVI   EBW010,DEVCTOC          NO  - SET DEFAULT DEVICE IN ECB
         B     QEA00650                GO ENTER RESTART SEGMENT QEA2
QEA00630 DS    0H
         CLC   1(4,R7),=C'CTOC'        WAS DEVICE = CTOC SPECIFIED?
         BNE   QEA00640                NO  - GO CHECK IF DEVICE = MTOM
         MVI   EBW010,DEVCTOC          YES - SET DEVICE = CTOC IN ECB
         B     QEA00650                GO ENTER RESTART SEGMENT QEA2
QEA00640 DS    0H
         CLC   1(4,R7),=C'MTOM'        WAS DEVICE = MTOM SPECIFIED?
         BNE   QEA03050                NO  - GO ISSUE ERROR MESSAGE
         MVI   EBW010,DEVMTOM          YES - SET DEVICE = MTOM IN ECB
         SPACE 2
QEA00650 DS    0H
         ENTNC QEA2                    CALL DRIVER STARTUP PROGRAM
         EJECT
*********************************************************************
* "ZTEST MPIF SEND" WAS ENTERED.  THE PARAMETERS FROM THE COMMAND
* WILL BE PARSED, ERROR CHECKED, AND PLACED IN THE EBW000 WORK AREA.
* QEA3 WILL BE CALLED TO PROCESS THE REQUEST.
*
* THE MESSAGES WILL BE SENT IN A BLOCK DESCRIBED BY AN INDEX IN
* EBW008.  THE INDEX IS DEFINED AS FOLLOWS:
*
*        X'00' - 381  BYTE 'ORDINARY' BLOCK (NON-SHAREABLE)
*        X'01' - 1055 BYTE 'ORDINARY' BLOCK (NON-SHAREABLE)
*        X'02' - 4096 BYTE 'ORDINARY' BLOCK (NON-SHAREABLE)
*        X'03' - 381  BYTE COMMON BLOCK
*        X'04' - 1055 BYTE COMMON BLOCK
*        X'05' - 4096 BYTE COMMON BLOCK
*********************************************************************
         SPACE 2
QEA00700 DS    0H
         BAS   R2,QEA02000             GO DO SOME FRONT-END PROCESSING
         BAS   R2,QEA01000             GO MOVE USER NAME INTO ECB
         SPACE 2
         L     R7,MSGS                 GET ADDRESS OF MSGS PARAMETER
         LTR   R7,R7                   WAS MSGS PARAMETER SPECIFIED?
         BZ    QEA00710                NO - GO GIVE DEFAULT MSGS
         CLC   2(2,R7),=X'0000'        WAS MSGS = 0 SPECIFIED?
         BE    QEA03060                YES - GO ISSUE ERROR MESSAGE
         MVC   EBW010(2),2(R7)         NUMBER MESSAGES TO ECB
         B     QEA00720
QEA00710 DS    0H
         MVC   EBW010(2),=X'000A'      MOVE DEFAULT MSGS INTO ECB
         SPACE 2
QEA00720 DS    0H
         L     R7,SIZE                 GET ADDRESS OF SIZE PARAMETER
         LTR   R7,R7                   WAS SIZE PARAMETER SPECIFIED?
         BNZ   QEA00730                YES - CONTINUE MESSAGE PARSING
         LA    R2,1                    GIVE DEFAULT SIZE = 1055
         B     QEA00760                GO ENTER 'SEND' PROGRAM (QEA3)
QEA00730 DS    0H
         CLC   0(4,R7),=X'03F3F8F1'    SENDING 381 BYTE BLOCKS?
         BNE   QEA00740                NO - GO CHECK FOR 1055 BLOCK
         LA    R2,0                    INDICATE SENDING 381 BYTE BLOCK
         B     QEA00760                GO ENTER 'SEND' PROGRAM (QEA3)
QEA00740 DS    0H
         CLC   0(5,R7),=X'04F1F0F5F5'  SENDING 1055 BYTE BLOCKS?
         BNE   QEA00750                NO - GO CHECK FOR 4K BLOCK
         LA    R2,1                    INDICATE SENDING 1055 BYTE BLOCK
         B     QEA00760                GO ENTER 'SEND' PROGRAM (QEA3)
QEA00750 DS    0H
         CLC   0(3,R7),=X'02F4D2'      SENDING 4K BYTE BLOCKS?
         BNE   QEA03070                NO - INVALID SIZE ENTERED
         LA    R2,2                    INDICATE SENDING 4K BYTE BLOCK
         SPACE 2
QEA00760 DS    0H
         L     R7,COMMON               GET ADDRESS OF COMMON PARAMETER
         LTR   R7,R7                   WAS COMMON PARAMETER SPECIFIED?
         BZ    QEA00780                NO  - NO NEED TO ADJUST INDEX
         CLC   0(4,R7),=X'03E8C5E2'    USER WANT TO SEND COMMON BLOCKS?
         BNE   QEA00770                NO - GO CHECK FOR COMMON = NO
         A     R2,=F'3'                ADJUST INDEX IN R2 FOR COMMON
         B     QEA00780                GO ENTER 'SEND' PROGRAM (QEA3)
QEA00770 DS    0H
         CLC   0(3,R7),=X'02D5D6'      USER SPECIFY COMMON = NO?
         BNE   QEA03100                NO - GO ISSUE ERROR MESSAGE
         SPACE 2
QEA00780 DS    0H
         STCM  R2,B'0001',EBW008       SAVE SIZE/TYPE INDICATOR IN ECB
         XC    EBW020(4),EBW020        INDICATE ENTRY NOT FROM QEA3
         ENTNC QEA3                    CALL DRIVER MESSAGE SEND PROGRAM
         EJECT
*********************************************************************
* "ZTEST MPIF FORGET" WAS ENTERED.  THE NAME PARAMETER FROM THE
* COMMAND WILL BE PARSED, ERROR CHECKED, AND PLACED IN THE EBW000
* WORK AREA. QEA4 WILL THEN BE CALLED TO PROCESS THE REQUEST.
*********************************************************************
         SPACE 2
QEA00800 DS    0H
         BAS   R2,QEA02000             GO DO SOME FRONT-END PROCESSING
         BAS   R2,QEA01000             GO MOVE USER NAME INTO ECB
         ENTNC QEA4                    CALL FORGET PROCESSING
         EJECT
*********************************************************************
* "ZTEST MPIF HELP" WAS ENTERED.
*********************************************************************
         SPACE 2
QEA00900 DS    0H
         BAS   R2,QEA02000             GO DO SOME FRONT-END PROCESSING
         SPACE 2
         WTOPC TEXTA=HELPMSG1,NUM=01,LET=I,PREFIX=QEA0,                X
               CHAIN=YES,ENDOFM=NO
         WTOPC TEXTA=HELPMSG2,HEADER=NO,CHAIN=YES,ENDOFM=NO
         WTOPC TEXTA=HELPMSG3,HEADER=NO,CHAIN=YES,ENDOFM=YES
         EXITC
         SPACE 2
HELPMSG1 DS    0D
         DC    AL1(END1-BEGIN1)
BEGIN1   DC    C'VALID MPIF TEST DRIVER COMMANDS:',AL1(#CAR),AL1(#CAR)
         DC    C'  ZTEST MPIF SETUP',AL1(#CAR),AL1(#CAR)
         DC    C'  ZTEST MPIF Restart User-xxxxxxxx,Class-y,Device-(MTOX
               M|CTOC)',AL1(#CAR)
         DC    C'  ZTEST MPIF SENd User-xxxxxxxx,Msgs-yy,Size-(381|1055X
               |4K),Common-(YES|NO)'
END1     EQU   *
         SPACE 2
HELPMSG2 DS    0D
         DC    AL1(END2-BEGIN2)
BEGIN2   DC    C'  ZTEST MPIF Forget User-xxxxxxxx',AL1(#CAR),AL1(#CAR)
         DC    C'  ZTEST MPIF START (Level-mm) (Interval-nn)',AL1(#CAR)
         DC    C'  ZTEST MPIF STOP (ALL)',AL1(#CAR)
         DC    C'  ZTEST MPIF STATus (ALL)',AL1(#CAR)
END2     EQU   *
         SPACE 2
HELPMSG3 DS    0D
         DC    AL1(END3-BEGIN3)
BEGIN3   DC    C'Defaults are provided for the LEVEL and INTERVAL paramx
               eters on the START',AL1(#CAR)
         DC    C'message. The defaults should be used for most SYSTEM aX
               nd REGRESSION test runs.'
END3     EQU   *
         EJECT
*********************************************************************
* THIS ROUTINE WILL MOVE THE NAME OF THE USER SPECIFIED IN THE INPUT
* MESSAGE TO EBW000.  THE NAME WILL BE LEFT JUSTIFIED AND PADDED WITH
* BLANKS.  BPKDC GUARANTEES THE USER PARAMETER WAS SPECIFIED, AND
* DOES NOT EXCEED 8 CHARACTERS.  THIS ROUTINE ALSO CHECKS TO SEE IF
* THE USER NAME IS ONE OF THE RESERVED NAMES.  IF SO, AN ERROR
* MESSAGE IS PRINTED.
*********************************************************************
         SPACE 2
QEA01000 DS    0H
         L     R7,USER                 GET ADDRESS OF USER PARAMETER
         SPACE 2
         CLC   0(5,R7),=X'04D4D7C9C6'  IS THE USER NAME "MPIF"?
         BE    QEA03080                YES - NAME IS INVALID
         CLC   0(4,R7),=X'03C9D7C3'    IS THE USER NAME "IPC"?
         BE    QEA03080                YES - NAME IS INVALID
         SPACE 2
         SR    R5,R5                   CLEAR LENGTH REGISTER
         IC    R5,0(R7)                LOAD LENGTH OF INPUT NAME
         LA    R7,1(,R7)               POINT TO FIRST CHARACTER OF NAME
         LA    R4,EBW000               LOAD DESTINATION OF THE NAME
         LA    R3,EBW007               ADDRESS OF END OF DESTINATION
         SPACE 2
QEA01010 DS    0H
         MVC   0(1,R4),0(R7)           MOVE ONE CHARACTER OF INPUT NAME
         LA    R7,1(,R7)               POINT TO NEXT CHARACTER IN NAME
         LA    R4,1(,R4)               POINT NEXT CHAR. IN DESTINATION
         BCT   R5,QEA01010             BRANCH IF MORE CHARS. TO MOVE
         SPACE 2
QEA01020 DS    0H
         CLR   R4,R3                   ARE WE AT END OF DESTINATION?
         BHR   R2                      YES - RETURN TO CALLER
         MVI   0(R4),C' '              NO  - BLANK A CHARACTER
         LA    R4,1(,R4)               POINT NEXT CHAR. IN DESTINATION
         B     QEA01020                GO MOVE ANOTHER CHARACTER
         EJECT
*********************************************************************
* THIS ROUTINE DOES SOME COMMON FRONT-END PROCESSING FOR MOST MPIF
* REQUESTS.  FIRST, CHECK TO SEE IF THE SYSTEM IS IN RESTART.  IF SO,
* AN ERROR MESSAGE WILL BE ISSUED, BECAUSE MOST MPIF DRIVER FUNCTIONS
* CANNOT RUN DURING RESTART BEFORE GLOBALS ARE LOADED.  NEXT, THE
* ROUTINE WILL SETUP THE @EBROUT AND @CE1CPD FIELDS IN THE DRIVER
* GLOBAL.
*********************************************************************
         SPACE 2
QEA02000 DS    0H
         CINFC R,CMMOPE,F              GET SYSTEM ERROR OPTION WORD
         TM    1(R14),X'02'            IS THE SYSTEM IN RESTART?
         BO    QEA03090                YES - GO ISSUE AN ERROR MESSAGE
         SPACE 2
         MVC   @EBROUT(3),EBROUT       SAVE EBROUT FOR MESSAGE ROUTING
         MVC   @CE1CPD(1),CE1CPD       SAVE CE1CPD FOR MESSAGE ROUTING
         BR    R2                      RETURN TO CALLER
         EJECT
*********************************************************************
* THE FOLLOWING ROUTINES WILL ISSUE ERROR MESSAGES FOR INVALID
* PARAMETERS AND ERROR CONDITIONS.
*********************************************************************
         SPACE 2
QEA03000 DS    0H
         LA    R4,2                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG02L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03010 DS    0H
         LA    R4,3                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG03L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03020 DS    0H
         LA    R4,4                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG04L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03030 DS    0H
         LA    R4,5                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG05L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03040 DS    0H
         LA    R4,6                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG06L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03050 DS    0H
         LA    R4,7                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG07L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03060 DS    0H
         LA    R4,8                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG08L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03070 DS    0H
         LA    R4,9                    LOAD THE MESSAGE NUMBER
         LA    R5,MSG09L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03080 DS    0H
         LA    R4,10                   LOAD THE MESSAGE NUMBER
         LA    R5,MSG10L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03090 DS    0H
         LA    R4,11                   LOAD THE MESSAGE NUMBER
         LA    R5,MSG11L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03100 DS    0H
         LA    R4,12                   LOAD THE MESSAGE NUMBER
         LA    R5,MSG12L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03110 DS    0H
         LA    R4,13                   LOAD THE MESSAGE NUMBER
         LA    R5,MSG13L               POINT TO THE MESSAGE TEXT
         B     QEA03500                GO ISSUE THE WTOPC
         SPACE 2
QEA03500 DS    0H
         WTOPC TEXTA=(R5),PREFIX=QEA0,NUM=(R4),LET=E
         EXITC
         EJECT
*********************************************************************
* MESSAGE TEXTS FOR THE WTOPC ROUTINES.
*********************************************************************
         SPACE 2
MSG02L   DC    AL1(L'MSG02)
MSG02    DC    C'MPIF NOT INSTALLED'
         SPACE 2
MSG03L   DC    AL1(L'MSG03)
MSG03    DC    C'INVALID FOR LOADER GENERAL FILE IPL'
         SPACE 2
MSG04L   DC    AL1(L'MSG04)
MSG04    DC    C'INTERVAL PARAMETER CANNOT BE ZERO'
         SPACE 2
MSG05L   DC    AL1(L'MSG05)
MSG05    DC    C'FORMAT IS: ZTEST MPIF STOP (ALL)'
         SPACE 2
MSG06L   DC    AL1(L'MSG06)
MSG06    DC    C'FORMAT IS: ZTEST MPIF STATus (ALL)'
         SPACE 2
MSG07L   DC    AL1(L'MSG07)
MSG07    DC    C'DEVICE PARAMETER MUST BE CTOC OR MTOM'
         SPACE 2
MSG08L   DC    AL1(L'MSG08)
MSG08    DC    C'MSGS PARAMETER CANNOT BE ZERO'
         SPACE 2
MSG09L   DC    AL1(L'MSG09)
MSG09    DC    C'SIZE PARAMETER MUST BE 381, 1055, OR 4K'
         SPACE 2
MSG10L   DC    AL1(L'MSG10)
MSG10    DC    C'USER PARAMETER CANNOT BE MPIF OR IPC'
         SPACE 2
MSG11L   DC    AL1(L'MSG11)
MSG11    DC    C'FUNCTION CANNOT EXECUTE DURING RESTART'
         SPACE 2
MSG12L   DC    AL1(L'MSG12)
MSG12    DC    C'COMMON PARAMETER MUST BE YES OR NO'
         SPACE 2
MSG13L   DC    AL1(L'MSG13)
MSG13    DC    C'FUNCTION CAN ONLY EXECUTE ON THE MAIN I-STREAM'
         EJECT
*********************************************************************
* BPKDC MACROS TO DEFINE THE LAYOUT OF A VALID INPUT MESSAGE.
*********************************************************************
         SPACE 2
MAINBPK  BPKDC (PSR,ACTION,ACTBPP)
         SPACE 2
ACTBPP   BPPSC (SETUP,5),                                              X
               (START,5,STARTBPK),                                     X
               (STOP,4,STOPBPK),                                       X
               (STATUS,4,STATBPK),                                     X
               (RESTART,1,RESTBPK),                                    X
               (SEND,3,SENDBPK),                                       X
               (FORGET,1,FORGBPK),                                     X
               (HELP,1)
         SPACE 2
STARTBPK BPKDC (PSR,ACTION,ACTBPP),                                    X
               (KD,LEVEL,1,4),                                         X
               (KD,INTERVAL,1,2)
         SPACE 2
STOPBPK  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (P,SUBSYSTM)
         SPACE 2
STATBPK  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (P,SUBSYSTM)
         SPACE 2
RESTBPK  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (KR,USER,1,8),                                          X
               (K,CLASS,1,1),                                          X
               (K,DEVICE,1,4)
         SPACE 2
SENDBPK  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (KR,USER,1,8),                                          X
               (K,SIZE,1,4),                                           X
               (KD,MSGS,1,4),                                          X
               (K,COMMON,1,3)
         SPACE 2
FORGBPK  BPKDC (PSR,ACTION,ACTBPP),                                    X
               (KR,USER,1,8)
         EJECT
*********************************************************************
* ASSIGN MORE MEANINGFUL NAMES TO THE LABEL'S GENERATED BY BPKDC.
*********************************************************************
         SPACE 2
ACTION   EQU   BPKOPM1                 ACTION   PARAM. AT LABEL BPKOPM1
USER     EQU   BPKOPM2                 USER     PARAM. AT LABEL BPKOPM2
LEVEL    EQU   BPKOPM2                 LEVEL    PARAM. AT LABEL BPKOPM2
SUBSYS   EQU   BPKOPM2                 SUBSYS   PARAM. AT LABEL BPKOPM2
INTERVAL EQU   BPKOPM3                 INTERVAL PARAM. AT LABEL BPKOPM3
CLASS    EQU   BPKOPM3                 CLASS    PARAM. AT LABEL BPKOPM3
SIZE     EQU   BPKOPM3                 SIZE     PARAM. AT LABEL BPKOPM3
DEVICE   EQU   BPKOPM4                 DEVICE   PARAM. AT LABEL BPKOPM4
MSGS     EQU   BPKOPM4                 MSGS     PARAM. AT LABEL BPKOPM4
COMMON   EQU   BPKOPM5                 COMMON   PARAM. AT LABEL BPKOPM4
         EJECT
*********************************************************************
* THE FOLLOWING TABLE IS USED TO CONVERT THE EBCDIC PATH CLASS TO
* BINARY.
*********************************************************************
         SPACE 2
CLSTRNS  DC    X'FF000102030405060708FFFFFFFFFFFF'
         DC    X'FF090A0B0C0D0E0F1011FFFFFFFFFFFF'
         DC    X'FFFF1213141516171819FFFFFFFFFFFF'
         DC    X'1A1B1C1D1E1F20212223'
         EJECT
         LTORG
         FINIS QEA0
         END   ,
