         PRINT NOGEN
         BEGIN NAME=QEA1,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEA1                                       *
* RELATED MODULES. QEA0, QEA2-QEA6, QEAA-QEAF                 *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF CONTINUOUS ACTIVITY GENERATION        *
* LEVEL........... VERSION 2  MODIFICATION 3                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS RESPONSIBLE FOR ISSUING THE     *
*             COMMANDS NECESSARY TO GENERATE CONTINUOUS MPIF  *
*             ACTIVITY.  WHEN A "ZTEST MPIF START" IS ENTERED *
*             THE DRIVER WILL ISSUE 'RESTART' REQUESTS AND    *
*             WAIT FOR CONNECTIONS TO COMPLETE.  'SEND'       *
*             PROCESSING THEN BEGINS AND CONTINUES UNTIL A    *
*             "ZTEST MPIF STOP" IS ENTERED.  WHEN THIS        *
*             OCCURS, "FORGET" REQUESTS ARE ISSUED TO STOP    *
*             THE DRIVER.                                     *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, CORE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBW000 - QEA1 ENTRY TYPE INDICATOR    *
*                       EBX040 - REGISTER SAVE AREA           *
*                       EBX050 - WORK AREA                    *
*                                                             *
*                       THE EBW000 WORK AREA WILL BE FILLED   *
*                       WITH APPROPRIATE VALUES WHEN THE      *
*                       'RESTART', 'SEND', AND 'FORGET'       *
*                       SEGMENTS ARE CALLED (QEA2, QEA3,      *
*                       QEA4).  SEE THE PROLOGUES OF THESE    *
*                       SEGMENTS FOR THE EXPECTED EBW000      *
*                       LAYOUT.                               *
*                                                             *
* REGISTER-CONVENTIONS..R1 - SD0RV BASE REGISTER              *
*                       R2 - DCTMPR BASE REGISTER             *
*                       R4 - GLOBAL 1 BASE REGISTER           *
*                       R5 - LINKAGE REGISTER                 *
*                       R7 - DCTMUS BASE REGISTER             *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... SEGMENTS ENTERING QEA1 SHOULD SET UP EBW000 *
*                 TO REQUEST THE APPROPRIATE SERVICE.         *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - "ZTEST MPIF START|STOP" COMMANDS CAN ONLY *
*                   BE ENTERED ON THE MAIN I-STREAM (THIS     *
*                   ALSO IMPLIES LOCKING IS NOT REQUIRED      *
*                   WHILE PROCESSING GLOBAL FIELDS).          *
*                 - ONE CONTINUOUS ACTIVITY ECB SHOULD BE     *
*                   ALLOWED IN EACH SUBSYSTEM-USER.           *
*                 - MPIF CONTINUOUS ACTIVITY IS CONSIDERED    *
*                   A 'LETHAL' UTILITY.  TPF CANNOT CYCLE     *
*                   SYSTEM STATES WITH THIS ACTIVITY RUNNING. *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * EBW000 = REQUESTED   * NONE                 *
*               * SERVICE (SEE BELOW)  *                      *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * NONE                 * NONE                 *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  @ISSDRV (DRIVER GLOBALS)                 *
*                        - @MPRADD  : DRIVER PRIMARY RECORD   *
*                                     ADDRESS                 *
*                        - @SYSLEV  : LEVEL OF SYSTEM TEST    *
*                                     ACTIVITY                *
*                        - @SYSINT  : FREQUENCY OF SYSTEM     *
*                                     TEST ACTIVITY           *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. EXITC                                         *
*     -ERROR... SERRC EEA006 - INVALID ENTRY TO QEA1          *
*                                                             *
***************************************************************
* ENTRY TO QEA1 CAN HAPPEN SIX DIFFERENT WAYS:                *
*                                                             *
*    1.) ENTRY FROM QEA0 DUE TO A "ZTEST MPIF START" MESSAGE. *
*        EBW000 WILL CONTAIN X'00'.  THE DRIVER WILL ISSUE    *
*        'RESTART' REQUESTS, AND THEN CRETC TO ITSELF TO GIVE *
*        CONNECTIONS A CHANCE TO ESTABLISH.                   *
*                                                             *
*    2.) QEA1 ISSUED A CRETC TO ITSELF TO FINISH STARTING THE *
*        DRIVER.  EBW000 WILL CONTAIN X'01'.  ALL CONNECTIONS *
*        ARE ASSUMED TO BE COMPLETE.  THE DRIVER WILL NOW     *
*        ISSUE THE FIRST SET OF SEND'S AND CRETC TO ITSELF    *
*        TO ISSUE MORE SEND'S.                                *
*                                                             *
*    3.) QEA1 ISSUED A CRETC TO ITSELF TO SEND THE NEXT GROUP *
*        OF MESSAGES.  EBW000 WILL CONTAIN X'02'. THE DRIVER  *
*        KEYPOINT WILL CONTAIN AN ADDRESS WITHIN THE PRIMARY  *
*        BLOCK THAT POINTS TO THE USER THAT SENT THE LAST     *
*        GROUP OF MESSAGES.                                   *
*                                                             *
*    4.) ENTRY FROM QEA0 DUE TO "ZTEST MPIF STOP" MESSAGE.    *
*        EBW000 WILL CONTAIN X'03'.  THE DRIVER WILL SET THE  *
*        'STOPPING' FLAG AND CRETC TO ITSELF TO GIVE THE ECB  *
*        ISSUING THE SEND'S A CHANCE TO COMPLETE.  QEA0 WILL  *
*        ENTER QEA1 MULTIPLE TIMES IF "ZTEST MPIF STOP ALL"   *
*        WAS ENTERED.                                         *
*                                                             *
*    5.) QEA1 ISSUED A CRETC TO ITSELF TO CONTINUE THE DRIVER *
*        STOP PROCESSING.  EBW000 WILL CONTAIN X'04'.  ECB'S  *
*        WHICH WERE ISSUING SEND'S SHOULD NOW BE COMPLETE.    *
*        IF NOT, WE WILL CRETC AGAIN TO THIS ROUTINE, AND     *
*        CONTINUE TO DO SO UNTIL THERE ARE NO OTHER ECB'S.    *
*        THE DRIVER WILL THEN ISSUE 'FORGET' REQUESTS AND     *
*        CRETC TO THE NEXT ROUTINE TO GIVE THE 'FORGET'       *
*        REQUESTS TIME TO COMPLETE.                           *
*                                                             *
*    6.) QEA1 ISSUED A CRETC TO ITSELF TO FINISH THE DRIVER   *
*        STOP PROCESSING.  EBW000 WILL CONTAIN X'05'.  THE    *
*        'FORGET' REQUESTS ARE ASSUMED TO BE COMPLETE, AND    *
*        THE DRIVER CAN STOP.                                 *
*                                                             *
***************************************************************
         EJECT
***************************************************************
*                                                             *
*    **WARNING**          **WARNING**          **WARNING**    *
*                                                             *
*  THERE IS AN UNDERLYING ASSUMPTION IN THIS SEGMENT WHICH    *
*  MUST BE UNDERSTOOD BEFORE MODIFYING ANY CODE IN IT.        *
*                                                             *
*  IN THE NORMAL COURSE OF A SYSTEM TEST RUN, THERE WILL BE   *
*  ONE ECB WHICH WILL CONTINUALLY CRETC TO ITSELF.  THE CRETC *
*  TIME IS DETERMINED BY THE "INTERVAL" PARAMETER OF THE      *
*  "ZTEST MPIF START" MESSAGE.  EACH TIME THIS ECB IS         *
*  ACTIVATED IT WILL CREATE 6 OTHER ECB'S TO SEND A GROUP OF  *
*  MESSAGES (ONE EACH FOR 381 COMMON, 1055 COMMON, 4K COMMON, *
*  381 'ORDINARY', 1055 'ORDINARY', AND 4K 'ORDINARY').       *
*  THESE ECB'S ARE CREATED BY A CREMC TO QEA3, AND WILL       *
*  TERMINATE VIA EXITC WHEN THE MESSAGES HAVE BEEN SENT.      *
*                                                             *
*  THE ALGORITHMS IN THIS SEGMENT ASSUME THE ECB'S EXECUTING  *
*  IN QEA3 WILL COMPLETE BEFORE THE NEXT QEA1 ACTIVATION.     *
*  THIS IS PARTICULARLY TRUE IN "ZTEST MPIF STOP" PROCESSING, *
*  WHICH WILL STOP THE "QEA1" ECB AT A CRETC ACTIVATION.  AT  *
*  THIS TIME, SINCE THE CRETC HAS BEEN STOPPED, AND IT IS     *
*  ASSUMED THE "QEA3" ECB'S HAVE EXITED, THE DRIVER CAN       *
*  PROCEED TO ISSUE "FORGET" REQUESTS.  IF THE "QEA3" ECB'S   *
*  HAVE NOT COMPLETED, THE "FORGET" REQUESTS WILL NOT         *
*  COMPLETE PROPERLY.                                         *
*                                                             *
*  THE MPIF DRIVER USER'S GUIDE HAS INFORMATION ON THE        *
*  IMPLICATIONS OF THIS FROM THE USER'S POINT OF VIEW.  THIS  *
*  INFORMATION CAN BE FOUND IN THE SECTION ON THE 'ZTEST MPIF *
*  START' MESSAGE.                                            *
*                                                             *
*    **WARNING**          **WARNING**          **WARNING**    *
*                                                             *
***************************************************************
         EJECT
         DCTMPR REG=R2                 DRIVER PRIMARY RECORD DSECT
         DCTMUS REG=R7                 DRIVER SECONDARY RECORD'S DSECT
         SPACE  2
         SD0RV  REG=R1                 DRIVER INFORMATION AREA DSECT
         GLOBZ  REGR=R4                GET GLOBAL 1 ADDRESS
         L      R1,@ISSDRV             SET @ISSDRV BASE REGISTER
         SPACE  2
         MVC   EBROUT(3),@EBROUT       INDICATE WHERE ROUTE OUTPUT MSGS
         MVC   CE1CPD(1),@CE1CPD       CPU ID NEEDED FOR ROUTING ALSO
         SPACE 2
         CLI   EBW000,X'00'            ENTRY DUE TO START MESSAGE?
         BE    QEA11000                YES - GO PROCESS START MESSAGE
         SPACE 2
         CLI   EBW000,X'01'            CRETC ENTRY TO FINISH STARTING?
         BE    QEA12000                YES - GO FINISH STARTING DRIVER
         SPACE 2
         CLI   EBW000,X'02'            CRETC ENTRY TO SEND MORE MSGS?
         BE    QEA13000                YES - GO SEND THOSE MESSAGES
         SPACE 2
         CLI   EBW000,X'03'            ENTRY DUE TO STOP MESSAGE?
         BE    QEA14000                YES - GO PROCESS STOP MESSAGE
         SPACE 2
         CLI   EBW000,X'04'            CRETC TO CONTINUE STOPPING?
         BE    QEA15000                YES - GO CONTINUE STOPPING
         SPACE 2
         CLI   EBW000,X'05'            CRETC ENTRY TO FINISH STOPPING?
         BE    QEA16000                YES - GO FINISH STOPPING
         SPACE 2
         LA    R0,EEA6MSG              WILL APPEND MESSAGE TO DUMP
         SERRC E,EEA006,MSG,OP,ECB     DUMP, INVALID ENTRY TO QEA1
EEA6MSG  DS    0D
         DC    AL1(ENDEEA6-BEGEEA6)
BEGEEA6  DC    C'INVALID ENTRY TO THE CONTINUOUS ACTIVITY SEGMENT QEA1'
ENDEEA6  EQU   *
         EJECT
***************************************************************
* ENTRY WAS DUE TO A START MESSAGE.
*
* - IF THE DRIVER IS STOPPING, ISSUE A MESSAGE SAYING A START
*   IS NOT ALLOWED, AND DO NOT CONTINUE PROCESSING.
*
* - IF THE DRIVER IS ALREADY STARTING OR STARTED, IT MEANS
*   ANOTHER 'ZTEST MPIF START' REQUEST WAS ENTERED.  THE ONLY
*   EFFECT OF SUCH A MESSAGE SHOULD BE TO CHANGE THE 'LEVEL'
*   AND 'INTERVAL' VALUES, WHICH IS DONE BY QEA0.  QEA1 WILL
*   ONLY PRINT A MESSAGE SAYING THE DRIVER IS 'STARTING' OR
*   'STARTED' IN ORDER TO DISPLAY THE NEW VALUES.  THE ECB
*   CREATED BY THE ORIGINAL 'ZTEST MPIF START' REQUEST IS STILL
*   RUNNING, AND WILL GET THE NEW VALUES FROM THE GLOBAL
*   FIELDS.
*
* - IF THE DRIVER IS STOPPED, CREATE NEW ECB'S TO RESTART THE
*   SYSTEM TEST USER'S.  AFTER THE ECB'S HAVE BEEN CREATED,
*   QEA1 WILL CRETC TO ITSELF TO GIVE THE USER'S TIME TO
*   ESTABLISH CONNECTIONS.  THE NAME OF EACH USER WILL BE
*   SUFFIXED BY A SUBSYSTEM-USER UNIQUE IDENTIFIER.  THIS
*   ALLOWS THE DRIVER TO RUN IN MULTIPLE SUBSYSTEM-USERS.
*
*   NOTE: IF THE CRETC TIME PLACED INTO R14 IS CHANGED IN THIS
*   ROUTINE, THEN THE MINIMUM CRETC TIME ENFORCED IN ROUTINE
*   QEA14100 SHOULD ALSO BE CHANGED.
***************************************************************
         SPACE 2
QEA11000 DS    0H
         TM    @SYSTEST,STOPPING       IS THE DRIVER STOPPING?
         BZ    QEA11100                NO  - GO CONTINUE PROCESSING
         WTOPC TEXT='MPIF DRIVER IS STOPPING, START REJECTED',         X
               PREFIX=QEA1,NUM=03,LET=E
         EXITC                         DO NOT CONTINUE TO START DRIVER
         SPACE 2
QEA11100 DS    0H
         TM    @SYSTEST,STARTING       IS THE DRIVER STARTING?
         BZ    QEA11200                NO  - GO CONTINUE PROCESSING
         BAS   R5,QEA18000             GO PRINT 'STARTING' MESSAGE
         EXITC                         DO NOT CONTINUE TO START DRIVER
         SPACE 2
QEA11200 DS    0H
         TM    @SYSTEST,STARTED        IS THE DRIVER STARTED?
         BZ    QEA11300                NO  - GO CONTINUE PROCESSING
         BAS   R5,QEA18100             GO PRINT 'STARTED' MESSAGE
         EXITC                         DO NOT CONTINUE TO START DRIVER
         SPACE 2
QEA11300 DS    0H
         MVI   @SYSTEST,STARTING+SYSRUN    INDICATE SYSTEM TEST RUN
         BAS   R5,QEA18000                 GO PRINT 'STARTING' MESSAGE
         SPACE 2
         GLOBZ REGR=R4                 GET GLOBAL 1 ADDRESS
         GLMOD GLOBAL1                 OBTAIN GLOBAL1 PROTECTION KEY
         OI    @SWITCH+1,X'10'         MAKE DRIVER LETHAL (CAN'T ---
         KEYRC                         --- CYCLE WITH DRIVER RUNNING)
         SPACE 2
         SR    R14,R14                 CLEAR A REGISTER
         IC    R14,CE1SSU+1            LOAD PART OF SUBSYSTEM-USER ID
         LA    R14,1(,R14)             INCREMENT THE SUBSYSTEM-USER ID
         O     R14,=X'000000F0'        CREATE AN EBCDIC SSU ID
         STC   R14,EBW007              PLACE SSU ID INTO USER'S NAME
         SPACE 2
         MVC   EBW000(7),=C'UMTOM1A'   INDICATE USER'S NAME
         MVI   EBW008,C'A'             INDICATE USER'S PATH CLASS
         MVI   EBW009,X'00'            INDICATE PATH CLASS IN BINARY
         MVI   EBW010,DEVMTOM          INDICATE USER'S DEVICE TYPE
         LA    R14,11                  NUMBER OF PARAMETERS TO PASS
         LA    R15,EBW000              POINT TO PARAMETER LIST
         CREMC QEA2                    CREATE ECB TO RESTART USER
         SPACE 2
         MVC   EBW000(7),=C'UMTOM1B'   INDICATE USER'S NAME
         MVI   EBW008,C'B'             INDICATE USER'S PATH CLASS
         MVI   EBW009,X'01'            INDICATE PATH CLASS IN BINARY
         LA    R14,11                  NUMBER OF PARAMETERS TO PASS
         LA    R15,EBW000              POINT TO PARAMETER LIST
         CREMC QEA2                    CREATE ECB TO RESTART USER
         SPACE 2
         MVC   EBW000(7),=C'UMTOM2B'   INDICATE USER'S NAME
         LA    R14,11                  NUMBER OF PARAMETERS TO PASS
         LA    R15,EBW000              POINT TO PARAMETER LIST
         CREMC QEA2                    CREATE ECB TO RESTART USER
         SPACE 2
         MVC   EBW000(7),=C'UCTOC1A'   INDICATE USER'S NAME
         MVI   EBW008,C'A'             INDICATE USER'S PATH CLASS
         MVI   EBW009,X'00'            INDICATE PATH CLASS IN BINARY
         MVI   EBW010,DEVCTOC          INDICATE USER'S DEVICE TYPE
         LA    R14,11                  NUMBER OF PARAMETERS TO PASS
         LA    R15,EBW000              POINT TO PARAMETER LIST
         CREMC QEA2                    CREATE ECB TO RESTART USER
         SPACE 2
         MVC   EBW000(7),=C'UCTOC1B'   INDICATE USER'S NAME
         MVI   EBW008,C'B'             INDICATE USER'S PATH CLASS
         MVI   EBW009,X'01'            INDICATE PATH CLASS IN BINARY
         LA    R14,11                  NUMBER OF PARAMETERS TO PASS
         LA    R15,EBW000              POINT TO PARAMETER LIST
         CREMC QEA2                    CREATE ECB TO RESTART USER
         SPACE 2
         MVC   EBW000(7),=C'UCTOC2B'   INDICATE USER'S NAME
         LA    R14,11                  NUMBER OF PARAMETERS TO PASS
         LA    R15,EBW000              POINT TO PARAMETER LIST
         CREMC QEA2                    CREATE ECB TO RESTART USER
         SPACE 2
         L     R15,=X'01000000'        QEA1 ENTRY TO FINISH STARTING
         LA    R14,15                  SETUP THE CRETC INTERVAL
         CRETC S,QEA1,STATE=1052       WAIT FOR CONNECTIONS ESTABLISH
         EXITC                         THIS ECB CAN EXIT
         EJECT
***************************************************************
* ENTRY WAS DUE TO QEA1 ISSUING A CRETC TO ITSELF TO COMPLETE
* THE DRIVER START-UP PROCESSING.  INDICATE THE DRIVER IS NOW
* STARTED AND SET UP TO SEND THE FIRST GROUP OF MESSAGES.
*
* IF A STOP MESSAGE WAS ENTERED WHILE THE DRIVER WAS STARTING,
* THE 'STOPPING' FLAG WILL BE SET.  IN THIS CASE THIS ECB CAN
* EXIT, SINCE THERE IS NO WORK TO DO.
***************************************************************
         SPACE 2
QEA12000 DS    0H
         TM    @SYSTEST,STOPPING       IS THE DRIVER STOPPING?
         BZ    QEA12100                NO  - GO CONTINUE PROCESSING
         EXITC                         YES - STOP ALL WORK BY THIS ECB
         SPACE 2
QEA12100 DS    0H
         NI    @SYSTEST,X'FF'-STARTING INDICATE DRIVER DONE STARTING
         OI    @SYSTEST,STARTED        INDICATE DRIVER IS NOW STARTED
         BAS   R5,QEA18100             GO PRINT 'STARTED' MESSAGE
         L     R2,@MPRADD              SET UP FOR 'SEND' PROCESSING
         B     QEA13200                GO SEND THE FIRST GROUP MESSAGES
         EJECT
***************************************************************
* ENTRY WAS DUE TO QEA1 ISSUING A CRETC TO ITSELF TO SEND THE
* NEXT GROUP OF MESSAGES.  IF ANOTHER ECB HAS INDICATED THE
* DRIVER IS TO STOP, THIS ECB WILL EXIT.  THIS WILL STOP THE
* 'SEND' PROCESS.
*
* THE DRIVER KEYPOINT CONTAINS AN ADDRESS WITHIN THE PRIMARY
* BLOCK.  THIS IS THE ENTRY FOR THE USER WHICH SENT THE LAST
* GROUP OF MESSAGES.  THE PRIMARY BLOCK WILL BE SEARCHED FROM
* THIS POINT TO FIND THE NEXT SLOT WHICH CONTAINS A USER NAME,
* THE USER HAS CONNECTIONS TO SEND OVER, AND THE USER HAS NO
* 'INCOMPLETE' CONNECTIONS.
*
* A SUBROUTINE WILL BE USED TO MANIPULATE THE POINTER INTO THE
* PRIMARY BLOCK.  THIS IS TO INSURE 'WRAP-AROUND' CONDITIONS
* (SEARCHING PAST THE END OF THE BLOCK, AND HAVING TO RESTART
* AT THE TOP) ARE HANDLED PROPERLY.
*
* IF THE DRIVER CANNOT FIND A USER THAT HAS CONNECTIONS, THE
* DRIVER WILL BE AUTOMATICALLY STOPPED.
***************************************************************
         SPACE 2
QEA13000 DS    0H
         TM    @SYSTEST,STOPPING       IS THE DRIVER STOPPING?
         BZ    QEA13100                NO  - BRANCH AND CONTINUE
         EXITC                         YES - DON'T SEND MESSAGES
         SPACE 2
QEA13100 DS    0H
         L     R2,@LASTUSR             LAST USER TO SEND MESSAGES
         BAS   R5,QEA19000             GO GET NEXT SLOT IN MPR RECORD
         SPACE 2
QEA13200 DS    0H
         LA    R6,MPRMAX               LOAD MAXIMUM NUMBER OF USERS
QEA13300 DS    0H
         MONTC
         OC    MPRNAM(8),MPRNAM        USER NAME IN THIS MPR SLOT?
         LMONC
         BZ    QEA13400                NO  - GO LOOK AT NEXT SLOT
         L     R7,MPRMUS               DCTMUS BASE REG FOR THIS USER
         OC    MUSNCN(2),MUSNCN        IS THE USER CONNECTED?
         BZ    QEA13400                NO  - GO LOOK AT NEXT SLOT
         OC    MUSICON(2),MUSICON      ANY INCOMPLETE CONNECTIONS?
         BZ    QEA13500                NO  - CAN SEND ACROSS THIS USER
         SPACE 2
QEA13400 DS    0H
         BAS   R5,QEA19000             GO GET NEXT SLOT IN MPR RECORD
         BCT   R6,QEA13300             BRANCH IF MORE SLOTS TO EXAMINE
         WTOPC TEXT='NO CONNECTIONS FOR MPIF DRIVER TO EXECUTE',       X
               PREFIX=QEA1,NUM=04,LET=T,ROUT=PRC
         B     QEA14000                GO STOP THE DRIVER
         SPACE 2
* SEND MESSAGES FROM THE USER FOUND BY SEARCHING THE PRIMARY BLOCK.
* FOR EACH OF THE SIX BLOCK SIZE/TYPES, AN EQUAL NUMBER OF MESSAGES
* (3 * @SYSLEV) WILL BE SENT, WITH ONE ECB BEING CREATED FOR EACH.
*
* AFTER THE MESSAGES ARE SENT, QEA1 WILL CRETC TO ITSELF TO SEND THE
* NEXT GROUP OF MESSAGES.  THE CRETC INTERVAL IS DETERMINED BY @SYSINT.
*
* IF @SYSLEV IS ZERO, THE OPERATOR DOES NOT WANT ANY MESSAGES SENT.
* SIMPLY CRETC TO THE NEXT INTERVAL, AND DO NOT CREATE ANY SEND ECB'S.
QEA13500 DS    0H
         CLC   @SYSLEV(2),=X'0000'     IS @SYSLEV = 0?
         BE    QEA13600                YES - DO NOT SEND ANY MESSAGES
         SPACE 2
         LA    R6,3                    WILL SEND 3*@SYSLEVEL MESSAGES
         MH    R6,@SYSLEV
         MVC   EBW000(8),MPRNAM        SET UP USER NAME IN ECB
         STH   R6,EBW010               SET UP NUMBER OF MESSAGES IN ECB
         XC    EBW020(4),EBW020        QEA3 ENTRY NOT BY A QEA3 CREMC
         SPACE 2
         MVI   EBW008,X'00'            SEND 381 NON-SHAREABLE BLOCKS
         LA    R14,30                  SIZE OF CREMC PARAMETER LIST
         LA    R15,EBW000              ADDRESS OF PARAMETER LIST
         CREMC QEA3                    CALL MESSAGE SEND PROGRAM
         SPACE 2
         MVI   EBW008,X'01'            SEND 1055 NON-SHAREABLE BLOCKS
         LA    R14,30                  SIZE OF CREMC PARAMETER LIST
         LA    R15,EBW000              ADDRESS OF PARAMETER LIST
         CREMC QEA3                    CALL MESSAGE SEND PROGRAM
         SPACE 2
         MVI   EBW008,X'02'            SEND 4K NON-SHAREABLE BLOCKS
         LA    R14,30                  SIZE OF CREMC PARAMETER LIST
         LA    R15,EBW000              ADDRESS OF PARAMETER LIST
         CREMC QEA3                    CALL MESSAGE SEND PROGRAM
         SPACE 2
         MVI   EBW008,X'03'            SEND 381 COMMON BLOCKS
         LA    R14,30                  SIZE OF CREMC PARAMETER LIST
         LA    R15,EBW000              ADDRESS OF PARAMETER LIST
         CREMC QEA3                    CALL MESSAGE SEND PROGRAM
         SPACE 2
         MVI   EBW008,X'04'            SEND 1055 COMMON BLOCKS
         LA    R14,30                  SIZE OF CREMC PARAMETER LIST
         LA    R15,EBW000              ADDRESS OF PARAMETER LIST
         CREMC QEA3                    CALL MESSAGE SEND PROGRAM
         SPACE 2
         MVI   EBW008,X'05'            SEND 4K COMMON BLOCKS
         LA    R14,30                  SIZE OF CREMC PARAMETER LIST
         LA    R15,EBW000              ADDRESS OF PARAMETER LIST
         CREMC QEA3                    CALL MESSAGE SEND PROGRAM
         SPACE 2
QEA13600 DS    0H
         ST    R2,@LASTUSR             SAVE LAST USER TO SEND MESSAGES
         L     R15,=X'02000000'        SET UP CRETC PARAMETERS
         LH    R14,@SYSINT             WILL CRETC IN THIS MANY SECONDS
         CRETC S,QEA1,STATE=1052       ISSUE THE CRETC
         EXITC                         THIS ECB IS FINISHED
         EJECT
***************************************************************
* ENTRY WAS DUE TO A "ZTEST MPIF STOP" MESSAGE.  TURN ON THE
* "STOPPING" INDICATOR AND THEN CRETC TO GIVE OTHER SYSTEM TEST
* ECB'S A CHANCE TO STOP.  THE OTHER ECB'S WILL EITHER BE
* INVOLVED IN STARTING THE DRIVER, OR IN SENDING MESSAGES.  IN
* EITHER CASE, THESE ECB'S ISSUE CRETC MACROS.  AT THEIR NEXT
* CRETC ACTIVATION THEY WILL 'SEE' THE 'STOPPING' FLAG AND
* TERMINATE.
*
* THE CRETC INTERVAL WILL BE EQUAL TO @SYSINT PLUS 1 SECOND,
* WHERE @SYSINT CONTAINS THE SYSTEM TEST INTERVAL.  THIS
* ENSURES ECB'S INVOLVED IN SENDING MESSAGES WILL BE ACTIVATED
* SOMETIME DURING THIS ECB'S CRETC, AND WILL TERMINATE BECAUSE
* OF THE 'STOPPING' FLAG.
*
* WE MUST ALSO GUARANTEE IF AN ECB IS INVOLVED IN STARTING THE
* DRIVER, THAT IT WILL ALSO 'SEE' THE STOP FLAG.  SINCE THIS
* OTHER ECB MAY NOT BE ACTIVATED IN @SYSINT+1 SECONDS, WE WILL
* ENFORCE A MINIMUM VALUE ON THE CRETC INTERVAL OF THIS ECB.
* THIS MINIMUM VALUE WILL BE GREATER THAN THE TIME IT TAKES
* THE DRIVER TO MOVE FROM 'STARTING' TO 'STARTED'.
*
* IF THE DRIVER IS 'STOPPING' OR 'STOPPED', AN ERROR MESSAGE
* WILL BE PRINTED AND NO WILL BE ACTION TAKEN.
***************************************************************
         SPACE 2
QEA14000 DS    0H
         TM    @SYSTEST,STARTING+STARTED  DRIVER STARTING OR STARTED?
         BNZ   QEA14100                   YES - CAN GO STOP THE DRIVER
         WTOPC TEXT='MPIF DRIVER IS ALREADY STOPPING OR STOPPED',      X
               PREFIX=QEA1,NUM=05,LET=E
         EXITC                            DO NOT PROCESS STOP REQUEST
         SPACE 2
QEA14100 DS    0H
         OI    @SYSTEST,STOPPING       INDICATE DRIVER STOPPING
         NI    @SYSTEST,X'FF'-STARTING DRIVER NO LONGER STARTING
         NI    @SYSTEST,X'FF'-STARTED  DRIVER NO LONGER STARTED
         SPACE 2
         WTOPC TEXT='MPIF DRIVER STOPPING',                            X
               PREFIX=QEA1,NUM=06,LET=I,ROUT=PRC
         SPACE 2
         LH    R14,@SYSINT             CRETC IN THIS MANY SECONDS ---
         LA    R14,1(,R14)             --- PLUS ONE
         C     R14,=F'16'              IS CRETC MORE THAN THE MINIMUM?
         BH    QEA14200                YES - CAN GO ISSUE THE CRETC
         LA    R14,16                  NO  - ENFORCE MINIMUM CRETC TIME
         SPACE 2
QEA14200 DS    0H
         L     R15,=X'04000000'        SET UP CRETC PARAMETERS
         CRETC S,QEA1,STATE=1052       CRETC TO CONTINUE STOPPING
         EXITC                         THIS ECB IS FINISHED, CAN EXIT
         EJECT
***************************************************************
* ENTRY WAS DUE TO QEA1 ISSUING A CRETC TO ITSELF TO CONTINUE
* STOP PROCESSING.  WE WILL CHECK TO SEE IF ANY USERS STILL
* HAVE ACTIVE ECB'S.  IF SO, WE WILL ISSUE ANOTHER CRETC, AND
* RETURN BACK TO THIS ROUTINE.  OTHERWISE, 'FORGET' REQUESTS
* WILL BE ISSUED ON THE SYSTEM TEST USER'S, AND QEA1 WILL CRETC
* TO ITSELF TO GIVE 'FORGET' PROCESSING A CHANCE TO COMPLETE.
***************************************************************
         SPACE 2
QEA15000 DS    0H
         L     R2,@MPRADD              LOAD BASE OF PRIMARY BLOCK
         LA    R6,MPRMAX               LOAD MAXIMUM NUMBER OF USERS
         SPACE 2
QEA15100 DS    0H
         MONTC
         OC    MPRNAM(8),MPRNAM        IS THERE USER IN THIS MPR SLOT?
         LMONC
         BZ    QEA15200                NO  - GO LOOK AT NEXT SLOT
         L     R7,MPRMUS               USER'S SECONDARY BLOCK ADDRESS
         L     R14,MUSECB              LOAD NUMBER OF ACTIVE ECB'S
         LTR   R14,R14                 ANY ECB'S ACTIVE?
         BZ    QEA15200                NO  - GO CHECK NEXT SLOT
         SPACE 2
         LA    R14,30                  YES - TRY AGAIN IN 30 SECONDS
         L     R15,=X'04000000'        SET UP CRETC PARAMETERS
         CRETC S,QEA1,STATE=1052       CRETC TO TRY THIS ROUTINE AGAIN
         EXITC                         THIS ECB IS FINISHED, CAN EXIT
         SPACE 2
QEA15200 DS    0H
         LA    R2,MPRLEN(,R2)          LOOK AT NEXT ENTRY IN TABLE
         BCT   R6,QEA15100             BRANCH IF MORE SLOTS TO CHECK
*                                      OTHERWISE, ISSUE THE "FORGETS"
         SPACE 2
         SR    R14,R14                 CLEAR A REGISTER
         IC    R14,CE1SSU+1            LOAD PART OF SUBSYSTEM-USER ID
         LA    R14,1(,R14)             INCREMENT THE SUBSYSTEM-USER ID
         O     R14,=X'000000F0'        CREATE AN EBCDIC SSU ID
         STC   R14,EBW007              PLACE SSU ID INTO USER'S NAME
         SPACE 2
         MVC   EBW000(7),=C'UCTOC1A'   USER'S NAME IN PARAMETER LIST
         LA    R14,8                   LENGTH OF CREMC PARAMETER
         LA    R15,EBW000              ADDRESS OF CREMC PARAMETER
         CREMC QEA4                    CALL 'FORGET' ROUTINE
         SPACE 2
         MVC   EBW000(7),=C'UCTOC1B'   USER'S NAME IN PARAMETER LIST
         LA    R14,8                   LENGTH OF CREMC PARAMETER
         LA    R15,EBW000              ADDRESS OF CREMC PARAMETER
         CREMC QEA4                    CALL 'FORGET' ROUTINE
         SPACE 2
         MVC   EBW000(7),=C'UCTOC2B'   USER'S NAME IN PARAMETER LIST
         LA    R14,8                   LENGTH OF CREMC PARAMETER
         LA    R15,EBW000              ADDRESS OF CREMC PARAMETER
         CREMC QEA4                    CALL 'FORGET' ROUTINE
         SPACE 2
         MVC   EBW000(7),=C'UMTOM1A'   USER'S NAME IN PARAMETER LIST
         LA    R14,8                   LENGTH OF CREMC PARAMETER
         LA    R15,EBW000              ADDRESS OF CREMC PARAMETER
         CREMC QEA4                    CALL 'FORGET' ROUTINE
         SPACE 2
         MVC   EBW000(7),=C'UMTOM1B'   USER'S NAME IN PARAMETER LIST
         LA    R14,8                   LENGTH OF CREMC PARAMETER
         LA    R15,EBW000              ADDRESS OF CREMC PARAMETER
         CREMC QEA4                    CALL 'FORGET' ROUTINE
         SPACE 2
         MVC   EBW000(7),=C'UMTOM2B'   USER'S NAME IN PARAMETER LIST
         LA    R14,8                   LENGTH OF CREMC PARAMETER
         LA    R15,EBW000              ADDRESS OF CREMC PARAMETER
         CREMC QEA4                    CALL 'FORGET' ROUTINE
         SPACE 2
         LA    R14,10                  WILL CRETC IN THIS MANY SECONDS
         L     R15,=X'05000000'        SET UP CRETC PARAMETER FOR QEA1
         CRETC S,QEA1,STATE=1052       CRETC TO FINISH STOP PROCESSING
         EXITC                         THIS ECB HAS FINISHED PROCESSING
         EJECT
***************************************************************
* ENTRY WAS DUE TO QEA1 ISSUING A CRETC TO ITSELF TO FINISH
* STOP PROCESSING.  ALL ECB'S ISSUING FORGET'S ARE NOW ASSUMED
* TO HAVE EXITED.  CLEAR ALL INDICATORS, ISSUE A MESSAGE, AND
* EXIT.
***************************************************************
         SPACE 2
QEA16000 DS    0H
         NI    @SYSTEST,X'FF'-SYSRUN   SYSTEM TEST RUN OVER
         NI    @SYSTEST,X'FF'-STARTING CLEAR ALL OTHER INDICATORS
         NI    @SYSTEST,X'FF'-STARTED
         NI    @SYSTEST,X'FF'-STOPPING
         SPACE 2
         GLOBZ REGR=R4                 GET GLOBAL 1 ADDRESS
         GLMOD GLOBAL1                 OBTAIN GLOBAL1 PROTECTION KEY
         NI    @SWITCH+1,X'EF'         TURN OFF LETHAL UTILITY
         KEYRC                         RESTORE PROTECTION KEY
         SPACE 2
         WTOPC TEXT='MPIF DRIVER STOPPED',                             X
               PREFIX=QEA1,NUM=07,LET=I,ROUT=PRC
         EXITC
         EJECT
***************************************************************
* THESE ROUTINES WILL PRINT THE 'MPIF DRIVER STARTING' (ENTRY
* AT LABEL QEA18000), AND 'MPIF DRIVER STARTED' (ENTRY AT LABEL
* QEA18100) MESSAGES.
*
* IF BOTH 'LEVEL' AND 'INTERVAL' ARE AT GIVEN DEFAULT VALUES,
* AN ALTERNATE MESSAGE WILL BE PRINTED.  THIS MESSAGE WILL
* SUPPRESS THE 'LEVEL' AND 'INTERVAL' VALUES.
*
* REGISTER USE IN THIS ROUTINE IS AS FOLLOWS:
*      R0   : UNUSED
*      R1   : MUST CONTAIN @ISSDRV BASE ADDRESS UPON ENTRY
*      R2-R4: USED, BUT RESTORED BEFORE RETURN
*      R5   : LINKAGE REGISTER
*      R6-R7: USED, AND NOT RESTORED BEFORE RETURN
***************************************************************
         SPACE 2
QEA18000 DS    0H
         LA    R6,1                    LOAD THE MESSAGE NUMBER
         LA    R7,=CL8'STARTING'       MESSAGE IS 'DRIVER STARTING'
         B     QEA18200                GO BRANCH TO COMMON ROUTINE
         SPACE 2
QEA18100 DS    0H
         LA    R6,2                    LOAD THE MESSAGE NUMBER
         LA    R7,=CL8'STARTED'        MESSAGE IS 'DRIVER STARTED'
         SPACE 2
QEA18200 DS    0H
         STM   R2,R4,EBX040            SAVE REGISTERS ACROSS ROUTINE
         LA    R2,MSG1VER1             MESSAGE TEXT ADDRESS FOR WTOPC
         LH    R3,@SYSINT              SYSTEM TEST INTERVAL FOR WTOPC
         LH    R4,@SYSLEV              SYSTEM TEST LEVEL FOR WTOPC
         SPACE 2
         LA    R15,LEVDEF              LOAD THE DEFAULT LEVEL VALUE
         CH    R15,@SYSLEV             DIFFERENT FROM ACTUAL VALUE?
         BNE   QEA18300                YES - CAN GO PRINT THE MESSAGE
         SPACE 2
         LA    R15,INTDEF              LOAD THE DEFAULT INTERVAL VALUE
         CH    R15,@SYSINT             DIFFERENT FROM ACTUAL VALUE?
         BNE   QEA18300                YES - CAN GO PRINT THE MESSAGE
         LA    R2,MSG1VER2             NO  - POINT TO ALTERNATE MESSAGE
         SPACE 2
QEA18300 DS    0H
         WTOPC TEXTA=(R2),PREFIX=QEA1,NUM=(R6),LET=I,ROUT=PRC,         X
               SUB=(CHARA,(R7),DEC,(R4),DEC,(R3)),COMP=YES
         LM    R2,R4,EBX040            RESTORE REGISTERS BEFORE RETURN
         BR    R5                      RETURN TO CALLER
         SPACE 2
MSG1VER1 DC    AL1(L'MSG1TXT1)
MSG1TXT1 DC    C'MPIF DRIVER ........, LEVEL = ...., INTERVAL = ....'
         SPACE 2
MSG1VER2 DC    AL1(L'MSG1TXT2)
MSG1TXT2 DC    C'MPIF DRIVER ........'
         EJECT
***************************************************************
* THIS ROUTINE IS RESPONSIBLE FOR MANIPULATING THE PRIMARY
* BLOCK BASE REGISTER, R2.  THE REGISTER WILL BE CHANGED SO IT
* IS POINTING TO THE NEXT SLOT IN THE RECORD.  THIS INCLUDES
* ACCOUNTING FOR 'WRAP-AROUND', IF CURRENTLY POINTING TO THE
* LAST ENTRY IN THE TABLE.
*
* THE MAXIMUM POSSIBLE VALUE FOR R2 WILL BE CALCULATED FIRST.
* THIS WILL BE BASED ON THE STARTING ADDRESS OF THE BLOCK, THE
* MAXIMUM NUMBER OF ENTRIES, AND THE SIZE OF EACH ENTRY.  R2
* WILL THEN BE INCREMENTED THE LENGTH OF ONE ENTRY, AND IF IT
* EXCEEDS THE MAXIMUM, 'WRAP-AROUND' HAS OCCURRED.  R2 WILL
* THEN BE FORCED TO THE TOP OF THE PRIMARY BLOCK.
***************************************************************
         SPACE 2
QEA19000 DS    0H
         LA    R15,MPRLEN              GET THE LENGTH OF ONE ENTRY
         STH   R15,EBX050              SAVE LENGTH OF ENTRY
         LA    R14,MPRMAX              GET MAXIMUM NUMBER OF ENTRIES
         MH    R14,EBX050              LENGTH ENTRY * NUMBER ENTRIES
         SPACE 2
         L     R15,@MPRADD             LOAD STARTING ADDRESS OF MPR
         AR    R14,R15                 CREATE MAXIMUM VALUE FOR R2
         SPACE 2
         LA    R2,MPRLEN(,R2)          INCREMENT R2 LENGTH ONE ENTRY
         CLR   R2,R14                  COMPARE R2 TO MAXIMUM VALUE
         BLR   R5                      R2 LESS THAN MAX, CAN RETURN
         SPACE 2
         L     R2,@MPRADD              WRAP R2 BACK TO TOP OF BLOCK
         BR    R5                      RETURN TO CALLER
         EJECT
         LTORG
         FINIS QEA1
         END   ,
