         PRINT NOGEN
         BEGIN NAME=QEA2,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEA2                                       *
* RELATED MODULES. QEA0-QEA1, QEA3-QEA6, QEAA-QEAF            *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... "ZTEST MPIF RESTART" PROCESSING            *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS RESPONSIBLE FOR PROCESSING      *
*             "ZTEST MPIF RESTART" REQUESTS.  IT WILL ISSUE   *
*             A MPIFC IDENTIFY FOR THE USER, AND A MPIFC      *
*             CONNECT TO ANY OTHER USER IN THE SYSTEM WITH    *
*             THE SAME NAME.                                  *
*                                                             *
*             QEA2 CAN BE CALLED BY QEA0 TO PROCESS A "ZTEST  *
*             MPIF RESTART" REQUEST, OR IT CAN BE CALLED BY   *
*             QEA1 TO RESTART USER'S FOR A "ZTEST MPIF START" *
*             REQUEST.                                        *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBX000 - MPIFC PARAMETER LISTS        *
*                       EBW000 - USER'S NAME                  *
*                       EBW008 - USER'S PATH CLASS (EBCDIC)   *
*                       EBW009 - USER'S PATH CLASS (BINARY)   *
*                       EBW010 - USER'S DEVICE TYPE           *
*                       EBW020 - SYSTEM WANT TO CONNECT TO    *
*                       EBW032 - REGISTER SAVE AREA           *
*                       CE1CR2 - DRIVER'S PRIMARY BLOCK       *
*                       CE1CR3 - DRIVER'S SECONDARY BLOCK     *
*                       CE1CR4 - QUERY REPLY AREA             *
*                       CE1CR6 - QUERY REPLY AREA             *
*                                                             *
* REGISTER-CONVENTIONS..R1  - SD0RV  BASE REGISTER            *
*                       R1  - DCTMGT BASE REGISTER            *
*                       R2  - DCTMPR BASE REGISTER            *
*                       R3  - NUMBER OF USER'S IN SYSTEM WITH *
*                             THE SAME NAME AS THIS USER      *
*                       R4  - DCTMUS BASE REGISTER            *
*                       R5  - DCTMUP BASE REGISTER            *
*                       R6  - DCTMUS BASE REGISTER, SUFFIX=A  *
*                       R7  - NUMBER OF AVAILABLE PATHS WITH  *
*                             THE CORRECT CLASS TO A GIVEN    *
*                             SYSTEM.                         *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... QEA0 MUST HAVE PARSED AND ERROR CHECKED THE *
*                 COMMAND.  IT MUST ALSO HAVE SET UP THE      *
*                 EBW000 WORK AREA WITH INFORMATION ABOUT THE *
*                 USER (SEE BELOW)                            *
*                                                             *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - "RESTART" COMMANDS CAN BE ENTERED ON ANY  *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * EBW000 - USER'S NAME * NONE                 *
*               * EBW008 - PATH CLASS  *                      *
*               *          (EBCDIC)    *                      *
*               * EBW009 - PATH CLASS  *                      *
*               *          (BINARY)    *                      *
*               * EBW010 - DEVICE TYPE *                      *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * NONE                 * NONE                 *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  MGT - MPIF GLOBAL TABLE                  *
*                    @ISSDRV (DRIVER GLOBALS)                 *
*                        - @MPRADD  : DRIVER PRIMARY RECORD   *
*                                     ADDRESS                 *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. EXITC                                         *
*     -ERROR... EXITC        (BAD MPIFC RETURN CODE)          *
*               SERRC EEA001 (MPIFC QUERY DID NOT FIND ANY    *
*                             OTHER USERS)                    *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* ESTABLISH DSECTS, LOAD BASE REGISTERS, AND GET INFORMATION
* ABOUT ROUTING AND THE DRIVER'S STATE FROM THE @ISSDRV GLOBAL.
***************************************************************
         SPACE  2
         DCTMUP REG=R5,LTYPE=ALL       MPIFC PARAMETER DSECTS
         SPACE  2
         DCTPFX REG=R0                 PREFIX PAGE FOR STPOC USE
         DCTMPR REG=R2                 MPIF DRIVER PRIMARY BLOCK DSECT
         DCTMUS REG=R4                 MPIF DRIVER SECONDARY BLK DSECT
         DCTMUS REG=R6,SUFFIX=A        DSECT THE CONNECTION SLOTS ---
*                                      --- WITHIN THE SECONDARY BLOCK
         SPACE  2
         SD0RV  REG=R1                 DSECT DRIVER INFORMATION AREA
         GLOBZ  REGR=R14               GET GLOBAL AREA ADDRESS
         L      R1,@ISSDRV             LOAD @ISSDRV BASE REGISTER
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE O/P MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         EJECT
***************************************************************
* SEE IF A PRIMARY BLOCK HAS BEEN CREATED YET.  IF NOT, A BLOCK
* NEEDS TO BE CREATED, INITIALIZED, AND IT'S ADDRESS STORED IN
* THE GLOBAL @ISSDRV.  COMPARE AND SWAP LOGIC IS USED TO UPDATE
* THE @MPRADD FIELD TO INSURE ANOTHER ECB DOESN'T CREATE A
* SECOND PRIMARY BLOCK ON TOP OF THIS ONE.  THE ADDRESS OF THE
* MPR IS USED TO LOCK ON THE MPR ITSELF WHILE IT IS BEING
* UPDATED.
*
* A COMMON BLOCK IS USED SINCE THE PRIMARY BLOCK WILL BE
* DETACHED FROM THIS ECB, AND ACCESSED BY OTHER ECB'S.
***************************************************************
         SPACE 2
         L     R2,@MPRADD              LOAD PRIMARY BLOCK ADDRESS
         OC    @MPRADD,@MPRADD         PRIMARY BLOCK BEEN CREATED YET?
         BZ    QEA21000                NO  - GO CREATE A BLOCK
         CORHC (R2)                    YES - LOCK ON MPR TO UPDATE IT
         B     QEA21200                GO CONTINUE RESTART PROCESSING
         SPACE 2
QEA21000 DS    0H
* REPLACE GETCC WITH GSYSC 2 FRAMES OF 4K EACH FROM SYSTEMdddd
*        GETCC D2,L4,COMMON=YES,FILL=00     GET THE PRIMARY BLOCK
*
         LA    R3,@MPRADD
         LA    R2,2
         GSYSC FRAMES=R2,TYPE=4KB,TOKEN=R3
         LA    R3,2048                 CLEAR THE SYSTEM HEAP
         SLA   R3,2(0)                 MULTIPLY BY 2 FOR THE 8K CLEAR
         LGR   R7,R3                   SAVE POINTER TO HEAP
         SR    R15,R15                 CLEAR OUT R15 FOR INIT
         MONTC
         MVCL  R2,R14
         STPOC OVERRIDE=ON,SAVECR0=EBX008
         LMONC
*
         SR    R3,R3                   CLEAR R3 FOR COMPARE AND SWAP
         SR    R2,R7                   RESET HEAP POINTER
         CS    R3,R2,@MPRADD           SAVE PRIMARY BLOCK ADDRESS
         BNZ   QEA21100                BRANCH IF BLOCK ALREADY ATTACHED
         SPACE 2
         CORHC (R2)                    LOCK ON MPR TO UPDATE IT
         B     QEA21200                BRANCH AND CONTINUE
         SPACE 2
QEA21100 DS    0H                      COMPARE AND SWAP FAILED
         LR    R2,R3                   LOAD OTHER PRIMARY BLOCK ADDRESS
         CORHC (R2)                    LOCK ON MPR WHILE MAKING UPDATES
*        RELCC D2                      THE BLOCK WE GOT WON'T BE USED
         EJECT
***************************************************************
* THE PRIMARY BLOCK IS POINTED TO BY R2.  TRY AND FIND AN EMPTY
* SLOT IN IT FOR THE USER TO BE RESTARTED.
***************************************************************
         SPACE 2
QEA21200 DS    0H
         LA    R15,MPRMAX              LOAD MAXIMUM NUMBER OF USERS
         SPACE 2
QEA21300 DS    0H
         MONTC
         OC    MPRNAM,MPRNAM           IS THIS ENTRY IN USE?
         LMONC
         BZ    QEA22000                NO  - USE IT FOR RESTART REQUEST
         LA    R2,MPRLEN(,R2)          YES - BUMP R2 TO NEXT ENTRY
         BCT   R15,QEA21300            CHECK NEXT ENTRY IF ONE EXISTS
         SPACE 2
         WTOPC TEXT='CANNOT RESTART ........ - MAXIMUM NUMBER OF USERS X
               EXCEEDED',COMP=YES,PREFIX=QEA2,LET=E,TIME=YES,          X
               NUM=01,SUB=(CHARA,EBW000)
         L     R15,@MPRADD             LOAD PRIMARY BLOCK ADDRESS
         CORUC (R15)                   UNLOCK THE PRIMARY BLOCK
         EXITC
         EJECT
***************************************************************
* CREATE AND INITIALIZE A SECONDARY BLOCK FOR THE USER.  THE
* SECONDARY BLOCK WILL BE LOCKED FOR THE REMAINDER OF THIS
* SEGMENT.
*
* A COMMON BLOCK IS USED SINCE THE SECONDARY BLOCK WILL BE
* DETACHED FROM THIS ECB, AND ACCESSED BY OTHER ECB'S.
***************************************************************
         SPACE 2
QEA22000 DS    0H
         GETCC D3,L4,COMMON=YES,FILL=00     GET THE SECONDARY BLOCK
         L     R4,CE1CR3                    LOAD THE BLOCK ADDRESS
         CORHC (R4)                         LOCK THE SECONDARY BLOCK
         EJECT
***************************************************************
* THE USER NEEDS TO BE IDENTIFIED TO THE TPF SYSTEM USING THE
* MPIFC MACRO.  THE UITOK THE USER WILL GIVE TO MPIF WILL BE
* THE ADDRESS OF THE USER'S SECONDARY BLOCK.  IN ADDITION, THE
* CONNECTION REQUEST (CRX), CONNECTION COMPLETE (CCX) AND ERROR
* (ERX) EXITS WILL ALSO BE GIVEN TO MPIF.
***************************************************************
         SPACE 2
         LA    R5,EBX000               BASE OF MPIFC PARAMETER LIST
         XC    0(IDRPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVC   IDMYNAME,EBW000         COPY USER NAME TO PARAMETER LIST
         ST    R4,IDUITOK              SAVE UITOK IN PARAMETER LIST
         MVC   IDCRXAD,QEA2CRX         COPY CRX VECTOR TO PARAM. LIST
         MVC   IDCCXAD,QEA2CCX         COPY CCX VECTOR TO PARAM. LIST
         MVC   IDERXAD,QEA2ERX         COPY ERROR EXIT TO PARAM. LIST
         SPACE 2
         MPIFC RTYPE=IDENTIFY,REG=R5   ISSUE THE IDENTIFY REQUEST
         LTR   R15,R15                 WAS THE OPERATION SUCCESSFUL?
         BZ    QEA23000                YES - BRANCH AND CONTINUE
         SPACE 2
         LR    R6,R15                  SAVE THE MPIFC RETURN CODE
         L     R15,@MPRADD             LOAD PRIMARY BLOCK ADDRESS
         CORUC (R15)                   UNLOCK THE PRIMARY BLOCK
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         RELCC D3                      GET RID OF USER'S SECONDARY BLK
         CH    R6,=H'4'                WAS THE USER ALREADY IDENTIFIED?
         BNE   QEA29000                NO  - BRANCH, ISSUE ERROR MSG
         TM    EBCM01,SYSRUN           SYSTEM TEST RUN?
         BO    QEA22500                YES - DON'T PRINT MESSAGE
         WTOPC TEXT='........ HAS ALREADY BEEN RESTARTED',NUM=02,      X
               COMP=YES,PREFIX=QEA2,TIME=YES,SUB=(CHARA,EBW000),LET=E
QEA22500 DS    0H
         EXITC
         EJECT
***************************************************************
* THE USER HAS BEEN PROPERLY IDENTIFIED, SO THE PRIMARY AND
* SECONDARY BLOCKS MUST BE UPDATED TO CONTAIN INFORMATION ABOUT
* THE USER.
***************************************************************
         SPACE 2
QEA23000 DS    0H
         TM    EBCM01,SYSRUN           SYSTEM TEST RUN?
         BO    QEA23100                YES - DON'T PRINT MESSAGE
         WTOPC TEXT='SUCCESSFUL IDENTIFY REQUEST FOR ........',LET=I,  X
               PREFIX=QEA2,NUM=03,ROUT=PRC,SUB=(CHARA,EBW000),COMP=YES
         SPACE 2
QEA23100 DS    0H
         MVC   CE1CT3,=AL2(LIOCB)      UNHOOK SECONDARY BLOCK FROM ECB
         MONTC
         MVC   MPRNAM,EBW000           COPY USER NAME TO PRIMARY BLOCK
         ST    R4,MPRMUS               SAVE ADDRESS OF SECONDARY BLOCK
         MVC   MUSNAM,EBW000           COPY USER NAME TO SECONDARY BLK
         LMONC
         MVC   MUSITK,IDTOK            SAVE IDTOK OF THIS USER
         MVC   MUSPCL,EBW008           COPY PATH CLASS (EBCDIC)
         MVI   MUSPCL+1,C' '           BLANK FOR WTOPC'S IN QEA2, QEAB
         MVC   MUSPCB,EBW009           COPY PATH CLASS (BINARY)
         MVC   MUSDEV(1),EBW010        COPY THE DEVICE TYPE
         SPACE 2
         L     R15,@MPRADD             LOAD MPR ADDRESS
         CORUC (R15)                   RELEASE THE MPR LOCK
         EJECT
***************************************************************
* A MPIFC QUERY WILL BE ISSUED TO LIST ALL INSTANCES OF USERS
* WITH THE SAME NAME AS THE USER JUST IDENTIFIED.  THIS LIST
* WILL BE USED TO ESTABLISH CONNECTIONS WITH THOSE OTHER USERS.
*
* THE QUERY WILL BE ISSUED WITH DIRECTORY UPDATE REQUESTED.
* THIS MEANS, IF AT A LATER TIME, ANOTHER USER WITH THE SAME
* NAME ISSUES A MPIFC IDENTIFY, THE DUX FOR THIS USER WILL BE
* KICKED OFF.
***************************************************************
         SPACE 2
         GETCC D4,L4                   BLOCK FOR QUERY REPLY AREA
         XC    0(QURPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         SPACE 2
         MVI   QUOPTS1,QUP1UNL         QUERY OF USERS IN ALL SYSTEMS
         MVI   QUOPTS4,QUP4DUX         DIRECTORY UPDATE EXIT SPECIFIED
         MVC   QUIDTOK,MUSITK          PUT USER'S IDTOK IN PARAM. LIST
         MVC   QUCLASS,EBW008          PUT PATH CLASS IN PARAMETER LIST
         MVC   QURUNAM,MUSNAM          NAME OF REQUESTING USER
         MVC   QUDUNAM,MUSNAM          NAME OF DESTINATION USER
         MVI   QUDSNAM,C'*'            QUERY ALL SYSTEMS IN COMPLEX
         LH    R14,CE1CC4              LOAD SIZE OF REPLY AREA
         ST    R14,QUFBAL              PUT THE SIZE IN PARAMETER LIST
         MVC   QUFBAA,CE1CR4           REPLY AREA ADDRESS IN PARAM LIST
         MVC   QUDUXAD,QEA2DUX         DIRECTORY UPDATE (DUX) VECTOR
         SPACE 2
         MPIFC RTYPE=QUERY,REG=R5      ISSUE THE QUERY WITH DUX EXIT
         LTR   R15,R15                 WAS THE QUERY SUCCESSFUL?
         BNZ   QEA29100                NO  - BRANCH, ISSUE ERROR MSG
         EJECT
***************************************************************
* THE QUERY MUST BE REPEATED FOR THE PATH ACTIVATION
* NOTIFICATION EXIT.  THIS MEANS, IF AT A LATER TIME, A NEW
* PATH TO THE QUERIED USERS BECOMES AVAILABLE, THIS USER'S PAX
* WILL BE KICKED OFF.
***************************************************************
         SPACE 2
         MVI   QUOPTS4,QUP4PAX         INDICATE PATH ACTIVATION EXIT
         MVC   QUPAXAD,QEA2PAX         PATH ACTIVATION (PAX) VECTOR
         MPIFC RTYPE=QUERY,REG=R5      ISSUE QUERY WITH PAX INDICATED
         LTR   R15,R15                 WAS THE QUERY SUCCESSFUL?
         BNZ   QEA29100                NO  - BRANCH, ISSUE ERROR MSG
         EJECT
***************************************************************
* R3 WILL BE LOADED WITH THE NUMBER OF USERS IN THE COMPLEX
* THAT HAVE THE SAME NAME AS THIS USER.  IF THIS NUMBER IS
* ZERO, A SERRC IS ISSUED SINCE THE USER JUST IDENTIFIED SHOULD
* AT LEAST BE FOUND.
***************************************************************
         SPACE 2
         L     R5,CE1CR4               SET QUERY REPLY AREA BASE REG.
         LH    R3,QRACT                LOAD NUMBER OF USERS IN COMPLEX
         LTR   R3,R3                   WERE ANY USERS FOUND?
         BNZ   QEA24000                YES - BRANCH AND CONTINUE
         SPACE 2
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         LA    R0,EEA1MSG              WILL APPEND MESSAGE TO DUMP
         SERRC E,EEA001,MSG,OP,ECB     DUMP, NO USER'S FOUND
EEA1MSG  DS    0D
         DC    AL1(ENDEEA1-BEGEEA1)
BEGEEA1  DC    C'NO USERS FOUND ON A MPIFC QUERY REQUEST'
ENDEEA1  EQU   *
         EJECT
***************************************************************
* THE BLOCK RETURNED WITH THE LIST OF USERS WILL BE SCANNED.
* THE SYSTEM NAME OF EACH USER WILL BE COMPARED TO THE NAME OF
* THIS SYSTEM.  IF THE NAMES MATCH, AND THE RESTART MESSAGE
* SPECIFIED DEVICE = MTOM, A CONNECTION WILL BE ESTABLISHED.
* IF THE SYSTEM NAMES DON'T MATCH, DEVICE = CTOC WAS SPECIFIED,
* AND THIS SYSTEM HAS A LARGER ALPHABETICAL NAME, THEN A CTOC
* CONNECTION WILL BE MADE.  THIS SYSTEM MUST HAVE A "LARGER"
* NAME SO ONLY ONE OF THE USERS WILL ISSUE THE CONNECT REQUEST
* (THE OTHER USER SHOULD HAVE HAD IT'S DUX INVOKED WHEN THIS
* USER ISSUED THE MPIFC IDENTIFY).
***************************************************************
         SPACE 2
QEA24000 DS    0H
         DROP  R1                      SD0RV DSECT NO LONGER NEEDED
         DCTMGT REG=R1                 MPIF GLOBAL TABLE (MGT) DSECT
         CINFC R,CMMPIF,F,REG=R1,BSS=YES    LOAD MGT BASE REGISTER
         SPACE 2
QEA24100 DS    0H
         CLI   EBW010,DEVCTOC          WAS DEVICE = CTOC SPECIFIED?
         BNE   QEA24200                NO - TRY MAKE MTOM CONNECTION
         CLC   MGTMSSN(8),QRUSYS       THIS SYSTEM > OTHER SYSTEM
         BNH   QEA27000                NO  - BRANCH, DON'T CONNECT
         B     QEA24300                YES - GO MAKE CONNECTION
         SPACE 2
QEA24200 DS    0H                      TRY AND MAKE MTOM CONNECTION
         CLC   MGTMSSN(8),QRUSYS       SAME SYSTEM NAMES?
         BNE   QEA27000                NO  - DON'T MAKE CONNECTION
         SPACE 2
QEA24300 DS    0H
         MVC   EBW020(8),QRUSYS        SAVE THE OTHER SYSTEM'S NAME
         DROP  R1                      MGT DSECT NO LONGER NEEDED
         EJECT
***************************************************************
* BEFORE TRYING TO ESTABLISH THE CONNECTION A MPIFC QUERY MUST
* BE ISSUED TO FIND OUT WHAT PATHS ARE AVAILABLE TO THE OTHER
* SYSTEM.  THIS LIST WILL BE RETURNED ON DATA LEVEL 6, AND WILL
* BE SEARCHED TO SEE WHICH PATHS MATCH THE CLASS OF THIS USER.
* THE COUNT OF THESE PATHS WILL BE STORED IN R7.
***************************************************************
         SPACE 2
         LEVTA LEVEL=6,INUSE=QEA25000  BLOCK ON LEVEL 6?
         GETCC D6,L2                   NO  - BLOCK FOR QUERY REPLY AREA
         SPACE 2
QEA25000 DS    0H
         LA    R7,EBX000               MPIFC PARAMETER LIST ADDRESS
         XC    0(QURPLEN,R7),0(R7)     CLEAR THE PARAMETER LIST
         MVI   QUOPTS1-DCTMUP(R7),QUP1PTHL   WILL QUERY FOR LIST PATHS
         MVC   QUIDTOK-DCTMUP(4,R7),MUSITK   INDICATE USER'S IDTOK
         MVC   QUCLASS-DCTMUP(1,R7),MUSPCL   INDICATE THE PATH CLASS
         MVC   QURUNAM-DCTMUP(8,R7),MUSNAM   NAME OF REQUESTING USER
         MVC   QUDSNAM-DCTMUP(8,R7),QRUSYS   NAME OF DESTINATION SYS.
         L     R15,CE1CR6              INDICATE THE REPLY AREA ADDRESS
         ST    R15,QUFBAA-DCTMUP(,R7)  STORE THE REPLY AREA ADDRESS
         LA    R15,1055                LOAD LENGTH OF REPLY AREA
         ST    R15,QUFBAL-DCTMUP(,R7)  STORE THE LENGTH IN PARAM. LIST
         SPACE 2
         MPIFC RTYPE=QUERY,REG=R7      ISSUE THE MPIFC QUERY
         LTR   R15,R15                 WAS THE QUERY SUCCESSFUL?
         BNZ   QEA29100                NO  - BRANCH, ISSUE ERROR MSG
         SPACE 2
         L     R14,CE1CR6              LOAD REPLY AREA BASE REGISTER
         LH    R15,QRACT-DCTMUP(,R14)  LOAD NUMBER OF ALL AVAIL. PATHS
         LTR   R15,R15                 ARE THERE ANY PATHS AVAILABLE?
         BZ    QEA25300                NO  - GO ISSUE ERROR MESSAGE
         SR    R7,R7                   R7 WILL HOLD NUMBER OF PATHS
*                                      WITH THE CORRECT PATH CLASS
         SPACE 2
QEA25100 DS    0H
         CLC   QRPCLSI-DCTMUP(L'QRPCLSI,R14),MUSPCL   CLASSES MATCH?
         BNE   QEA25200                NO  - BRANCH, CHECK NEXT PATH
         LA    R7,1(,R7)               INCREMENT COUNT OF MATCHED PATHS
QEA25200 DS    0H
         LA    R14,QRALEN5(,R14)       POINT TO NEXT PATH IN LIST
         BCT   R15,QEA25100            LOOP IF MORE PATHS TO CHECK
         SPACE 2
         LTR   R7,R7                   ARE THERE ANY PATHS AVAILABLE?
         BNZ   QEA26000                YES - BRANCH AND CONTINUE
         SPACE 2
QEA25300 DS    0H
         WTOPC TEXT='NO CLASS .. PATHS TO ........, NO CONNECTION MADE X
               FOR ........',PREFIX=QEA2,NUM=04,TIME=YES,LET=E,        X
               SUB=(CHARA,MUSPCL,CHARA,QRUSYS,CHARA,EBW000),COMP=YES,  X
               ROUT=PRC
         B     QEA27000                DON'T MAKE A CONNECTION
         EJECT
***************************************************************
* FOR EACH AVAILABLE PATH TO THE OTHER SYSTEM A CONNECT SLOT
* IN THE SECONDARY BLOCK WILL BE CREATED, AND THE CONNECT
* SUBROUTINE WILL BE CALLED.
*
* IF THE NEW CONNECTION WILL CAUSE THE USER TO HAVE TOO MANY,
* AN ERROR MESSAGE IS PRINTED AND THE CONNECTION IS NOT MADE.
* THE MESSAGE WILL BE PRINTED ONCE FOR EACH CONNECTION THAT IS
* NOT ESTABLISHED.
***************************************************************
         SPACE 2
QEA26000 DS    0H
         LA    R15,MUSMAX              LOAD MAXIMUM NUMBER CONNECTIONS
         CH    R15,MUSNCN              WILL EXCEED MAX NUM CONNECTIONS?
         BH    QEA26100                NO  - BRANCH AND CONTINUE
         WTOPC TEXT='CONNECT REQUEST SUPPRESSED FOR ........ TO SYSTEM X
               ........',COMP=YES,SUB=(CHARA,EBW000,CHARA,EBW020),     X
               PREFIX=QEA2,NUM=05,LET=E,ROUT=PRC
         BCT   R7,QEA26000             LOOP FOR EVERY MISSED CONNECTION
         B     QEA27000                GO PROCESS THE NEXT USER
         SPACE 2
QEA26100 DS    0H
         LA    R6,MUSHLN(,R4)          POINT TO FIRST CONNECT SLOT
QEA26200 DS    0H                      FIND FREE CONNECT SLOT
         TM    MUSINDA,MUSUSE          IS THE SLOT IN USE?
         BZ    QEA26300                NO  - USE THIS ENTRY
         LA    R6,MUSCLN(,R6)          YES - POINT TO THE NEXT SLOT
         B     QEA26200                CHECK IF NEXT SLOT IS IN USE
         SPACE 2
QEA26300 DS    0H
         XC    0(MUSCLN,R6),0(R6)      CLEAR THE CONNECT SLOT
         ST    R5,EBW032               SAVE R5 ACROSS SUBROUTINE CALL
         ST    R7,EBW036               SAVE R7 ACROSS SUBROUTINE CALL
         ST    R1,EBW040               SAVE R1 ACROSS SUBROUTINE CALL
         BAS   R7,QEA28000             CALL THE CONNECT SUBROUTINE
         L     R5,EBW032               RESTORE R5
         L     R7,EBW036               RESTORE R7
         L     R1,EBW040               RESTORE R1
         SPACE 2
         OI    MUSINDA,MUSUSE          INDICATE CONNECT SLOT IS IN USE
         OI    MUSINDA,MUSINC          CONNECTION IS INCOMPLETE
         MVC   MUSSYSA,EBW020          SAVE DEST. SYSTEM IN SEC. BLOCK
         SPACE 2
         LH    R6,MUSNCN               LOAD TOTAL NUMBER OF CONNECTIONS
         LA    R6,1(,R6)               INCREMENT THAT NUMBER
         STH   R6,MUSNCN               SAVE NEW NUMBER OF CONNECTIONS
         SPACE 2
         LH    R6,MUSICON              NUMBER OF INCOMPLETE CONNECTIONS
         LA    R6,1(,R6)               INCREMENT THAT NUMBER
         STH   R6,MUSICON              NEW NUMBER OF INCOMPLETE CONN.
         SPACE 2
         TM    EBCM01,SYSRUN           SYSTEM TEST RUN?
         BO    QEA26400                YES - DON'T PRINT MESSAGE
         WTOPC TEXT='SUCCESSFUL CONNECT REQUEST FOR ........ TO SYSTEM X
               ........',COMP=YES,SUB=(CHARA,EBW000,CHARA,EBW020),     X
               PREFIX=QEA2,NUM=06,LET=I,ROUT=PRC
QEA26400 DS    0H
         BCT   R7,QEA26000             BRANCH IF THERE ARE MORE  ---
*                                      ---  PATHS TO CONNECT OVER
         EJECT
***************************************************************
* THE NEXT USER IN THE LIST POINTED TO BY R5 MUST BE PROCESSED.
* WHEN THERE ARE NO MORE USERS, THE PROGRAM CAN EXIT.
***************************************************************
         SPACE 2
QEA27000 DS    0H
         LA    R5,QRALEN1(,R5)         POINT TO NEXT USER IN LIST
         BCT   R3,QEA24100             BRANCH IF THERE ARE MORE USERS
         SPACE 2
         CRUSA S0=4,S1=6               RELEASE BLOCKS ON D4 AND D6
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         RESTART IS COMPLETE
         EJECT
***************************************************************
* THIS ROUTINE ISSUES THE MPIFC CONNECT REQUESTS.  THE UCTOK
* WILL BE THE ADDRESS OF THE CONNECTION SLOT.  THE UCTOK WILL
* ALSO BE PASSED IN THE USER DATA AREA (CNDATA).  THIS ALLOWS
* MTOM CONNECTIONS TO HAVE 'MATCHED' CONNECTION SLOTS LINKED
* TOGETHER (MTOM USERS HAVE 2 CONNECTION SLOTS FOR EACH
* CONNECTION).  THE LINK IS NECESSARY SO OTHER SEGMENTS DO NOT
* PERFORM THE SAME SERVICE TWICE ON A CONNECTION.  FOR EXAMPLE,
* QEA3 USES THE LINK TO KEEP FROM SENDING THE SAME MESSAGES
* TWICE OVER THE SAME CONNECTION.
***************************************************************
         SPACE 2
QEA28000 DS    0H
         LA    R5,EBX000               MPIFC PARAMETER LIST ADDRESS
         XC    0(CNRPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVC   CNPCLASS,EBW009         COPY THE PATH CLASS
         MVI   CNOPTS1,CNMPATH         INDICATE MULTI-PATHING
         MVC   CNIDTOK,MUSITK          COPY THE USER'S IDTOK
         MVC   CNOUNAM,MUSNAM          COPY THE USER'S NAME
         ST    R6,CNUCTOK              GIVE MPIF THE USER'S UCTOK
         MVC   CNDUNAM,MUSNAM          INDICATE DESTINATION USER'S NAME
         MVC   CNDSNAM,EBW020          DESTINATION SYSTEM NAME
         MVC   CNDRXAD,QEA2DRX         COPY DRX VECTOR INTO PARAM LIST
         MVC   CNMS,=AL4(4096)         MAX DATA REQUESTOR CAN SEND
         MVC   CNMR,=AL4(4096)         MAX DATA REQUESTOR CAN RECEIVE
         SR    R1,R1                   CLEAR ADDRESS REGISTER
         LA    R1,CNUCTOK              LOAD PTR TO THIS SIDE'S UCTOK
         ST    R1,CNDATA               UCTOK TO OTHER SIDE CONNECTION
         MVC   CNDATAL,=AL4(4)         LENGTH OF DATA IN CNDATA
         MPIFC RTYPE=CONNECT,REG=R5    ISSUE THE CONNECT REQUEST
         SPACE 2
         LTR   R15,R15                 WAS THE CONNECTION SUCCESSFUL?
         BZR   R7                      YES - RETURN TO CALLER
         B     QEA29200                NO  - BRANCH, ISSUE ERROR MSG
         EJECT
***************************************************************
* ISSUE ERROR MESSAGES FOR BAD MPIFC RETURN CODES.
***************************************************************
         SPACE 2
QEA29000 DS    0H
         WTOPC TEXT='MPIFC IDENTIFY RETURN CODE .... FOR ........',    X
               COMP=YES,PREFIX=QEA2,LET=E,TIME=YES,NUM=07,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,EBW000)
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEA29100 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC QUERY RETURN CODE .... FOR ........',       X
               COMP=YES,PREFIX=QEA2,LET=E,TIME=YES,NUM=08,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,EBW000)
         CRUSA S0=4,S1=6               RELEASE D4 AND D6 IF HELD
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEA29200 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC CONNECT RETURN CODE .... FOR ........',     X
               COMP=YES,PREFIX=QEA2,LET=E,TIME=YES,NUM=09,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,EBW000)
         CRUSA S0=4,S1=6               RELEASE D4 AND D6 IF HELD
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
***************************************************************
* THESE MPIF EXIT VECTORS INDICATE WHICH PROGRAMS ARE TO BE
* ACTIVATED WHEN A MPIF EXIT IS INVOKED.
***************************************************************
         SPACE 2
QEA2CRX  SWISC QEA5,IS=R0,TYPE=ENTER   CONNECTION REQUEST EXIT
QEA2CCX  SWISC QEA6,IS=R0,TYPE=ENTER   CONNECTION COMPLETE EXIT
QEA2ERX  SWISC QEAD,IS=R0,TYPE=ENTER   ERROR EXIT
QEA2DRX  SWISC QEAA,IS=R0,TYPE=ENTER   DATA RECEIVED EXIT
QEA2DUX  SWISC QEAB,IS=R0,TYPE=ENTER   DIRECTORY UPDATE EXIT
QEA2PAX  SWISC QEAC,IS=R0,TYPE=ENTER   PATH ACTIVATION NOTIF. EXIT
         EJECT
         LTORG
         FINIS QEA2
         END   ,
