         PRINT NOGEN
         BEGIN NAME=QEA3,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEA3                                       *
* RELATED MODULES. QEA0-QEA2, QEA4-QEA6, QEAA-QEAF            *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... "ZTEST MPIF SEND" PROCESSING               *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS RESPONSIBLE FOR SENDING         *
*             MESSAGES ACROSS MPIF CONNECTIONS.  IT IS CALLED *
*             EXPLICITLY BY A "ZTEST MPIF SEND" FROM QEA0, OR *
*             IMPLICITLY BY A "ZTEST MPIF START" FROM QEA1.   *
*             IT CAN ALSO BE CALLED BY A CREMC FROM ITSELF.   *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, CORE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBW000 - USER'S NAME                  *
*                       EBW008 - MESSAGE BLOCK SIZE AND TYPE  *
*                       EBW010 - NUMBER MESSAGES TO SEND      *
*                       EBW020 - QEA3 ENTRY INDICATOR         *
*                       EBW024 - QEA3 CREMC PARAMETER AREA    *
*                       EBW028 - CONNECT SLOT SAVE AREA       *
*                       EBW040 - @MSGCNT SAVE AREA            *
*                       EBW052 - DATA 1                       *
*                       EBX000 - MPIFC PARAMETER LIST         *
*                       CE1CR3 - DATA 2                       *
*                       CE1FAC - USED TO ESTABLISH EVENTS     *
*                                                             *
* REGISTER-CONVENTIONS..R1  - SD0RV  BASE REGISTER            *
*                       R2  - DCTMPR BASE REGISTER            *
*                       R3  - DCTMUS BASE REGISTER            *
*                       R5  - DCTMUP BASE REGISTER            *
*                       R7  - DCTMUS BASE REGISTER, SUFFIX=A  *
*                       R15 - DCTMUS BASE REGISTER, SUFFIX=B  *
*                                                             *
***************************************************************
*                                                             *
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... THE INVOKING SEGMENT MUST HAVE SET UP THE   *
*                 EBW000 WORK AREA WITH INFORMATION ABOUT THE *
*                 USER, AND THE TYPE OF ENTRY (SEE BELOW).    *
*                                                             *
*                 IT IS ASSUMED QEA0 HAS PARSED AND ERROR     *
*                 CHECKED ANY INPUT MESSAGES.                 *
*                                                             *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - "SEND" COMMANDS CAN BE ENTERED ON ANY     *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * EBW000 - USER'S NAME * NONE                 *
*               * EBW008 - MESSAGE     *                      *
*               *          SIZE AND    *                      *
*               *          TYPE INDEX  *                      *
*               * EBW010 - NUMBER MSGS *                      *
*               *          TO SEND     *                      *
*               * EBW020 - ENTRY       *                      *
*               *          INDICATOR   *                      *
*               *          (SEE BELOW) *                      *
*               * EBW024 - QEA3 CREMC  *                      *
*               *          PARAMETER   *                      *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * NONE                 * NONE                 *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  @ISSDRV (DRIVER GLOBALS)                 *
*                        - @MPRADD  : DRIVER PRIMARY RECORD   *
*                                     ADDRESS                 *
*                        - @MSGCNT  : MESSAGE COUNTER         *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. EXITC, AFTER PROCESSING THE REQUESTED         *
*     -ERROR... SERRC EEA002 - EVNTC NAME USED TWICE          *
*               SERRC EEA003 - EVNWC NAME NOT FOUND           *
*                                                             *
***************************************************************
*                                                             *
* ENTRY TO QEA3 CAN HAPPEN 3 WAYS:                            *
*                                                             *
* 1.) ENTRY VIA ENTNC FROM QEA0, DUE TO A "ZTEST MPIF SEND"   *
*     REQUEST. EBW020 WILL CONTAIN X'00000000'.               *
*                                                             *
* 2.) ENTRY VIA CREMC FROM QEA1, DUE TO A "ZTEST MPIF START"  *
*     REQUEST.  EBW020 WILL CONTAIN X'00000000', AND          *
*     PROCESSING IS IDENTICAL TO AN ENTRY FROM QEA0.          *
*                                                             *
* 3.) ENTRY VIA CREMC FROM QEA3, BECAUSE THE USER HAS MORE    *
*     THAN ONE CONNECTION.  QEA3 WILL CREMC TO ITSELF ONCE    *
*     FOR EACH ADDITIONAL CONNECTION, AND EACH COPY WILL SEND *
*     MESSAGES OVER ONE CONNECTION.  EBW020 WILL CONTAIN      *
*     C'QEA3', AND EBW024 WILL CONTAIN AN ADDRESS OF A        *
*     CONNECTION SLOT WITHIN THE USER'S SECONDARY BLOCK.  THE *
*     MESSAGES WILL BE SENT OVER THIS CONNECTION.             *
*                                                             *
* IN ALL THREE CASES, ADDITIONAL INFORMATION MUST ALSO BE     *
* PASSED.  EBW000-EBW007 MUST CONTAIN THE USER'S NAME, EBW008 *
* MUST CONTAIN THE SIZE AND TYPE OF THE MESSAGES TO BE SENT,  *
* AND EBW010 MUST CONTAIN THE NUMBER OF MESSAGES TO BE SENT.  *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* ESTABLISH DSECTS, LOAD BASE REGISTERS, AND GET INFORMATION
* ABOUT ROUTING AND THE DRIVER'S STATE FROM THE @ISSDRV GLOBAL.
***************************************************************
         SPACE  2
         DCTMUP LTYPE=SEND,REG=R5      MPIFC SEND PARAMETER LIST
         SPACE  2
         DCTMPR REG=R2                 MPIF DRIVER PRIMARY RECORD DSECT
         DCTMUS REG=R3                 DRIVER SECONDARY RECORD DSECT
         DCTMUS REG=R7,SUFFIX=A        DSECT THE CONNECTION SLOTS ---
         DCTMUS REG=R15,SUFFIX=B       --- WITHIN THE SECONDARY BLOCK
         SPACE  2
         SD0RV  REG=R1                 DSECT DRIVER INFORMATION AREA
         GLOBZ  REGR=R14               GET GLOBAL AREA ADDRESS
         L      R1,@ISSDRV             LOAD SD0RV BASE REGISTER
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE O/P MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         EJECT
***************************************************************
* THE DRIVER'S PRIMARY RECORD HAS TO BE SEARCHED TO SEE IF THE
* USER SPECIFIED IN EBW000 HAS BEEN IDENTIFIED TO THE SYSTEM.
* IF FOUND:
*
* 1.) LOAD THE USER'S SECONDARY BLOCK BASE REGISTER.
* 2.) SEE IF THE USER IS CONNECTED.  IF NOT, ISSUE AN ERROR
*     MESSAGE AND EXIT.
* 3.) LOCK THE SECONDARY BLOCK SINCE IT MAY BE UPDATED.
* 4.) SEE HOW THE ECB WAS CREATED.  IF IT WAS CREATED BY A
*     CREMC FROM QEA3, LOAD THE BASE REGISTER FOR THE
*     CONNECTION SLOT AND GO SEND MESSAGES OVER THAT
*     CONNECTION.
***************************************************************
         SPACE 2
         L     R2,@MPRADD              GET DRIVER'S PRIMARY BLOCK ADDR.
         LTR   R2,R2                   HAS THE BLOCK BEEN CREATED YET?
         BZ    QEA38000                NO  - GO ISSUE AN ERROR MESSAGE
         SPACE 2
         LA    R0,MPRMAX               LOAD NUMBER OF ENTRY'S IN TABLE
QEA31000 DS    0H
         CLC   MPRNAM,EBW000           DO THE USER NAMES MATCH?
         BE    QEA31100                YES - BRANCH, USE THIS MPR SLOT
         LA    R2,MPRLEN(,R2)          NO  - POINT TO THE NEXT MPR SLOT
         BCT   R0,QEA31000             BRANCH IF THERE ARE MORE SLOTS
         B     QEA38000                NO MORE SLOTS, GO ISSUE ERROR
         SPACE 2
QEA31100 DS    0H
         L     R3,MPRMUS               LOAD THE USER'S SEC. BLOCK ADDR
         OC    MUSNCN,MUSNCN           IS THE USER CONNECTED TO ANYONE?
         BZ    QEA38100                NO  - GO ISSUE ERROR MESSAGE
         OC    MUSICON,MUSICON         HAVE INCOMPLETE CONNECTIONS?
         BNZ   QEA38150                YES - GO ISSUE ERROR MESSAGE
         CORHC (R3)                    LOCK THE SECONDARY BLOCK
         SPACE 2
         CLC   EBW020(4),=C'QEA3'      ECB CREATED BY CREMC MACRO?
         BNE   QEA32000                NO  - BRANCH AND CONTINUE
         L     R7,EBW024               CONNECT SLOT TO SEND MSGS OVER
         B     QEA34000                GO SEND THE MESSAGES
         EJECT
***************************************************************
* THIS ECB WAS NOT KICKED OFF BY A CREMC FROM QEA3, MEANING IT
* RECEIVED CONTROL DIRECTLY AS A RESULT OF A SEND REQUEST.
* THIS ECB WILL SEND DATA OVER THE FIRST CONNECTION IN THE
* SECONDARY BLOCK, AND QEA3 WILL CREMC TO ITSELF TO SEND
* MESSAGES OVER ANY ADDITIONAL CONNECTIONS.
*
* SINCE MTOM USERS HAVE TWO CONNECTIONS SLOTS FOR EACH
* CONNECTION, A ROUTINE WILL BE CALLED TO SET FLAGS IN THE
* SECONDARY BLOCK.  THIS WILL PREVENT DATA FROM BEING SENT IN
* BOTH DIRECTIONS OVER THE CONNECTION.
***************************************************************
         SPACE 2
QEA32000 DS    0H
         LA    R7,MUSHLN(,R3)          LOAD ADDRESS FIRST CONNECT SLOT
         LA    R4,MUSMAX               LOAD MAXIMUM NUMBER CONNECTIONS
         XC    EBW028(4),EBW028        CLEAR A WORK AREA
         SPACE 2
QEA32100 DS    0H
         TM    MUSINDA,MUSUSE          IS THIS CONNECTION SLOT IN USE?
         BZ    QEA32400                NO  - GO LOOK AT THE NEXT SLOT
         CLI   MUSDEV,DEVCTOC          IS THIS A CTOC TYPE USER?
         BE    QEA32200                YES - NO NEED TO CALL SUBROUTINE
         BAS   R5,QEA33000             GO TAKE CARE OF MTOM CONNECTIONS
         LTR   R14,R14                 SEND MSGS OVER THIS END CONN.?
         BZ    QEA32400                NO  - BRANCH, LOOK AT NEXT SLOT
         SPACE 2
QEA32200 DS    0H
         OC    EBW028(4),EBW028        IS THIS FIRST CONNECT SLOT?
         BNZ   QEA32300                NO  - GO ISSUE A CREMC
         ST    R7,EBW028               YES - SAVE THE SLOT'S ADDRESS
         B     QEA32400                GO LOOK AT THE NEXT SLOT
         SPACE 2
QEA32300 DS    0H
         MVC   EBW020(4),=C'QEA3'      INDICATE QEA3 ENTRY FROM QEA3
         ST    R7,EBW024               PASS THIS CONNECT SLOT ADDRESS
         LA    R14,30                  NUMBER OF BYTES TO PASS IN CREMC
         LA    R15,EBW000              PASS WORK AREA IN CREMC MACRO
         CREMC QEA3                    GO SEND OVER THIS CONNECTION
         SPACE 2
QEA32400 DS    0H
         LA    R7,MUSCLN(,R7)          POINT TO THE NEXT CONNECT SLOT
         BCT   R4,QEA32100             BRANCH IF MORE SLOTS TO CHECK
         L     R7,EBW028               LOAD FIRST CONNECT SLOT FOUND
         B     QEA34000                GO SEND OVER FIRST CONNECTION
         EJECT
***************************************************************
* THIS IS A MTOM USER, SO IT WILL HAVE BOTH 'ENDS' OF ANY
* CONNECTIONS IN THE SAME SECONDARY BLOCK.  STEPS NEED TO BE
* TO ENSURE MESSAGES ARE SENT IN ONLY ONE DIRECTION OVER THE
* CONNECTION.
*
* IF THIS IS THE FIRST 'END' OF THE CONNECTION, A FLAG WILL SET
* IN THE CONNECTION SLOT OF THE OTHER 'END.' THIS WILL INDICATE
* THE MESSAGES HAVE ALREADY BEEN SENT.
*
* THERE IS A SEPARATE INDICATOR FOR EACH MESSAGE SIZE AND TYPE.
* THIS IS TO PREVENT THE ECB'S CREATED BY QEA1 DURING SYSTEM
* TEST RUNS FROM RACING ON ONE INDICATOR.
*
* THIS ROUTINE WILL RETURN ZERO IN R14 IF NO MESSAGES SHOULD BE
* SENT OVER THE CONNECTION POINTED TO BY R7.
***************************************************************
         SPACE 2
QEA33000 DS    0H
         SR    R14,R14                 CLEAR A RETURN CODE REGISTER
         SR    R15,R15                 CLEAR AN INDEX REGISTER
         IC    R15,EBW008              LOAD BLOCK TYPE INDEX FROM ECB
         SLL   R15,2                   CREATE A BRANCH INDEX
         B     QEA33100(R15)           BRANCH BASED ON THE BLOCK TYPE
QEA33100 DS    0H
         B     QEA33200                381  BYTE NON-SHAREABLE BLOCK
         B     QEA33300                1055 BYTE NON-SHAREABLE BLOCK
         B     QEA33400                4K   BYTE NON-SHAREABLE BLOCK
         B     QEA33500                381  BYTE COMMON BLOCK
         B     QEA33600                1055 BYTE COMMON BLOCK
         B     QEA33700                4K   BYTE COMMON BLOCK
         SPACE 2
QEA33200 DS    0H
         TM    MUSIND2A,MUS3SN         381 NON-SHAREABLE ALREADY SENT?
         BZ    QEA33250                NO  - BRANCH AND SEND MESSAGES
         NI    MUSIND2A,X'FF'-MUS3SN   YES - CLEAR INDICATOR
         BR    R5                      RETURN TO CALLER
QEA33250 DS    0H
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSIND2B,MUS3SN         IND. TO OTHER SIDE MSGS SENT
         LA    R14,1(,R14)             SEND MESSAGES OVER CONNECTION
         BR    R5                      RETURN TO CALLER
         SPACE 2
QEA33300 DS    0H
         TM    MUSIND2A,MUS1SN         1055 NON-SHAREABLE ALREADY SENT?
         BZ    QEA33350                NO  - BRANCH AND SEND MESSAGES
         NI    MUSIND2A,X'FF'-MUS1SN   YES - CLEAR INDICATOR
         BR    R5                      RETURN TO CALLER
QEA33350 DS    0H
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSIND2B,MUS1SN         IND. TO OTHER SIDE MSGS SENT
         LA    R14,1(,R14)             SEND MESSAGES OVER CONNECTION
         BR    R5                      RETURN TO CALLER
         SPACE 2
QEA33400 DS    0H
         TM    MUSIND2A,MUS4SN         4K NON-SHAREABLE ALREADY SENT?
         BZ    QEA33450                NO  - BRANCH AND SEND MESSAGES
         NI    MUSIND2A,X'FF'-MUS4SN   YES - CLEAR INDICATOR
         BR    R5                      RETURN TO CALLER
QEA33450 DS    0H
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSIND2B,MUS4SN         IND. TO OTHER SIDE MSGS SENT
         LA    R14,1(,R14)             SEND MESSAGES OVER CONNECTION
         BR    R5                      RETURN TO CALLER
         SPACE 2
QEA33500 DS    0H
         TM    MUSIND2A,MUS3CSN        381 COMMON ALREADY SENT?
         BZ    QEA33550                NO  - BRANCH AND SEND MESSAGES
         NI    MUSIND2A,X'FF'-MUS3CSN  YES - CLEAR INDICATOR
         BR    R5                      RETURN TO CALLER
QEA33550 DS    0H
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSIND2B,MUS3CSN        IND. TO OTHER SIDE MSGS SENT
         LA    R14,1(,R14)             SEND MESSAGES OVER CONNECTION
         BR    R5                      RETURN TO CALLER
         SPACE 2
QEA33600 DS    0H
         TM    MUSIND2A,MUS1CSN        1055 COMMON ALREADY SENT?
         BZ    QEA33650                NO  - BRANCH AND SEND MESSAGES
         NI    MUSIND2A,X'FF'-MUS1CSN  YES - CLEAR INDICATOR
         BR    R5                      RETURN TO CALLER
QEA33650 DS    0H
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSIND2B,MUS1CSN        IND. TO OTHER SIDE MSGS SENT
         LA    R14,1(,R14)             SEND MESSAGES OVER CONNECTION
         BR    R5                      RETURN TO CALLER
         SPACE 2
QEA33700 DS    0H
         TM    MUSIND2A,MUS4CSN        4K COMMON ALREADY SENT?
         BZ    QEA33750                NO  - BRANCH AND SEND MESSAGES
         NI    MUSIND2A,X'FF'-MUS4CSN  YES - CLEAR INDICATOR
         BR    R5                      RETURN TO CALLER
QEA33750 DS    0H
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSIND2B,MUS4CSN        IND. TO OTHER SIDE MSGS SENT
         LA    R14,1(,R14)             SEND MESSAGES OVER CONNECTION
         BR    R5                      RETURN TO CALLER
         EJECT
***************************************************************
* THE GLOBAL @MSGCNT IS A COUNTER USED TO CREATE THE MESSAGES.
* IT'S VALUE WILL BE PLACED IN THE FIRST FULLWORD OF DATA 2,
* AND EACH SUBSEQUENT FULLWORD WILL BE GIVEN A VALUE ONE
* GREATER THAN THE PREVIOUS FULLWORD.
***************************************************************
         SPACE 2
QEA34000 DS    0H
         LA    R5,@MSGCNT              LOCK ON @MSGCNT ADDRESS
         CORHC (R5)                    LOCK THE @MSGCNT COUNTER
         L     R6,@MSGCNT              LOAD THE CURRENT @MSGCNT VALUE
         A     R6,=F'1'                INCREMENT THE @MSGCNT VALUE
         ST    R6,@MSGCNT              SAVE THE NEW VALUE FOR NEXT ECB
         CORUC (R5)                    UNLOCK THE GLOBAL
         ST    R6,EBW040               SAVE THE UNIQUE COUNTER VALUE
         EJECT
***************************************************************
* MPIFC CAN SEND 2 DATA MESSAGES.  THE FIRST (DATA 1) WILL BE
* BUILT IN EBW052 AND WILL CONTAIN THE FOLLOWING:
*
*       EBW052 - BIT 0 = 0 THIS IS AN UNSOLICITED MESSAGE
*                BIT 0 = 1 THIS IS A MESSAGE ACKNOWLEDGEMENT
*
*       EBW053 THROUGH EBW054 - NUMBER OF MESSAGES BEING SENT
*                               BY THIS USER.
*
*       EBW055 THROUGH EBW062 - THE CHARACTERS 'VERIFYMSG'
*
* EBW053-54 ARE MEANINGFUL ONLY WHEN THE MESSAGE IS UNSOLICITED
* (BIT 0 OF EBW052 = 0).
*
* MESSAGE ACKNOWLEDGEMENT WILL COME FROM THE DATA RECEIVED
* EXIT (QEAA).
*
* THE CHARACTERS 'VERIFYMSG' ARE PROVIDED SO QEAA CAN DETERMINE
* IF DATA 1 HAS BEEN CORRUPTED DURING TRANSMISSION.  IF THESE
* BYTES ARE NOT CORRUPTED, QEAA WILL ASSUME THE OTHER FIELDS
* (EBW052 THROUGH EBW054) ARE ALSO VALID.
***************************************************************
         SPACE 2
         LH    R6,EBW010               LOAD NUMBER OF MESSAGES TO SEND
         XC    EBW052(1),EBW052        INDICATE UNSOLICITED MESSAGE
         MVC   EBW053(2),EBW010        MOVE NUMBER OF MSGS TO DATA 1
         MVC   EBW055(9),=C'VERIFYMSG' MOVE IN VERIFICATION CHARACTERS
         EJECT
***************************************************************
* AN EVENT WILL BE CREATED, WHICH WILL BE USED TO SIGNAL THE
* MESSAGES HAVE BEEN ACKNOWLEDGED.  THE DATA RECEIVED EXIT WILL
* ISSUE A POSTC TO COMPLETE THE EVENT WHEN THE ACKNOWLEDGEMENT
* IS RECEIVED.  THE EVENT NAME IS BASED ON THE SUBSYSTEM-USER
* ID AND THE VALUE RETRIEVED FROM @MSGCNT.  THE TIMEOUT VALUE
* WILL BE ONE SECOND FOR EACH MESSAGE, WITH A MINIMUM OF 3
* MINUTES, AND A MAXIMUM OF 10 MINUTES.
***************************************************************
         SPACE 2
         MVC   CE1FAC(2),CE1SSU        EVENT NAME BASED ON SSU ID
         MVC   CE1FAC+2(2),CE1SSU
         MVC   CE1FAC+4(4),EBW040      EVENT NAME BASED ON @MSGCNT
         SPACE 2
         LR    R15,R6                  LOAD TIMEOUT VALUE = NUM. MSGS
         CL    R15,=F'180'             TIMEOUT LESS THAN 3 MINUTES?
         BNL   QEA35000                NO  - BRANCH AND CONTINUE
         LA    R15,180                 YES - GIVE AT LEAST 3 MINUTES
         B     QEA35100                BRANCH AND ESTABLISH EVENT
QEA35000 DS    0H
         CL    R15,=F'600'             TIMEOUT MORE THAN 10 MINUTES?
         BL    QEA35100                NO  - GO ESTABLISH EVENT
         LA    R15,600                 YES - ENFORCE 10 MINUTE LIMIT
QEA35100 DS    0H
         MVC   CE1CRC,=X'0001'         ONLY NEED ONE POSTC TO COMPLETE
         EVNTC TYPE=CNT,NAME=Y,DUPNAM=QEA38200,                        X
               LEVEL=DC,TIMEOUT=(R15),STATE=1052
         EJECT
***************************************************************
* RELEASE THE LOCK ON THE SECONDARY BLOCK.  THIS WILL ALLOW
* OTHER ECB'S TO BEGIN PROCESSING WHILE THIS ONE IS ISSUING
* I/O AND WAITING ON EVENTS.
*
* SINCE THE LOCK WILL BE RELEASED, THE ACTIVE ECB INDICATOR
* IN THE SECONDARY BLOCK MUST BE INCREMENTED.  THIS WILL
* PREVENT QEA4 FROM ISSUING A 'FORGET' REQUEST AND RELEASING
* THE SECONDARY BLOCK, WHICH WOULD CAUSE THIS SEGMENT TO ACT
* UNPREDICTABLY.
***************************************************************
         SPACE 2
         L     R15,MUSECB              LOAD NUMBER USER'S ACTIVE ECB'S
         LA    R15,1(,R15)             ONE MORE ECB IS ACTIVE
         ST    R15,MUSECB              NEW NUMBER USER'S ACTIVE ECB'S
         SPACE 2
         CORUC (R3)                    RELEASE LOCK ON SECONDARY BLOCK
         EJECT
***************************************************************
* THE SECOND MESSAGE (DATA 2) WILL BE CONTAINED ON DATA LEVEL
* 3.  AN APPROPRIATE SIZE AND TYPE BLOCK WILL BE RETRIEVED, AND
* IT WILL BE INITIALIZED USING THE VALUE RETRIEVED FROM
* @MSGCNT.  THE FIRST FULLWORD WILL CONTAIN @MSGCNT, THE SECOND
* WILL CONTAIN @MSGCNT+1, ETC..
*
* THE SIZE AND TYPE OF BLOCK IS INDICATED IN EBW008, AND IS
* DEFINED AS FOLLOWS:
*
*        X'00' - 381  BYTE 'ORDINARY' BLOCK (NON-SHAREABLE)
*        X'01' - 1055 BYTE 'ORDINARY' BLOCK (NON-SHAREABLE)
*        X'02' - 4096 BYTE 'ORDINARY' BLOCK (NON-SHAREABLE)
*        X'03' - 381  BYTE COMMON BLOCK
*        X'04' - 1055 BYTE COMMON BLOCK
*        X'05' - 4096 BYTE COMMON BLOCK
*
* PROCESSING FROM THIS POINT WILL BE REPEATED FOR EACH MESSAGE.
***************************************************************
         SPACE 2
QEA35150 DS    0H
         SR    R15,R15                 CLEAR AN INDEX REGISTER
         IC    R15,EBW008              LOAD BLOCK TYPE INDEX FROM ECB
         SLL   R15,2                   CREATE A BRANCH INDEX
         B     QEA35200(R15)           GET CORRECT BLOCK FOR DATA 2
QEA35200 DS    0H
         B     QEA35300                381  BYTE NON-SHAREABLE BLOCK
         B     QEA35400                1055 BYTE NON-SHAREABLE BLOCK
         B     QEA35500                4K   BYTE NON-SHAREABLE BLOCK
         B     QEA35600                381  BYTE COMMON BLOCK
         B     QEA35700                1055 BYTE COMMON BLOCK
         B     QEA35800                4K   BYTE COMMON BLOCK
         SPACE 2
QEA35300 DS    0H
         GETCC D3,L1                   GET 381  BYTE NON-SHAREABLE BLK
         LA    R4,95                   WILL PROPAGATE @MSGCNT 95 TIMES
         B     QEA35900                GO FILL THE MESSAGE BLOCK
         SPACE 2
QEA35400 DS    0H
         GETCC D3,L2                   GET 1055 BYTE NON-SHAREABLE BLK
         LA    R4,263                  WILL PROPAGATE @MSGCNT 263X
         B     QEA35900                GO FILL THE MESSAGE BLOCK
         SPACE 2
QEA35500 DS    0H
         GETCC D3,L4                   GET 4K   BYTE NON-SHAREABLE BLK
         LA    R4,1023                 WILL PROPAGATE @MSGCNT 1023X
         B     QEA35900                GO FILL THE MESSAGE BLOCK
         SPACE 2
QEA35600 DS    0H
         GETCC D3,L1,COMMON=YES        GET 381  BYTE COMMON BLOCK
         LA    R4,95                   WILL PROPAGATE @MSGCNT 95 TIMES
         B     QEA35900                GO FILL THE MESSAGE BLOCK
         SPACE 2
QEA35700 DS    0H
         GETCC D3,L2,COMMON=YES        GET 1055 BYTE COMMON BLOCK
         LA    R4,263                  WILL PROPAGATE @MSGCNT 263X
         B     QEA35900                GO FILL THE MESSAGE BLOCK
         SPACE 2
QEA35800 DS    0H
         GETCC D3,L4,COMMON=YES        GET 4K   BYTE COMMON BLOCK
         LA    R4,1023                 WILL PROPAGATE @MSGCNT 1023X
         B     QEA35900                GO FILL THE MESSAGE BLOCK
         SPACE 2
QEA35900 DS    0H                      CREATE DATA 2 MESSAGE
         L     R15,EBW040              LOAD STARTING @MSGCNT VALUE
QEA35950 DS    0H
         ST    R15,0(R14)              STORE @MSGCNT VALUE INTO DATA 2
         A     R15,=F'1'               CREATE NEXT @MSGCNT VALUE
         LA    R14,4(,R14)             POINT TO NEXT FULLWORD IN DATA 2
         BCT   R4,QEA35950             BRANCH IF MORE FULLWORDS TO DO
         EJECT
***************************************************************
* BUILD THE MPIFC SEND PARAMETER LIST IN EBX000.
***************************************************************
         SPACE 2
         LA    R5,EBX000               SEND PARAM. LIST BASE REGISTER
         XC    0(SNRPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVC   SNCONTOK,MUSCTKA        INDICATE USER'S CONNECT TOKEN
         MVC   SNDSNAM,MUSSYSA         INDICATE THE DESTINATION SYSTEM
         LA    R15,EBW052              INDICATE THE DATA 1 ADDRESS
         ST    R15,SNDATA1
         MVC   SNDBSZ1,=AL2(12)        INDICATE SIZE OF DATA 1
         MVC   SNDATA2(8),CE1CR3       INDICATE ADDRESS OF DATA 2
         MVC   SNDLVL2,=AL1(D3)        INDICATE DATA LEVEL OF DATA 2
         EJECT
***************************************************************
* ISSUE THE MPIFC SEND AND BRANCH ACCORDING THE RETURN CODE.
* IF THE SEND IS SUCCESSFUL THE PROGRAM CAN LOOP BACK TO SEND
* MORE MESSAGES.  R6 CONTAINS THE NUMBER OF MESSAGES BEING
* SENT.
*
* IF THE RETURN CODE IS 4 OR 12, THERE ARE NOT ENOUGH SYSTEM
* RESOURCES TO SEND THE MESSAGE.  THIS ECB WILL BE PLACED ON
* THE INPUT LIST, AND UPON RETURN, WILL TRY TO SEND THE MESSAGE
* AGAIN.
*
*      **** NOTE ****    **** NOTE ****    **** NOTE ****
*
* DATA LEVEL 3, WHICH CONTAINS DATA 2, MUST BE RELEASED BEFORE
* THIS ECB IS PLACED ON THE INPUT LIST.  OTHERWISE THE DRIVER
* CAN DEADLOCK, WAITING FOR RESOURCES TO FREE UP, WHEN IN FACT
* THE DRIVER IS HOLDING ALL OF THEM.
*
*      **** NOTE ****    **** NOTE ****    **** NOTE ****
*
***************************************************************
         SPACE 2
QEA36000 DS    0H
         MPIFC RTYPE=SEND,REG=R5       ISSUE THE MPIFC SEND
         B     QEA36100(R15)           BRANCH BASED ON THE RETURN CODE
         SPACE 2
QEA36100 DS    0H
         B     QEA36200                 0 - NORMAL COMPLETION
         B     QEA36300                 4 - MPIF OUTPUT QUEUE EXCEEDED
         B     QEA38300                 8 - SENDING OVER INACTIVE PATH
         B     QEA36300                12 - NO CONTROL STORAGE AVAIL.
         B     QEA38400                16 - NO PACING CREDITS AVAIL.
         B     QEA38400                20 - INVALID PARAMETER
         SPACE 2
QEA36200 DS    0H                      RETURN CODE 0
         CORHC (R3)                    RE-OBTAIN SECONDARY BLOCK LOCK
         L     R15,MUSMSN              LOAD NUMBER MSGS SENT BY USER
         LA    R15,1(,R15)             ONE MORE MESSAGE WAS SENT
         ST    R15,MUSMSN              STORE NEW NUMBER MESSAGES SENT
         CORUC (R3)                    UNLOCK THE SECONDARY BLOCK
         BCT   R6,QEA35150             BRANCH IF MORE MESSAGES TO SEND
         B     QEA37000                GO WAIT FOR ACKNOWLEDGEMENT
         SPACE 2
QEA36300 DS    0H                      RETURN CODE 4 OR 12
         RELCC D3                      RELEASE BLOCK WITH DATA 2
         DLAYC                         WAIT FOR RESOURCES TO FREE UP
         B     QEA35150                TRY TO SEND MESSAGE AGAIN
         EJECT
***************************************************************
* ALL OF THE MESSAGES HAVE BEEN SENT SUCCESSFULLY, SO THE EVENT
* MUST BE WAITED ON.  THE DATA RECEIVED EXIT (QEAA) WILL POST
* THE EVENT.
***************************************************************
         SPACE 2
QEA37000 DS    0H
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEA37100                YES - DON'T PRINT MESSAGES
         LH    R1,EBW010               LOAD NUMBER MESSAGES SENT
         WTOPC TEXT='ALL MESSAGES SENT SUCCESSFULLY BY ........',      X
               PREFIX=QEA3,NUM=01,ROUT=PRC,CHAIN=YES,ENDOFM=NO,LET=I,  X
               COMP=YES,SUB=(CHARA,EBW000)
         WTOPC TEXT='                   NUMBER MESSAGES: ...., EVENT: .X
               ......., DEST: ........',ROUT=PRC,CHAIN=YES,ENDOFM=YES, X
               SUB=(DEC,(R1),HEX4A,CE1FAC+4,CHARA,MUSSYSA),HEADER=NO
         SPACE 2
QEA37100 DS    0H                      WAIT FOR ACKNOWLEDGEMENT
         MVC   CE1FAC(2),CE1SSU        EVENT NAME BASED ON SSU ID
         MVC   CE1FAC+2(2),CE1SSU
         MVC   CE1FAC+4(4),EBW040      EVENT NAME BASED ON @MSGCNT
         EVNWC TYPE=CNT,LEVEL=DC,NFOUND=QEA38500,ERROR=QEA38600
         SPACE 2
         CORHC (R3)                    RE-OBTAIN SECONDARY BLOCK LOCK
         L     R15,MUSARC              NUMBER ACKNOWLEDGEMENTS RECEIVED
         AH    R15,EBW010              ADD NUMBER MESSAGES JUST SENT
         ST    R15,MUSARC              STORE NEW NUMBER ACK. RECEIVED
         CORUC (R3)                    UNLOCK THE SECONDARY BLOCK
         SPACE 2
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEA39000                YES - CAN GO EXIT SEGMENT
         WTOPC TEXT='ACKNOWLEDGEMENT RECEIVED FOR MESSAGES SENT BY ....X
               ....',PREFIX=QEA3,NUM=02,LET=I,ROUT=PRC,CHAIN=YES,      X
               ENDOFM=NO,COMP=YES,SUB=(CHARA,EBW000)
         WTOPC TEXT='                   NUMBER MESSAGES: ...., EVENT: .X
               ......., SOURCE: ........',ROUT=PRC,CHAIN=YES,HEADER=NO,X
               SUB=(DEC,(R1),HEX4A,CE1FAC+4,CHARA,MUSSYSA),ENDOFM=YES
         B     QEA39000                GO EXIT THIS SEGMENT
         EJECT
***********************************************************************
* ERROR ROUTINES.
***********************************************************************
         SPACE 2
QEA38000 DS    0H                      USER DOES NOT EXIST
         WTOPC TEXT='........ DOES NOT EXIST, NO MESSAGES SENT',       X
               PREFIX=QEA3,NUM=03,LET=E,COMP=YES,SUB=(CHARA,EBW000)
         EXITC
         SPACE 2
***********************************************************************
         SPACE 2
QEA38100 DS    0H                      USER NOT CONNECTED
         WTOPC TEXT='........ IS NOT CONNECTED TO ANOTHER USER',       X
               PREFIX=QEA3,NUM=04,LET=E,COMP=YES,SUB=(CHARA,EBW000)
         EXITC
         SPACE 2
***********************************************************************
         SPACE 2
QEA38150 DS    0H                      USER HAS INCOMPLETE CONNECTIONS
         WTOPC TEXT='........ HAS INCOMPLETE CONNECTIONS',             X
               PREFIX=QEA3,NUM=07,LET=E,COMP=YES,SUB=(CHARA,EBW000)
         EXITC
         SPACE 2
***********************************************************************
         SPACE 2
QEA38200 DS    0H                      DUPNAM BRANCH TAKEN ON EVNTC
         CORUC (R3)                    RELEASE LOCK ON SECONDARY BLOCK
         LA    R0,EEA2MSG              WILL APPEND MESSAGE TO DUMP
         SERRC E,EEA002,MSG,OP,ECB     DUMP, EVENT NAME USED TWICE
         SPACE 2
EEA2MSG  DS    0D
         DC    AL1(ENDEEA2-BEGEEA2)
BEGEEA2  DC    C'EVENT NAME USED BY EVNTC MACRO ALREADY IN USE'
ENDEEA2  EQU   *
         SPACE 2
***********************************************************************
         SPACE 2
QEA38300 DS    0H                      MPIFC RETURN CODE 8
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEA38450                YES - GO EXIT, DON'T PRINT MSG
         SPACE 2
QEA38400 DS    0H                      MPIFC RETURN CODE 16 OR 20
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC SEND RETURN CODE .... FOR ........',        X
               COMP=YES,PREFIX=QEA3,LET=E,TIME=YES,NUM=05,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,EBW000),CHAIN=YES,ENDOFM=NO
         WTOPC TEXT='                   EVENT NUMBER: ........, DESTINAX
               TION SYSTEM: ........',ROUT=PRC,CHAIN=YES,ENDOFM=YES,   X
               HEADER=NO,COMP=NO,SUB=(HEX4A,CE1FAC+4,CHARA,MUSSYSA)
         SPACE 2
QEA38450 DS    0H
         RELCC D3                      RELEASE DATA 2
         B     QEA39000                GO EXIT THIS SEGMENT
         SPACE 2
***********************************************************************
         SPACE 2
QEA38500 DS    0H                      NFOUND BRANCH TAKE ON EVNWC
         LA    R0,EEA3MSG              WILL APPEND MESSAGE TO DUMP
         SERRC R,EEA003,MSG,OP,ECB     DUMP, EVNWC NAME NOT FOUND
         B     QEA39000                GO EXIT THIS SEGMENT
         SPACE 2
EEA3MSG  DS    0D
         DC    AL1(ENDEEA3-BEGEEA3)
BEGEEA3  DC    C'EVENT NAME USED BY EVNWC MACRO NOT FOUND'
ENDEEA3  EQU   *
         SPACE 2
***********************************************************************
         SPACE 2
QEA38600 DS    0H                      EVENT WAIT ENDED WITH ERROR
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEA39000                YES - GO EXIT, DON'T PRINT MSG
         WTOPC TEXT='MESSAGES SENT BY ........ WERE NOT ACKNOWLEDGED', X
               SUB=(CHARA,EBW000),CHAIN=YES,ENDOFM=NO,COMP=YES,LET=E,  X
               PREFIX=QEA3,NUM=06,ROUT=PRC,TIME=YES
         WTOPC TEXT='                   NUMBER MESSAGES: ...., EVENT: .X
               ......., SOURCE: ........',ROUT=PRC,CHAIN=YES,HEADER=NO,X
               SUB=(DEC,(R1),HEX4A,CE1FAC+4,CHARA,MUSSYSA),ENDOFM=YES
         B     QEA39000                GO EXIT THIS SEGMENT
         SPACE 2
***********************************************************************
         EJECT
***************************************************************
* INDICATE THIS ECB IS NO LONGER ACTIVE AND EXIT.
***************************************************************
         SPACE 2
QEA39000 DS    0H
         CORHC (R3)                    RE-OBTAIN SECONDARY BLOCK LOCK
         L     R15,MUSECB              LOAD NUMBER USER'S ACTIVE ECB'S
         BCTR  R15,0                   ONE LESS ECB IS ACTIVE
         ST    R15,MUSECB              NEW NUMBER USER'S ACTIVE ECB'S
         CORUC (R3)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
         LTORG
         FINIS QEA3
         END   ,
