         PRINT NOGEN
         BEGIN NAME=QEA4,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEA4                                       *
* RELATED MODULES. QEA0-QEA3, QEA5-QEA6, QEAA-QEAF            *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... "ZTEST MPIF FORGET" PROCESSING             *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT HANDLES "ZTEST MPIF FORGET"        *
*             REQUESTS.  IT WILL ISSUE A MPIFC FORGET FOR THE *
*             USER SPECIFIED IN THE COMMAND.                  *
*                                                             *
*             QEA4 CAN ALSO BE CALLED FROM QEA1 TO 'FORGET'   *
*             USERS FOR A "ZTEST MPIF STOP" REQUEST.          *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBW000 - USER'S NAME                  *
*                       EBX000 - MPIFC FORGET PARAMETER LIST  *
*                                                             *
* REGISTER-CONVENTIONS..R1 - SD0RV  BASE REGISTER             *
*                       R2 - DCTMPR BASE REGISTER             *
*                       R3 - DCTMUS BASE REGISTER             *
*                       R4 - DCTMUP BASE REGISTER             *
*                       R5 - USED TO LOCK PRIMARY RECORD      *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... IT IS ASSUMED QEA0 HAS PARSED AND ERROR     *
*                 CHECKED THE INPUT MESSAGE.  QEA0 MUST PUT   *
*                 THE USER'S NAME IN EBW000-EBW007.  QEA4     *
*                 WILL CHECK IF THE SPECIFIED USER EXISTS IN  *
*                 THE SYSTEM.                                 *
*                                                             *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - "FORGET" COMMANDS CAN BE ENTERED ON ANY
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * EBW000 - USER'S NAME * NONE                 *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * NONE                 * NONE                 *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  @ISSDRV (DRIVER GLOBALS)                 *
*                        - @MPRADD  : DRIVER PRIMARY RECORD   *
*                                     ADDRESS                 *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. EXITC, OPERATION COMPLETED SUCCESSFULLY       *
*     -ERROR... EXITC, OPERATION WAS NOT SUCCESSFUL           *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* ESTABLISH DSECTS, LOAD BASE REGISTERS, AND GET INFORMATION
* ABOUT ROUTING AND THE DRIVER'S STATE FROM THE @ISSDRV GLOBAL.
***************************************************************
         SPACE  2
         DCTMPR REG=R2                 MPIF DRIVER PRIMARY BLOCK DSECT
         DCTMUS REG=R3                 MPIF DRIVER SECONDARY BLK DSECT
         DCTMUP REG=R4,LTYPE=FORGET    DSECT THE FORGET PARAMETER LIST
         SPACE  2
         SD0RV  REG=R1                 DSECT DRIVER INFORMATION AREA
         GLOBZ  REGR=R14               GET GLOBAL AREA DSECT
         L      R1,@ISSDRV             LOAD SD0RV DSECT BASE REGISTER
         SPACE  2
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE O/P MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         EJECT
***************************************************************
* TRY AND FIND THE USER IN THE PRIMARY BLOCK.  THE PRIMARY
* BLOCK'S ADDRESS WILL BE LOCKED ON WHILE THE RECORD IS BEING
* EXAMINED AND UPDATED.
***************************************************************
         SPACE 2
         L     R5,@MPRADD              LOAD THE MPR ADDRESS
         CORHC (R5)                    LOCK ON THE PRIMARY RECORD
         SPACE 2
         L     R2,@MPRADD              LOAD PRIMARY BLOCK ADDRESS
         LTR   R2,R2                   DOES A PRIMARY BLOCK EXIST YET?
         BZ    QEA44000                NO - GO ISSUE ERROR MESSAGE
         SPACE 2
         LA    R7,MPRMAX               LOAD MAXIMUM NUMBER OF USERS
QEA41000 DS    0H
         CLC   MPRNAM,EBW000           DO THE USER NAMES MATCH?
         BE    QEA42000                YES - ENTRY FOUND IN PRIMARY BLK
         LA    R2,MPRLEN(,R2)          NO  - BUMP TO THE NEXT USER
         BCT   R7,QEA41000             CHECK NEXT ENTRY IF MORE EXIST
         B     QEA44000                NO MORE ENTRIES - USER NOT FOUND
         EJECT
***************************************************************
* CHECK AND SEE IF THIS USER HAS ANY ACTIVE ECB'S IN THE SYSTEM
* THAT ARE SENDING OR RECEIVING MESSAGES.  IF SO, ISSUE AN
* ERROR MESSAGE AND EXIT, THE MPIFC FORGET WILL NOT BE ISSUED.
***************************************************************
         SPACE 2
QEA42000 DS    0H
         L     R3,MPRMUS               ADDR. OF SEC. BLK FOR THIS USER
         L     R4,MUSECB               LOAD NUMBER USER'S ACTIVE ECB'S
         LTR   R4,R4                   ARE THERE ANY ACTIVE ECB'S?
         BNZ   QEA44100                YES - GO ISSUE ERROR MESSAGE
         EJECT
***************************************************************
* FORGET THE USER.  THE SECONDARY BLOCK FOR THIS USER IS
* RELEASED AND THE ENTRY FOR THIS USER IN THE DRIVER'S PRIMARY
* BLOCK IS CLEARED.  IF THE MPIFC RETURN CODE IS ZERO, AND THIS
* IS A SYSTEM TEST RUN, THE FORGET MESSAGE IS SUPPRESSED.
***************************************************************
         SPACE 2
         LA    R4,EBX000               POINT TO FORGET PARAMETER LIST
         XC    EBX000(FGRPLEN),EBX000  CLEAR THE PARAMETER LIST
         MVC   FGIDTOK,MUSITK          COPY IDTOK TO PARAMETER LIST
         MVC   FGRUNAM,EBW000          COPY USER NAME TO PARAMETER LIST
         MPIFC RTYPE=FORGET,REG=R4     EXECUTE MPIFC FORGET
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         SPACE 2
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BZ    QEA43000                NO - GO REPORT RETURN CODE
         LTR   R6,R6                   WAS RETURN CODE ZERO?
         BZ    QEA43100                YES - DON'T REPORT RETURN CODE
QEA43000 DS    0H
         WTOPC TEXT='FORGET ISSUED FOR ........, RETURN CODE ....',    X
               COMP=YES,PREFIX=QEA4,LET=I,TIME=YES,NUM=01,ROUT=PRC,    X
               SUB=(CHARA,EBW000,DEC,(R6))
         SPACE 2
QEA43100 DS    0H
         ST    R3,CE1CR2               PUT SECONDARY BLOCK ON LEVEL 2
         MVC   CE1CT2,=AL2(L4)         SAVE BLOCK TYPE ON LEVEL 2
         RELCC D2                      RELEASE SECONDARY BLOCK
         MONTC
         XC    MPRNAM(MPRLEN),MPRNAM   CLEAR PRIMARY BLOCK ENTRY
         LMONC
         CORUC (R5)                    RELEASE THE MPR LOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
***************************************************************
* WTOPC FOR USER NOT FOUND ERROR AND USER HAS ACTIVE ECB'S.
***************************************************************
         SPACE 2
QEA44000 DS    0H
         CORUC (R5)                    RELEASE THE MPR LOCK
         WTOPC TEXT='........ DOES NOT EXIST, FORGET NOT ISSUED',      X
               PREFIX=QEA4,NUM=02,LET=E,COMP=YES,SUB=(CHARA,EBW000)
         EXITC
         SPACE 2
QEA44100 DS    0H
         CORUC (R5)                    RELEASE THE MPR LOCK
         WTOPC TEXT='........ HAS ACTIVE ECBS IN THE SYSTEM, FORGET NOTX
                ISSUED',PREFIX=QEA4,NUM=03,LET=E,COMP=YES,             X
               SUB=(CHARA,EBW000)
         EXITC
         EJECT
         LTORG
         FINIS QEA4
         END   ,
