         PRINT NOGEN
         BEGIN NAME=QEA5,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEA5                                       *
* RELATED MODULES. QEA0-QEA4, QEA6, QEAA-QEAF                 *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF TEST DRIVER CONNECTION REQUEST EXIT   *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS THE MPIF TEST DRIVER'S          *
*             CONNECTION REQUEST EXIT (CRX).  IT IS INVOKED   *
*             FOR A MPIF USER WHEN ANOTHER USER IN THE MPIF   *
*             COMPLEX WANTS TO ESTABLISH A CONNECTION.  A     *
*             MPIFC ACCEPT WILL BE ISSUED, UNLESS DOING SO    *
*             WILL EXCEED THIS USER'S MAXIMUM NUMBER OF       *
*             CONNECTIONS.                                    *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBW008 - NAME OF USER REQUESTING      *
*                                CONNECTION                   *
*                       EBW016 - CONNECTION'S CONTOK          *
*                       EBW020 - SYSTEM OF USER REQUESTING    *
*                                CONNECTION                   *
*                       EBW028 - UCTOK OF USER REQUESTING     *
*                                CONNECTION                   *
*                       EBX000 - MPIFC PARAMETER LISTS        *
*                                                             *
* REGISTER-CONVENTIONS..R1 - SD0RV BASE REGISTER              *
*                       R4 - DCTMUS BASE REGISTER             *
*                       R5 - DCTMUP BASE REGISTER             *
*                       R6 - DCTMUS BASE REGISTER, SUFFIX = A *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - THIS EXIT CAN BE INITIATED ON ANY         *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * NONE                 *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * R1 - CRX PARAMETER   * NONE                 *
*               *      LIST            *                      *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  @ISSDRV - DRIVER GLOBAL AREA             *
*                                                             *
* EXIT-NORMAL.. EXITC, CONNECTION SUCCESSFULLY ACCEPTED       *
*     -ERROR... EXITC, COULD NOT ACCEPT THE REQUEST           *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* ESTABLISH DSECTS, LOAD BASE REGISTERS, AND GET INFORMATION
* ABOUT ROUTING AND THE DRIVER'S STATE FROM THE @ISSDRV GLOBAL.
***************************************************************
         SPACE  2
         DCTMUP REG=R5,LTYPE=ALL       MPIFC PARAMETER DSECTS
         LR     R5,R1                  LOAD CRX PARAMETER LIST ADDRESS
         SPACE  2
         DCTMUS REG=R4                 DRIVER SECONDARY BLOCK DSECT
         DCTMUS REG=R6,SUFFIX=A        DSECT CONNECTION SLOTS
         SPACE  2
         SD0RV  REG=R1                 DRIVER INFORMATION AREA DSECT
         GLOBZ  REGR=R15               GET GLOBAL 1 ADDRESS
         L      R1,@ISSDRV             SET @ISSDRV BASE REGISTER
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE O/P MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         EJECT
***************************************************************
* THE SECONDARY BLOCK FOR THE USER WILL BE UPDATED TO REFLECT
* THE NEW CONNECTION, AND THE MPIFC ACCEPT WILL BE ISSUED.  THE
* ADDRESS OF THIS USER'S SECONDARY BLOCK IS THE UITOK RETURNED
* IN THE CRX PARAMETER LIST.  IF THE MAXIMUM NUMBER OF
* CONNECTIONS FOR THIS USER WILL BE EXCEEDED BY ISSUING THE
* ACCEPT, A DISCONNECT WILL BE ISSUED INSTEAD.
*
* THE UCTOK FOR THIS CONNECTION WILL BE THE ADDRESS OF THE
* CONNECTION SLOT.  THIS ADDRESS WILL BE ALSO BE PASSED IN THE
* USER DATA AREA (ACDATA).  THIS IS SO MTOM USERS CAN HAVE
* 'MATCHED' CONNECTION SLOTS LINKED TOGETHER.  MTOM USERS WILL
* HAVE 2 CONNECTION SLOTS FOR EACH CONNECTION.  THE LINK IS
* NECESSARY SO OTHER SEGMENTS DO NOT PERFORM THE SAME SERVICE
* TWICE ON A CONNECTION.  FOR EXAMPLE, QEA3 USES THE LINK TO
* KEEP FROM SENDING THE SAME MESSAGES TWICE OVER THE SAME
* CONNECTION.
*
* THE USER THAT ISSUED THE CONNECT REQUEST ALSO PASSED IT'S
* UCTOK IN THE DATA AREA, IN ORDER FOR THE LINK TO BE CREATED.
* IT IS IN THE CRECDATA FIELD, AND IT WILL BE MOVED INTO THIS
* USER'S CONNECTION SLOT.  THIS WILL CREATE 'HALF' OF THE LINK.
***************************************************************
         SPACE 2
         MVC   EBW016(4),CRECTOK       COPY THE CONNECTION'S CONTOK
         MVC   EBW008(8),CREOUNAM      NAME OF USER IN OTHER SYSTEM
         MVC   EBW020(8),CREOSNAM      SYSTEM CONTAINING OTHER USER
         MVC   EBW028(4),CRECDAT       UCTOK OF OTHER SIDE CONNECTION
         L     R4,CREUITOK             DCTMUS OF USER IN THIS SYSTEM
         CORHC (R4)                    LOCK THE SECONDARY BLOCK
         SPACE 2
         LA    R2,MUSMAX               LOAD MAXIMUM NUMBER CONNECTIONS
         CH    R2,MUSNCN               WILL EXCEED MAX NUM CONNECTIONS?
         BNH   QEA52000                YES - GO ISSUE THE DISCONNECT
         SPACE 2
         LA    R6,MUSHLN(,R4)          FIRST CONNECT SLOT IN SEC. BLK
QEA51000 DS    0H                      FIND AN EMPTY CONNECTION SLOT
         TM    MUSINDA,MUSUSE          IS THIS CONNECT SLOT IN USE?
         BZ    QEA51100                NO  - USE THIS CONNECT SLOT
         LA    R6,MUSCLN(,R6)          YES - POINT TO NEXT ENTRY
         B     QEA51000                SEE IF NEXT SLOT IS IN USE
         SPACE 2
QEA51100 DS    0H
         XC    0(MUSCLN,R6),0(R6)      CLEAR THIS CONNECT SLOT
         LA    R5,EBX000               ACCEPT PARAMETER LIST BASE REG.
         XC    0(ACRPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVC   ACIDTOK,MUSITK          COPY THIS USER'S IDTOK
         MVC   ACCONTOK,EBW016         COPY THE CONNECTION'S CONTOK
         ST    R6,ACUCTOK              DEFINE THE CONNECTION'S UCTOK
         MVC   ACMS,=AL4(4096)         MAX DATA THIS USER CAN SEND
         MVC   ACMR,=AL4(4096)         MAX DATA THIS USER CAN RECEIVE
         MVC   ACRNAM,EBW008           COPY THE OTHER USER'S NAME
         MVC   ACANAM,EBW008           COPY THIS USER'S NAME
         MVC   ACDRXAD,QEA5DRX         DEFINE DATA RECEIVED EXIT VECTOR
         SR    R1,R1                   CLEAR ADDRESS REGISTER
         LA    R1,ACUCTOK              LOAD PTR TO THIS SIDE'S UCTOK
         ST    R1,ACDATA               UCTOK TO OTHER SIDE CONNECTION
         MVC   ACDATAL,=AL4(4)         LENGTH OF DATA IN ACDATA
         MPIFC RTYPE=ACCEPT,REG=R5     ISSUE THE ACCEPT REQUEST
         LTR   R15,R15                 SUCCESSFUL ACCEPT REQUEST?
         BNZ   QEA53000                NO  - GO REPORT THE RETURN CODE
         SPACE 2
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEA51200                YES - DON'T PRINT MESSAGE
         WTOPC TEXT='CONNECTION REQUEST FROM SYSTEM ........ ACCEPTED BX
               Y ........',COMP=YES,SUB=(CHARA,EBW020,CHARA,MUSNAM),   X
               PREFIX=QEA5,NUM=01,LET=I,ROUT=PRC
         SPACE 2
QEA51200 DS    0H                      UPDATE THE CONNECTION SLOT
         MVC   MUSSYSA,EBW020          COPY THE OTHER SYSTEM'S NAME
         OI    MUSINDA,MUSUSE          MARK THE CONNECTION SLOT IN USE
         MVC   MUSCTKA,EBW016          SAVE THIS CONNECTION'S CONTOK
         MVC   MUSOCSA(4),EBW028       SAVE OTHER SIDE'S UCTOK
         LH    R14,MUSNCN              CURRENT NUMBER OF CONNECTIONS
         LA    R14,1(,R14)             INCREMENT NUMBER OF CONNECTIONS
         STH   R14,MUSNCN              SAVE THE NEW NUMBER
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC
         SPACE 2
QEA5DRX  SWISC QEAA,IS=R0,TYPE=ENTER   DATA RECEIVED EXIT VECTOR
         EJECT
***************************************************************
* THE USER WOULD HAVE EXCEEDED IT'S MAXIMUM NUMBER OF
* CONNECTIONS IF IT ACCEPTED THE CONNECTION REQUEST.  INSTEAD,
* THE USER WILL ISSUE A DISCONNECT TO REJECT THE REQUEST.  A
* REASON CODE IS PROVIDED IN THE DISCONNECT PARAMETER LIST TO
* INFORM THE OTHER USER AS TO WHY THE REQUEST WAS REJECTED.
***************************************************************
         SPACE 2
QEA52000 DS    0H
         LA    R5,EBX000                LOAD DISCONNECT PARAMETER BASE
         XC    EBX000(DSRPLEN),EBX000   CLEAR THE PARAMETER LIST
         MVC   DSIDTOK,MUSITK           USER'S ID TOKEN TO PARAM. LIST
         MVC   DSCONTOK(4),EBW016       THIS CONNECTION'S CONTOK
         MVC   DSRUNAM(8),MUSNAM        REQUESTING USER'S NAME TO LIST
         MVC   DSDUNAM(8),MUSNAM        DESTINATION USER'S NAME TO LIST
         MVC   DSREASON(4),=X'00004242' INDICATE REASON FOR DISCONNECT
         MPIFC RTYPE=DISCONNECT,REG=R5  ISSUE THE DISCONNECT REQUEST
         LTR   R15,R15                  MPIFC COMPLETE SUCCESSFULLY?
         BNZ   QEA53100                 NO  - GO ISSUE ERROR MESSAGE
         SPACE 2
         WTOPC TEXT='CONNECTION REQUEST FROM SYSTEM ........ REJECTED BX
               Y ........',SUB=(CHARA,EBW020,CHARA,MUSNAM),COMP=YES,   X
               PREFIX=QEA5,NUM=02,LET=E,ROUT=PRC,CHAIN=YES,ENDOFM=NO
         WTOPC TEXT='                   MAXIMUM NUMBER OF CONNECTIONS WX
               OULD BE EXCEEDED',TIME=NO,CHAIN=YES,ENDOFM=YES,ROUT=PRC,X
               HEADER=NO
         CORUC (R4)                     UNLOCK THE SECONDARY BLOCK
         EXITC
         EJECT
***************************************************************
* ISSUE ERROR MESSAGES FOR BAD RETURN CODES FROM MPIFC.
***************************************************************
         SPACE 2
QEA53000 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC ACCEPT RETURN CODE .... FOR ........',      X
               COMP=YES,PREFIX=QEA5,LET=E,TIME=YES,NUM=03,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,MUSNAM)
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEA53100 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC DISCONNECT RETURN CODE .... FOR ........',  X
               COMP=YES,PREFIX=QEA5,LET=E,TIME=YES,NUM=04,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,MUSNAM)
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
         LTORG
         FINIS QEA5
         END   ,
