         PRINT NOGEN
         BEGIN NAME=QEA6,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEA6                                       *
* RELATED MODULES. QEA0-QEA5, QEAA-QEAF                       *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF TEST DRIVER'S CONNECTION COMPLETE     *
*                  EXIT (CCX)                                 *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS THE MPIF TEST DRIVER'S          *
*             CONNECTION COMPLETE EXIT.  IT IS INVOKED WHEN   *
*             A USER HAS ACCEPTED A CONNECTION REQUEST THIS   *
*             USER HAD PREVIOUSLY ISSUED.  THE USER'S         *
*             SECONDARY BLOCK WILL BE UPDATED TO REFLECT THE  *
*             COMPLETED CONNECTION.                           *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
* REGISTER-CONVENTIONS..R1 - SD0RV BASE REGISTER              *
*                       R4 - DCTMUS BASE REGISTER             *
*                       R5 - DCTMUP BASE REGISTER             *
*                       R6 - DCTMUS BASE REGISTER, SUFFIX = A *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - THIS EXIT CAN BE INITIATED ON ANY         *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * NONE                 *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * R1 - CCX PARAMETER   * NONE                 *
*               *      LIST            *                      *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  @ISSDRV - DRIVER GLOBAL AREA             *
*                                                             *
* EXIT-NORMAL.. EXITC, CONNECTION COMPLETED SUCCESSFULLY      *
*     -ERROR... EXITC, CONNECTION DID NOT COMPLETE            *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* ESTABLISH DSECTS, LOAD BASE REGISTERS, AND GET INFORMATION
* ABOUT ROUTING AND THE DRIVER'S STATE FROM THE @ISSDRV GLOBAL.
***************************************************************
         SPACE  2
         DCTMUP REG=R5,LTYPE=ALL       MPIFC PARAMETER DSECTS
         LR     R5,R1                  LOAD CCX PARAMETER LIST ADDRESS
         SPACE  2
         DCTMUS REG=R4                 DRIVER SECONDARY BLOCK DSECT
         DCTMUS REG=R6,SUFFIX=A        DSECT CONNECTION SLOTS
         SPACE  2
         SD0RV  REG=R1                 DRIVER INFORMATION AREA DSECT
         GLOBZ  REGR=R15               GET GLOBAL 1 ADDRESS
         L      R1,@ISSDRV             LOAD @ISSDRV BASE ADDRESS
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE OUTPUT MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         EJECT
***************************************************************
* UPDATE THE USER'S SECONDARY BLOCK.  THE UITOK RECEIVED IN THE
* CCX PARAMETER LIST IS THE ADDRESS OF THE USER'S SECONDARY
* BLOCK.  THE UCTOK IS THE ADDRESS OF THE CONNECTION SLOT OVER
* WHICH THE ACCEPT REQUEST WAS RECEIVED.
*
* THE USER DATA AREA (CCECDAT) CONTAINS THE UCTOK FOR THE USER
* WHICH ISSUED THE ACCEPT REQUEST.  THIS COMPLETES THE LINKING
* OF CONNECTION SLOTS FOR MTOM USERS.
***************************************************************
         SPACE 2
         L     R6,CCEUCT               UCTOK = ADDR. OF CONNECTION SLOT
         L     R4,CCEUIT               UITOK = ADDR. OF USER'S DCTMUS
         CORHC (R4)                    LOCK THE SECONDARY BLOCK
         SPACE 2
         LH    R14,MUSICON             INDICATE ONE LESS INCOMPLETE ---
         BCTR  R14,0                   --- CONNECTION
         STH   R14,MUSICON
         SPACE 2
         OC    CCECV,CCECV             COMPLETE SUCCESSFULLY?
         BNZ   QEA62000                NO  - GO ISSUE ERROR MESSAGE
         SPACE 2
         MVC   MUSCTKA,CCECT           SAVE CONTOK IN SECONDARY BLOCK
         MVC   MUSOCSA,CCECDAT         UCTOK OF OTHER SIDE CONNECTION
         NI    MUSINDA,X'FF'-MUSINC    CONNECTION IS NOW COMPLETE
         SPACE 2
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEA61000                YES - DON'T PRINT MESSAGE
         WTOPC TEXT='CONNECTION COMPLETED FOR ........ TO SYSTEM ......X
               ..',PREFIX=QEA6,NUM=01,LET=I,ROUT=PRC,COMP=YES,         X
               SUB=(CHARA,MUSNAM,CHARA,CCEOSNAM)
         SPACE 2
QEA61000 DS    0H
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC
         EJECT
***************************************************************
* CONNECTION DID NOT COMPLETE SUCCESSFULLY.  IF THE COMPLETION
* VALUE IS X'4242', THE OTHER USER ISSUED A DISCONNECT INSTEAD
* OF AN ACCEPT.  THE DISCONNECT WAS ISSUED BECAUSE IF THE USER
* ISSUED AN ACCEPT, THEY WOULD HAVE EXCEEDED THEIR MAXIMUM
* NUMBER OF CONNECTIONS.
***************************************************************
         SPACE 2
QEA62000 DS    0H
         XC    MUSINDA,MUSINDA         MARK THE CONNECTION SLOT FREE
         LH    R14,MUSNCN              CURRENT NUMBER OF CONNECTIONS
         BCTR  R14,0                   DECREMENT NUMBER OF CONNECTIONS
         STH   R14,MUSNCN              SAVE THE NEW NUMBER
         SPACE 2
         CLC   CCECV(2),=X'4242'       DID THE OTHER USER DISCONNECT?
         BE    QEA62100                YES - GO ISSUE THE MESSAGE
         SPACE 2
         LA    R2,CCECV                LOAD ADDRESS OF COMPLETION VALUE
         WTOPC TEXT='CONNECTION NOT ESTABLISHED FOR ........,',        X
               COMP=YES,PREFIX=QEA6,LET=E,TIME=YES,NUM=02,CHAIN=YES,   X
               SUB=(CHARA,MUSNAM),ROUT=PRC,ENDOFM=NO
         WTOPC TEXT='                   COMPLETION VALUE ....',        X
               SUB=(HEX4A,(R2)),ROUT=PRC,ENDOFM=YES,                   X
               HEADER=NO,TIME=NO,CHAIN=YES,COMP=NO
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC
         SPACE 2
QEA62100 DS    0H
         WTOPC TEXT='CONNECTION NOT ESTABLISHED FOR ........,',        X
               COMP=YES,PREFIX=QEA6,LET=E,TIME=YES,NUM=03,CHAIN=YES,   X
               SUB=(CHARA,MUSNAM),ROUT=PRC,ENDOFM=NO
         WTOPC TEXT='                   CONNECT REQUEST WAS REJECTED', X
               COMP=NO,ROUT=PRC,ENDOFM=YES,HEADER=NO,TIME=NO,CHAIN=YES
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC
         EJECT
         LTORG
         FINIS QEA6
         END   ,
