         PRINT NOGEN
         BEGIN NAME=QEAA,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEAA                                       *
* RELATED MODULES. QEA0-QEA6, QEAB-QEAF                       *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF TEST DRIVER DATA RECEIVED EXIT (DRX)  *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS THE MPIF TEST DRIVER DATA       *
*             RECEIVED EXIT.  IT IS INVOKED WHENEVER A MPIF   *
*             USER RECEIVES A MESSAGE.  EACH MESSAGE WILL     *
*             FALL INTO ONE OF TWO CATEGORIES:                *
*                1.) THE MESSAGE WAS RECEIVED UNSOLICITED     *
*                    FROM ANOTHER USER.  (BIT 0 OF BYTE 1 OF  *
*                    DATA 1 = B'0').                          *
*                2.) THIS MESSAGE WAS RECEIVED FROM ANOTHER   *
*                    USER TO ACKNOWLEDGE A MESSAGE.  (BIT 0   *
*                    OF BYTE 1 OF DATA 1 = B'1').             *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, CORE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBX000 - MPIFC PARAMETER LIST         *
*                       CE1CR0 - MPIF DATA 1                  *
*                       CE1CR1 - MPIF DATA 2                  *
*                       CE1CRC - USED TO ESTABLISH EVENTS     *
*                       CE1FAC - USED TO ESTABLISH EVENTS     *
*                                                             *
* REGISTER-CONVENTIONS..R1 - SD0RV BASE REGISTER              *
*                       R3 - DCTMUS BASE REGISTER             *
*                       R4 - DCTMUS BASE REGISTER, SUFFIX=A   *
*                       R5 - DCTMUP BASE REGISTER             *
*                       R6 - DATA 1 ADDRESS                   *
*                       R7 - DATA 2 ADDRESS                   *
*                                                             *
***************************************************************
*                                                             *
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - THIS EXIT CAN BE INITIATED ON ANY         *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * NONE                 *
* DATA LEVELS.. * D0 - MPIF DATA 1     * NONE                 *
*               * D1 - MPIF DATA 2     *                      *
* REGISTERS.... * R1 - DRX PARAMETER   * NONE                 *
*               *      LIST            *                      *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  @ISSDRV (DRIVER GLOBALS)                 *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. EXITC, NORMAL COMPLETION                      *
*     -ERROR... SERRC EEA003 - EVNWC NAME NOT FOUND           *
*               SERRC EEA004 - MESSAGE CORRUPTED              *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* ESTABLISH DSECTS AND LOAD BASE REGISTERS.  THE UITOK IS THE
* ADDRESS OF THE USER'S SECONDARY BLOCK.  THE UCTOK IS THE
* ADDRESS OF A CONNECTION SLOT IN THE SECONDARY BLOCK.  THE
* MESSAGE WAS RECEIVED OVER THIS CONNECTION.
***************************************************************
         SPACE  2
         DCTMUP REG=R5,LTYPE=ALL       MPIFC PARAMETER DSECTS
         LR     R5,R1                  LOAD DRX PARAMETER LIST ADDRESS
         SPACE  2
         SD0RV  REG=R1                 DSECT DRIVER INFORMATION AREA
         GLOBZ  REGR=R14               GET GLOBAL AREA 1 ADDRESS
         L      R1,@ISSDRV             LOAD @ISSDRV BASE REGISTER
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST RUN INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE OUTPUT MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         SPACE  2
         DCTMUS REG=R3                 DRIVER SECONDARY BLOCK DSECT
         DCTMUS REG=R4,SUFFIX=A        DSECT CONNECTION SLOTS
         L      R3,DREUITOK            SECONDARY BLOCK (DCTMUS) BASE
         L      R4,DREUCTOK            DCTMUS' CONNECT SLOT BASE REG.
         EJECT
***************************************************************
* VERIFY THE DATA SENT OVER MPIF WAS NOT CORRUPTED.
*
* FOLLOWING THE INDICATORS IN DATA 1 THERE SHOULD BE THE
* CHARACTERS "VERIFYMSG".  IF SO, IT WILL BE ASSUMED THE
* INDICATORS ARE ALSO VALID.
*
* FOR DATA 2, EACH FULLWORD SHOULD CONTAIN A VALUE ONE GREATER
* THAN THE VALUE IN THE PREVIOUS FULLWORD.
***************************************************************
         SPACE 2
         L     R6,CE1CR0               LOAD DATA 1 ADDRESS
         L     R7,CE1CR1               LOAD DATA 2 ADDRESS
         SPACE 2
QEAA1000 DS    0H
         CLC   3(9,R6),=C'VERIFYMSG'   HAS DATA 1 BEEN CORRUPTED?
         BNE   QEAA1500                YES - GO ISSUE SYSTEM ERROR DUMP
         SPACE 2
         CLC   DREDBTP2(2),=AL2(L1)    DATA 2 IN 381 BYTE BLOCK?
         BNE   QEAA1100                NO  - CHECK FOR 1055 BLOCK
         LA    R1,94                   NEED TO CHECK 94 FULL WORDS
         B     QEAA1300                GO CHECK DATA 2 INTEGRITY
QEAA1100 DS    0H
         CLC   DREDBTP2(2),=AL2(L2)    DATA 2 IN 1055 BYTE BLOCK?
         BNE   QEAA1200                NO  - CHECK FOR 4K BLOCK
         LA    R1,262                  NEED TO CHECK 262 FULL WORDS
         B     QEAA1300                GO CHECK DATA 2 INTEGRITY
QEAA1200 DS    0H
         CLC   DREDBTP2(2),=AL2(L4)    DATA 2 IN 4K BYTE BLOCK?
         BNE   QEAA1500                NO  - DATA CORRUPTED, ISSUE DUMP
         LA    R1,1022                 NEED TO CHECK 1022 FULL WORDS
         SPACE 2
QEAA1300 DS    0H
         L     R2,0(,R7)               LOAD FIRST WORD OF DATA 2
QEAA1400 DS    0H
         A     R2,=F'1'                EXPECTED VALUE OF NEXT FULLWORD
         LA    R7,4(,R7)               POINT TO NEXT WORD OF DATA 2
         CL    R2,0(,R7)               THIS WORD = FIRST WORD?
         BNE   QEAA1500                NO  - DATA CORRUPTED, ISSUE DUMP
         BCT   R1,QEAA1400             BRANCH IF MORE WORDS TO CHECK
         B     QEAA2000                GO CONTINUE PROCESSING
         SPACE 2
QEAA1500 DS    0H
         LA    R0,EEA4MSG              WILL APPEND MESSAGE TO DUMP
         SERRC E,EEA004,MSG,OP,ECB     DUMP, DATA CORRUPTED
EEA4MSG  DS    0D
         DC    AL1(ENDEEA4-BEGEEA4)
BEGEEA4  DC    C'CORRUPTED MESSAGE RECEIVED BY THE DRX'
ENDEEA4  EQU   *
         EJECT
***************************************************************
* THE SECONDARY BLOCK WILL NOT BE LOCKED DURING PROCESSING.
* THIS WILL ALLOW OTHER ECB'S TO CONTINUE WHILE THIS ONE IS
* ISSUING I/O AND WAITING ON EVENTS.  TO PREVENT QEA4 FROM
* ISSUING A 'FORGET' REQUEST, AND RELEASING THE SECONDARY
* BLOCK, THE ACTIVE ECB COUNT WILL BE INCREMENTED.
***************************************************************
         SPACE 2
QEAA2000 DS    0H
         CORHC (R3)                    LOCK SECONDARY BLOCK TO UPDATE
         L     R15,MUSECB              LOAD NUMBER USER'S ACTIVE ECB'S
         LA    R15,1(,R15)             ONE MORE ECB IS ACTIVE
         ST    R15,MUSECB              NEW NUMBER USER'S ACTIVE ECB'S
         CORUC (R3)                    UNLOCK THE SECONDARY BLOCK
         EJECT
***************************************************************
* THE TYPE OF MESSAGE MUST BE DETERMINED, AND APPROPRIATE
* ACTION TAKEN.  BIT 0 OF BYTE 1 OF DATA 1 INDICATES WHETHER
* THE MESSAGE IS UNSOLICITED, OR AN ACKNOWLEDGEMENT TO A
* MESSAGE THE USER PREVIOUSLY SENT OUT.
***************************************************************
         SPACE 2
         TM    0(R6),X'80'             WAS THIS UNSOLICITED MESSAGE?
         BNZ   QEAA3000                NO  - MSG IS ACKNOWLEDGEMENT
         B     QEAA4000                YES - GO SEND ACKNOWLEDGEMENT
         EJECT
***************************************************************
* THIS IS AN ACKNOWLEDGEMENT TO A MESSAGE THE USER PREVIOUSLY
* SENT OUT.  THEREFORE, THERE IS ANOTHER ECB IN THE SYSTEM
* WAITING FOR IT.  THIS ECB WILL ISSUE A POSTC TO SIGNAL THE
* MESSAGE HAS BEEN ACKNOWLEDGED.
*
* THE EVENT NAME IS BASED ON THE FIRST FULLWORD IN DATA 2, AND
* THE SUBSYSTEM-USER ID.  IF THE EVENT DOES NOT EXIST, AN ERROR
* IS PRINTED SAYING THE USER HAS RECEIVED AN ACKNOWLEDGEMENT IT
* WASN'T WAITING FOR.
***************************************************************
         SPACE 2
QEAA3000 DS    0H
         L     R7,CE1CR1               LOAD DATA 2 ADDRESS
         L     R7,0(,R7)               LOAD FULL WORD EVENT NAME
         ST    R7,CE1FAC+4             EVENT NAME BASED ON DATA 2
         MVC   CE1FAC(2),CE1SSU        EVENT NAME BASED ON SSU ID
         MVC   CE1FAC+2(2),CE1SSU
         POSTC LEVEL=DC,TYPE=CNT,NFOUND=QEAA3100
         B     QEAA9000                GO TO COMMON EXIT ROUTINE
         SPACE 2
QEAA3100 DS    0H
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAA9000                YES - GO EXIT, DON'T PRINT MSG
         WTOPC TEXT='........ RECEIVED AN UNSOLICITED ACKNOWLEDGEMENT',X
               COMP=YES,PREFIX=QEAA,LET=E,TIME=YES,NUM=01,CHAIN=YES,   X
               SUB=(CHARA,MUSNAM),ROUT=PRC,ENDOFM=NO
         WTOPC TEXT='                   EVENT NUMBER: ........, SOURCE X
               SYSTEM: ........',SUB=(HEX4A,CE1FAC+4,CHARA,MUSSYSA),   X
               HEADER=NO,ROUT=PRC,CHAIN=YES,ENDOFM=YES,COMP=NO
         B     QEAA9000                GO TO COMMON EXIT ROUTINE
         EJECT
***************************************************************
* THE MESSAGE HAS BEEN IDENTIFIED AS BEING RECEIVED UNSOLICITED
* FROM ANOTHER USER, SO AN ACKNOWLEDGEMENT MUST BE SENT BACK.
* THE DRX WILL BE KICKED OFF ONCE FOR EACH MESSAGE SENT, BUT
* ONLY ONE ACKNOWLEDGEMENT SHOULD BE SENT BACK.  TO ACCOMPLISH
* THIS, ANOTHER EVENT WILL BE USED.
*
* THIS ECB WILL TRY TO ESTABLISH AN EVENT.  IF SUCCESSFUL, WE
* CAN ASSUME IT IS THE FIRST ONE AMONG IT'S GROUP TO BE
* RECEIVED.  WE WILL WAIT FOR THE EVENT TO BE POSTED BY THE
* OTHER ECB'S IN THE GROUP, AND THEN SEND THE ACKNOWLEDGEMENT.
*
* IF THE EVENT IS ALREADY ESTABLISHED, IT MEANS ANOTHER ECB
* RECEIVED THE MESSAGE FIRST.  SIMPLY POST THE EVENT, WHICH WILL
* INFORM THE OTHER ECB ANOTHER MESSAGE IN THE GROUP HAS
* ARRIVED.
*
* THE EVENT NAME WILL BE BASED ON THE FIRST FULLWORD IN DATA 2,
* THE SUBSYSTEM-USER ID, AND A UNIQUE IDENTIFIER.  THE
* IDENTIFIER IS USED TO MAKE THE EVENT NAMES DIFFERENT FROM
* THOSE IN QEA3.
*
* THE NUMBER OF MESSAGES IN THE GROUP IS IN DATA 1.
*
* IF ONLY ONE MESSAGE WAS SENT, EVENT PROCESSING WILL BE
* SKIPPED, AND THE ACKNOWLEDGEMENT WILL BE SENT IMMEDIATELY.
***************************************************************
         SPACE 2
QEAA4000 DS    0H                      CREATE THE UNIQUE EVENT NAME
         L     R7,CE1CR1               LOAD DATA 2 ADDRESS
         MVC   CE1FAC(2),=C'KJ'        NAME BASED ON UNIQUE IDENTIFIER
         MVC   CE1FAC+2(2),CE1SSU      NAME BASED ON ECB'S SSU ID
         MVC   CE1FAC+4(4),0(R7)       NAME BASED ON DATA IN DATA 2
         SPACE 2
* THE NUMBER OF POSTC MACROS NEEDED TO COMPLETE THE EVENT IS ONE LESS
* THAN THE NUMBER OF MESSAGES IN THE GROUP.  THE FIRST MESSAGE WILL
* CREATE AND WAIT ON THE EVENT, SO IT WILL NOT ISSUE A POSTC.
         LH    R1,1(,R6)               NUMBER MSGS SENT IS IN DATA 1
         BCTR  R1,0                    NUM POST'S = NUMBER MSGS MINUS 1
         STH   R1,CE1CRC               SAVE NUMBER OF POSTC'S NEEDED
         SPACE 2
* THE TIMEOUT VALUE WILL BE ONE SECOND FOR EACH EXPECTED POSTC, WITH A
* MINIMUM OF 3 MINUTES, AND A MAXIMUM OF 10 MINUTES.
         LTR   R2,R1                   TIMEOUT VALUE = NUMBER POST'S
         BZ    QEAA5000                BRANCH IF ONLY ONE MESSAGE SENT
         CL    R2,=F'180'              TIMEOUT LESS THAN 3 MINUTES?
         BNL   QEAA4100                NO  - BRANCH AND CONTINUE
         LA    R2,180                  YES - GIVE AT LEAST 3 MINUTES
         B     QEAA4200                BRANCH AND ESTABLISH EVENT
QEAA4100 DS    0H
         CL    R2,=F'600'              TIMEOUT MORE THAN 10 MINUTES?
         BL    QEAA4200                NO  - GO ESTABLISH EVENT
         LA    R2,600                  YES - ENFORCE 10 MINUTE LIMIT
         SPACE 2
* TRY TO CREATE THE EVENT, AND IF SUCCESSFUL, WAIT FOR IT TO COMPLETE.
* IF THE EVENT ALREADY EXISTS, ISSUE A POSTC AND EXIT.
QEAA4200 DS    0H
         EVNTC LEVEL=DC,TYPE=CNT,TIMEOUT=(R2),                         X
               NAME=Y,DUPNAM=QEAA4300,STATE=1052
         EVNWC LEVEL=DC,TYPE=CNT,ERROR=QEAA8000,NFOUND=QEAA8100
         B     QEAA5000                GO SEND ACKNOWLEDGEMENT
         SPACE 2
QEAA4300 DS    0H                      EVENT ALREADY ESTABLISHED
         POSTC LEVEL=DC,TYPE=CNT,NFOUND=QEAA8100
         B     QEAA9000                GO TO COMMON EXIT ROUTINE
         EJECT
***************************************************************
* THE MESSAGE IN DATA 2 (ON DATA LEVEL 1) WILL BE USED TO SEND
* THE ACKNOWLEDGEMENT.  THIS WILL ALLOW THE USER WHO RECEIVES
* IT TO IDENTIFY WHICH MESSAGE IS BEING ACKNOWLEDGED.
*
* IF THE MPIFC GIVES A RETURN CODE OF 4 OR 12, THE ECB WILL BE
* PUT ON THE INPUT LIST.  WHEN IT IS REACTIVATED, THE MPIFC
* WILL BE TRIED AGAIN, HOPING THAT ENOUGH RESOURCES ARE NOW
* FREE.
***************************************************************
         SPACE 2
QEAA5000 DS    0H
         LA    R5,EBX000               LOAD MPIFC PARAMETER LIST ADDR.
         XC    0(SNRPLEN,R5),0(R5)     CLEAR MPIFC PARAMETER LIST
         MVC   SNCONTOK,MUSCTKA        INDICATE THE CONNECTION'S CONTOK
         MVC   SNDSNAM,MUSSYSA         INDICATE THE DESTINATION SYSTEM
         MVI   0(R6),X'80'             INDICATE THIS IS ACKNOWLEDGEMENT
         ST    R6,SNDATA1              INDICATE THE DATA 1 ADDRESS
         MVC   SNDBSZ1,=AL2(12)        INDICATE THE SIZE OF DATA 1
         MVC   SNDATA2(8),CE1CR1       RETURN THE BLOCK ON D1 AS DATA 2
         MVC   SNDLVL2,=AL1(D1)        INDICATE DATA LEVEL OF DATA 2
         SPACE 2
QEAA5100 DS    0H
         MPIFC RTYPE=SEND,REG=R5       ISSUE THE MPIFC SEND
         B     QEAA5200(R15)           BRANCH BASED ON THE RETURN CODE
QEAA5200 DS    0H
         B     QEAA5300                 0 - NORMAL COMPLETION
         B     QEAA5400                 4 - MPIF OUTPUT QUEUE EXCEEDED
         B     QEAA8200                 8 - SENDING OVER INACTIVE PATH
         B     QEAA5400                12 - NO CONTROL STORAGE AVAIL.
         B     QEAA8300                16 - NO PACING CREDITS AVAIL.
         B     QEAA8300                20 - INVALID PARAMETER
         SPACE 2
QEAA5300 DS    0H                      CODE 0  - NORMAL COMPLETION
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAA9000                YES - EXIT, DON'T PRINT MESSAGE
         WTOPC TEXT='ACKNOWLEDGEMENT SUCCESSFULLY SENT BY ........',   X
               PREFIX=QEAA,NUM=02,LET=I,ROUT=PRC,CHAIN=YES,ENDOFM=NO,  X
               COMP=YES,SUB=(CHARA,MUSNAM)
         WTOPC TEXT='                   EVENT NUMBER: ........, DESTINAX
               TION SYSTEM: ........',HEADER=NO,ROUT=PRC,CHAIN=YES,    X
               SUB=(HEX4A,CE1FAC+4,CHARA,MUSSYSA),ENDOFM=YES,COMP=NO
         B     QEAA9000                GO TO COMMON EXIT ROUTINE
         SPACE 2
QEAA5400 DS    0H                      CODE 4 OR 12 - NO MPIF RESOURCES
         DLAYC                         WAIT FOR RESOURCES TO FREE UP
         B     QEAA5100                TRY SEND ACKNOWLEDGEMENT AGAIN
         EJECT
***********************************************************************
* ERROR ROUTINES.
***********************************************************************
         SPACE 2
QEAA8000 DS    0H                      EVNWC ENDED IN ERROR
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAA9000                YES - EXIT, DON'T PRINT THE MSG
         WTOPC TEXT='........ UNABLE TO SEND MESSAGE ACKNOWLEDGEMENT', X
               PREFIX=QEAA,NUM=03,LET=E,ROUT=PRC,CHAIN=YES,ENDOFM=NO,  X
               COMP=YES,SUB=(CHARA,MUSNAM)
         WTOPC TEXT='                   EVENT NUMBER: ........, DESTINAX
               TION SYSTEM: ........',HEADER=NO,ROUT=PRC,CHAIN=YES,    X
               SUB=(HEX4A,CE1FAC+4,CHARA,MUSSYSA),ENDOFM=YES,COMP=NO
         B     QEAA9000                GO TO THE COMMON EXIT ROUTINE
         SPACE 2
***********************************************************************
         SPACE 2
QEAA8100 DS    0H                      NFOUND BRANCH FROM EVNWC/POSTC
         LA    R0,EEA3MSG              WILL APPEND MESSAGE TO DUMP
         SERRC R,EEA003,MSG,OP,ECB     DUMP, EVNWC/POSTC NAME NOT FOUND
         B     QEAA9000                GO TO THE COMMON EXIT ROUTINE
         SPACE 2
EEA3MSG  DS    0D
         DC    AL1(ENDEEA3-BEGEEA3)
BEGEEA3  DC    C'EVENT NAME USED BY EVNWC OR POSTC NOT FOUND'
ENDEEA3  EQU   *
         SPACE 2
***********************************************************************
         SPACE 2
QEAA8200 DS    0H                      CODE 8 - PATH UNAVAILABLE
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAA9000                YES - EXIT, DON'T PRINT MESSAGE
         SPACE 2
QEAA8300 DS    0H                      CODE 16 OR 20 - PRINT ERROR MSG
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC SEND RETURN CODE .... FOR ........',        X
               COMP=YES,PREFIX=QEAA,LET=E,TIME=YES,NUM=04,ENDOFM=NO,   X
               SUB=(DEC,(R6),CHARA,MUSNAM),ROUT=PRC,CHAIN=YES
         WTOPC TEXT='                   EVENT NUMBER: ........, DESTINAX
               TION SYSTEM: ........',HEADER=NO,ROUT=PRC,CHAIN=YES,    X
               SUB=(HEX4A,CE1FAC+4,CHARA,MUSSYSA),ENDOFM=YES,COMP=NO
         B     QEAA9000                GO EXIT THIS SEGMENT
         SPACE 2
***********************************************************************
         EJECT
***************************************************************
* COMMON EXIT ROUTINE.  INDICATE THE ECB IS NO LONGER ACTIVE,
* AND EXIT.
***************************************************************
         SPACE 2
QEAA9000 DS    0H
         CORHC (R3)                    LOCK SECONDARY BLOCK TO UPDATE
         L     R15,MUSECB              LOAD NUMBER USER'S ACTIVE ECB'S
         BCTR  R15,0                   ONE LESS ECB IS ACTIVE
         ST    R15,MUSECB              NEW NUMBER USER'S ACTIVE ECB'S
         CORUC (R3)                    UNLOCK THE SECONDARY BLOCK
         EXITC
         EJECT
         LTORG
         FINIS QEAA
         END   ,
