         PRINT NOGEN
         BEGIN NAME=QEAB,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEAB                                       *
* RELATED MODULES. QEA0-QEA6, QEAA, QEAC-QEAF                 *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF TEST DRIVER'S DIRECTORY UPDATE EXIT   *
*                  (DUX)                                      *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS IS THE MPIF TEST DRIVER'S DIRECTORY        *
*             UPDATE EXIT (DUX).  IT IS INVOKED FOR A USER    *
*             WHEN ANOTHER USER WITH THE SAME NAME HAS        *
*             IDENTIFIED ITSELF TO THE SYSTEM.  THIS USER     *
*             WILL TRY TO ESTABLISH A CONNECTION WITH THE     *
*             OTHER USER.                                     *
*                                                             *
*             THIS EXIT WILL ALSO BE KICKED AFTER A RETURN    *
*             FROM AN ECHO CHECK FAILURE.  THEREFORE, PATH    *
*             START-UP MUST BE CONSIDERED.                    *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBW020 - DESTINATION SYSTEM SAVE AREA *
*                       EBW060 - TOD CLOCK SAVE AREA          *
*                       EBW070 - TOD CLOCK SAVE AREA          *
*                       EBX000 - MPIFC PARAMETER LIST         *
*                       CE1CR6 - QUERY REPLY AREA             *
*                                                             *
* REGISTER-CONVENTIONS..R1 - DCTMGT BASE REGISTER             *
*                       R2 - SD0RV  BASE REGISTER             *
*                       R4 - DCTMUS BASE REGISTER             *
*                       R5 - DCTMUP BASE REGISTER             *
*                       R6 - DCTMUS BASE REGISTER, SUFFIX=A   *
*                       R7 - NUMBER OF AVAILABLE PATHS TO THE *
*                            OTHER SYSTEM THE USER IS NOT     *
*                            ALREADY CONNECTED OVER.          *
*                                                             *
***************************************************************
*                                                             *
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - THIS EXIT CAN BE INITIATED ON ANY         *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * NONE                 *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * R1 - DUX PARAMETER   * NONE                 *
*               *      LIST            *                      *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  MGT - MPIF GLOBAL TABLE                  *
*                    @ISSDRV (DRIVER GLOBALS)                 *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. EXITC                                         *
*     -ERROR... SERRC EEA005 - TOD CLOCK ERROR                *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* THIS EXIT WILL BE KICKED OFF AFTER THE RETURN FROM AN ECHO
* CHECK FAILURE, WHICH ALSO INCLUDES PATH START-UP.  SINCE
* THE EXIT CAN BE KICKED OFF BEFORE THE PATHS HAVE COMPLETED
* START-UP, IT WILL DELAY PROCESSING TO GIVE START-UP TIME TO
* FINISH.
***************************************************************
         SPACE 2
         STCK  EBW060                  SAVE THE CURRENT TIME
         BNZ   QEAB1100                BRANCH IF COULDN'T SAVE TIME
         L     R15,EBW060              LOAD THE CURRENT TIME
         AL    R15,=F'3'               ADD 3 SECONDS TO CURRENT TIME
         ST    R15,EBW060              STORE THIS NEW "GOAL" TIME
         SPACE 2
QEAB1000 DS    0H
         DLAYC                         DELAY PROCESSING
         STCK  EBW070                  STORE NEW CURRENT TIME
         BNZ   QEAB1100                BRANCH IF COULDN'T SAVE TIME
         CLC   EBW070(4),EBW060        CURRENT TIME > "GOAL" TIME?
         BNH   QEAB1000                NO  - BRANCH, DELAY AGAIN
         B     QEAB2000                YES - BRANCH AND CONTINUE
         SPACE 2
QEAB1100 DS    0H
         LA    R0,EEA5MSG              WILL APPEND MESSAGE TO DUMP
         SERRC E,EEA005,MSG,CP,ECB     CLOCK ERROR - DUMP AND EXIT
EEA5MSG  DS    0D
         DC    AL1(ENDEEA5-BEGEEA5)
BEGEEA5  DC    C'ERROR STORING TOD CLOCK'
ENDEEA5  EQU   *
         EJECT
***************************************************************
* DO SOME PRELIMINARY PROCESSING, ESTABLISH DSECTS, AND LOAD
* BASE REGISTERS.  THE UITOK PASSED IN THE DUX PARAMETER LIST
* IS THE ADDRESS OF THE USER'S SECONDARY BLOCK, WHICH WILL BE
* LOCKED FOR THIS ENTIRE SEGMENT.
***************************************************************
         SPACE  2
QEAB2000 DS     0H
         DCTMGT REG=R1                 MPIF GLOBAL TABLE (MGT) DSECT
         DCTMUP REG=R5,LTYPE=ALL       MPIFC PARAMETER DSECTS
         DCTMUS REG=R4                 DRIVER SECONDARY BLOCK DSECT
         DCTMUS REG=R6,SUFFIX=A        DSECT CONNECTION SLOTS WITHIN --
*                                            --- THE SECONDARY BLOCK
         SPACE  2
         LR     R5,R1                  LOAD DUX PARAMETER LIST ADDRESS
         MVC    EBW020(8),DUEDSNAM     SAVE NAME OF OTHER USER'S SYSTEM
         L      R4,DUEUIT              LOAD SECONDARY BLOCK ADDRESS
         CORHC  (R4)                   LOCK THE SECONDARY BLOCK
         SPACE  2
         SD0RV  REG=R2                 DRIVER @ISSDRV DSECT
         GLOBZ  REGR=R14               GET GLOBAL 1 ADDRESS AND DSECT
         L      R2,@ISSDRV             LOAD @ISSDRV BASE REGISTER
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE OUTPUT MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         EJECT
***************************************************************
* THE SYSTEM NAME OF THE USER JUST IDENTIFIED MUST BE COMPARED
* TO THE USER BEING NOTIFIED.
*
* IF THE SYSTEM NAMES ARE THE SAME, AND THE USER BEING NOTIFIED
* IS A DEVICE=MTOM TYPE USER, THEN THE TWO USERS WILL CONNECT.
*
* IF THE SYSTEM NAMES ARE DIFFERENT, AND THE USER BEING
* NOTIFIED HAS DEVICE=CTOC, AND THIS SYSTEM HAS A LARGER
* ALPHABETIC NAME, THEN THE USER'S WILL CONNECT.  THIS SYSTEM
* MUST HAVE THE LARGER NAME SO ONLY ONE OF THE SYSTEMS WILL
* ISSUE THE MPIFC CONNECT REQUESTS.
***************************************************************
         SPACE 2
         CINFC R,CMMPIF,F,REG=R1,BSS=YES    LOAD MGT BASE REGISTER
         SPACE 2
         CLI   MUSDEV,DEVCTOC          IS THIS USER A DEV = CTOC USER?
         BNE   QEAB3000                NO - BRANCH, THIS IS MTOM USER
         CLC   MGTMSSN(8),EBW020       THIS SYSTEM > OTHER SYSTEM?
         BH    QEAB4000                YES - GO MAKE CONNECTIONS
         CORUC (R4)                    NO  - UNLOCK THE SECONDARY BLOCK
         EXITC                         DO NOT MAKE CONNECTIONS
         SPACE 2
QEAB3000 DS    0H                      TRY AND MAKE MTOM CONNECTION
         CLC   MGTMSSN,EBW020          SAME SYSTEM NAMES?
         BE    QEAB4000                YES - GO MAKE CONNECTION
         CORUC (R4)                    NO  - UNLOCK THE SECONDARY BLOCK
         EXITC                         DO NOT MAKE CONNECTION
         EJECT
***************************************************************
* BEFORE TRYING TO ESTABLISH THE CONNECTION A MPIFC QUERY MUST
* BE ISSUED TO FIND OUT WHAT PATHS ARE AVAILABLE TO THE OTHER
* SYSTEM.  THIS LIST WILL BE RETURNED ON DATA LEVEL 6, AND WILL
* BE SEARCHED TO SEE WHICH PATHS HAVE THE CORRECT CLASS.  THE
* COUNT OF THESE PATHS WILL BE STORED IN R7.
***************************************************************
         SPACE 2
QEAB4000 DS    0H
         GETCC D6,L2                   BLOCK FOR QUERY REPLY AREA
         LA    R5,EBX000               MPIFC PARAMETER LIST ADDRESS
         XC    0(QURPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVI   QUOPTS1,QUP1PTHL        WILL QUERY FOR A LIST OF PATHS
         MVC   QUIDTOK(4),MUSITK       INDICATE THE USER'S IDTOK
         MVC   QUCLASS(1),MUSPCL       INDICATE THE USER'S PATH CLASS
         MVC   QURUNAM(8),MUSNAM       NAME OF THE REQUESTING USER
         MVC   QUDSNAM(8),EBW020       NAME OF THE DESTINATION SYSTEM
         L     R15,CE1CR6              INDICATE THE REPLY AREA ADDRESS
         ST    R15,QUFBAA              STORE THE REPLY AREA ADDRESS
         LA    R15,1055                LOAD LENGTH OF REPLY AREA
         ST    R15,QUFBAL              STORE THE LENGTH IN PARAM. LIST
         MPIFC RTYPE=QUERY,REG=R5      ISSUE THE MPIFC QUERY
         LTR   R15,R15                 WAS THE QUERY SUCCESSFUL?
         BNZ   QEAB9000                NO  - BRANCH, ISSUE ERROR MSG
         SPACE 2
         L     R5,CE1CR6               LOAD REPLY AREA BASE REGISTER
         LH    R15,QRACT               LOAD NUMBER OF ALL AVAIL. PATHS
         LTR   R15,R15                 ARE THERE ANY PATHS AVAILABLE?
         BZ    QEAB9100                NO  - GO ISSUE ERROR MESSAGE
         SR    R7,R7                   R7 WILL HOLD NUMBER PATHS ---
*                                      --- WITH THE CORRECT PATH CLASS
         SPACE 2
QEAB4100 DS    0H
         CLC   QRPCLSI(1),MUSPCL       PATH CLASS = USER'S CLASS?
         BNE   QEAB4200                NO  - BRANCH, CHECK NEXT PATH
         LA    R7,1(,R7)               INCREMENT COUNT OF MATCHED PATHS
QEAB4200 DS    0H
         LA    R5,QRALEN5(,R5)         POINT TO NEXT PATH IN LIST
         BCT   R15,QEAB4100            LOOP IF MORE PATHS TO CHECK
         SPACE 2
         LTR   R7,R7                   ARE THERE ANY PATHS AVAILABLE?
         BZ    QEAB9100                NO  - GO ISSUE ERROR MESSAGE
         EJECT
***************************************************************
* THE COUNT OF AVAILABLE PATHS TO THE OTHER SYSTEM IS IN R7.
* HOWEVER, IT IS POSSIBLE THE USER IS ALREADY CONNECTED OVER
* SOME OR ALL OF THESE PATHS.  FOR EXAMPLE, ON THE RETURN FROM
* AN ECHO CHECK FAILURE, BOTH THE DUX AND THE PAX ARE INVOKED.
* IF THE PAX GETS CONTROL FIRST, IT WILL CONNECT OVER ALL
* PATHS, AND THIS SEGMENT WILL HAVE NOTHING TO DO.  THE
* SECONDARY BLOCK WILL BE SEARCHED, AND R7 WILL BE DECREMENTED
* ONCE FOR EACH CONNECTION THAT ALREADY EXISTS.
*
* NOTE: FOR A MTOM USER BOTH 'ENDS' OF THE CONNECTION ARE IN
* THIS SECONDARY BLOCK, AND R7 WILL BE DECREMENTED ONCE FOR
* EVERY TWO CONNECTION SLOTS.
***************************************************************
         SPACE 2
         LH    R14,MUSNCN              CURRENT NUMBER OF CONNECTIONS
         LTR   R14,R14                 USER HAVE ANY CONNECTIONS?
         BZ    QEAB6000                NO  - NO NEED TO SEARCH BLOCK
         SPACE 2
         LA    R6,MUSHLN(,R4)          POINT TO FIRST CONNECT SLOT
         LA    R14,MUSMAX              MAXIMUM NUMBER CONNECT SLOTS
         SR    R15,R15                 CLEAR A REGISTER TO COUNT WITH
         SPACE 2
* THIS CONNECTION'S SYSTEM EQUAL TO SYSTEM WANT TO CONNECT TO?
QEAB5000 DS    0H
         TM    MUSINDA,MUSUSE          IS THIS CONNECT SLOT IN USE?
         BZ    QEAB5100                NO  - BRANCH AND CONTINUE SEARCH
         CLC   MUSSYSA(8),EBW020       SYSTEM NAMES EQUAL?
         BNE   QEAB5100                NO  - BRANCH AND CONTINUE SEARCH
         LA    R15,1(,R15)             YES - FOUND ONE MORE CONNECTION
QEAB5100 DS    0H
         LA    R6,MUSCLN(,R6)          POINT TO NEXT CONNECTION SLOT
         BCT   R14,QEAB5000            BRANCH IF MORE SLOTS TO CHECK
         SPACE 2
* IF THIS A MTOM USER R7 WILL BE DECREMENTED ONCE FOR EVERY 2 ACTIVE
* CONNECTIONS FOUND IN THE SECONDARY BLOCK.
         CLI   MUSDEV,DEVCTOC          IS THIS A CTOC TYPE USER?
         BE    QEAB5200                YES  - NO NEED TO ADJUST R15
         SRL   R15,2                   MTOM - ADJUST NUMBER CONNECTIONS
         SPACE 2
* ADJUST R7 SO IT CONTAINS THE NUMBER OF AVAILABLE PATHS THE USER IS
* NOT ALREADY CONNECTED OVER.  IF R7 IS ZERO THEN EXITC, BECAUSE THERE
* ARE NO AVAILABLE PATHS.
QEAB5200 DS    0H
         SR    R7,R15                  ADJUST THE NUMBER OF PATHS
         BP    QEAB6000                BRANCH IF PATHS TO CONNECT OVER
         RELCC D6                      RELEASE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT - NO PATHS TO CONNECT OVER
         EJECT
***************************************************************
* FOR EACH AVAILABLE PATH, A CONNECT SLOT IN THE SECONDARY
* BLOCK WILL BE CREATED, AND THE CONNECT SUBROUTINE WILL BE
* CALLED.  IF THE CONNECTION WILL CAUSE THE USER TO EXCEED IT'S
* MAXIMUM NUMBER OF CONNECTIONS, THE REQUEST WILL NOT BE MADE.
***************************************************************
         SPACE 2
QEAB6000 DS    0H
         LA    R14,MUSMAX              LOAD MAXIMUM NUMBER CONNECTIONS
         CH    R14,MUSNCN              WILL EXCEED MAX NUM CONNECTIONS?
         BH    QEAB6100                NO  - BRANCH AND CONTINUE
         WTOPC TEXT='CONNECT REQUEST SUPPRESSED FOR ........ TO SYSTEM X
               ........',COMP=YES,SUB=(CHARA,MUSNAM,CHARA,EBW020),     X
               PREFIX=QEAB,NUM=01,LET=E,ROUT=PRC
         B     QEAB6400                BRANCH AND CONTINUE
         SPACE 2
QEAB6100 DS    0H
         LA    R6,MUSHLN(,R4)          POINT TO FIRST CONNECT SLOT
QEAB6200 DS    0H                      SEARCH FOR FREE CONNECT SLOT
         TM    MUSINDA,MUSUSE          IS THE SLOT IN USE?
         BZ    QEAB6300                NO  - USE THIS ENTRY
         LA    R6,MUSCLN(,R6)          YES - POINT TO THE NEXT SLOT
         B     QEAB6200                CHECK IF NEXT SLOT IS IN USE
         SPACE 2
QEAB6300 DS    0H
         XC    0(MUSCLN,R6),0(R6)      CLEAR THE CONNECT SLOT
         ST    R7,EBW036               SAVE R7 ACROSS SUBROUTINE CALL
         BAS   R7,QEAB7000             CALL THE CONNECT SUBROUTINE
         L     R7,EBW036               RESTORE R7
         SPACE 2
         OI    MUSINDA,MUSUSE          INDICATE CONNECT SLOT IS IN USE
         OI    MUSINDA,MUSINC          CONNECTION IS INCOMPLETE
         MVC   MUSSYSA,EBW020          SAVE DEST. SYSTEM IN SEC. BLOCK
         SPACE 2
         LH    R6,MUSNCN               LOAD NUMBER USER'S CONNECTIONS
         LA    R6,1(,R6)               INCREMENT THAT NUMBER
         STH   R6,MUSNCN               SAVE NEW NUMBER OF CONNECTIONS
         SPACE 2
         LH    R6,MUSICON              NUMBER OF INCOMPLETE CONNECTIONS
         LA    R6,1(,R6)               INCREMENT THAT NUMBER
         STH   R6,MUSICON              NEW NUMBER OF INCOMPLETE CONN.
         SPACE 2
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAB6400                YES - DON'T PRINT ERROR MESSAGE
         WTOPC TEXT='SUCCESSFUL CONNECT REQUEST FOR ........ TO SYSTEM X
               ........',COMP=YES,SUB=(CHARA,MUSNAM,CHARA,EBW020),     X
               PREFIX=QEAB,NUM=02,LET=I,ROUT=PRC
         SPACE 2
QEAB6400 DS    0H
         BCT   R7,QEAB6000             BRANCH IF THERE ARE MORE PATHS
         RELCC D6                      RELEASE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
***************************************************************
* THIS ROUTINE ISSUES THE MPIFC CONNECT REQUESTS.  THE UCTOK
* WILL BE THE ADDRESS OF THE CONNECTION SLOT.  THE UCTOK WILL
* ALSO BE PASSED IN THE USER DATA AREA (CNDATA).  THIS ALLOWS
* MTOM CONNECTIONS TO HAVE 'MATCHED' CONNECTION SLOTS LINKED
* TOGETHER (MTOM USERS HAVE 2 CONNECTION SLOTS FOR EACH
* CONNECTION).  THE LINK IS NECESSARY SO OTHER SEGMENTS DO NOT
* PERFORM THE SAME SERVICE TWICE ON A CONNECTION.  FOR EXAMPLE,
* QEA3 USES THE LINK TO KEEP FROM SENDING THE SAME MESSAGES
* TWICE OVER THE SAME CONNECTION.
***************************************************************
         SPACE 2
QEAB7000 DS    0H
         LA    R5,EBX000               MPIFC PARAMETER LIST ADDRESS
         XC    0(CNRPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVC   CNPCLASS,MUSPCB         COPY THE PATH CLASS
         MVI   CNOPTS1,CNMPATH         INDICATE MULTI-PATHING
         MVC   CNIDTOK,MUSITK          COPY THE USER'S IDTOK
         MVC   CNOUNAM,MUSNAM          COPY THE USER'S NAME
         ST    R6,CNUCTOK              SAVE THE USER'S UCTOK
         MVC   CNDUNAM,MUSNAM          INDICATE DESTINATION USER'S NAME
         MVC   CNDSNAM,EBW020          DESTINATION SYSTEM NAME
         MVC   CNDRXAD,QEABDRX         COPY DRX VECTOR INTO PARAM LIST
         MVC   CNMS,=AL4(4096)         MAX DATA REQUESTOR CAN SEND
         MVC   CNMR,=AL4(4096)         MAX DATA REQUESTOR CAN RECEIVE
         SR    R1,R1                   CLEAR ADDRESS REGISTER
         LA    R1,CNUCTOK              LOAD PTR TO THIS SIDE'S UCTOK
         ST    R1,CNDATA               UCTOK TO OTHER SIDE CONNECTION
         MVC   CNDATAL,=AL4(4)         LENGTH OF DATA IN CNDATA
         MPIFC RTYPE=CONNECT,REG=R5    ISSUE THE CONNECT REQUEST
         SPACE 2
         LTR   R15,R15                 WAS THE CONNECTION SUCCESSFUL?
         BZR   R7                      YES - RETURN TO CALLER
         B     QEAB9200                NO  - GO ISSUE ERROR MESSAGE
         SPACE 2
QEABDRX  SWISC QEAA,IS=R0,TYPE=ENTER   DATA RECEIVED EXIT VECTOR
         EJECT
***************************************************************
* ERROR MESSAGE ROUTINES.
***************************************************************
         SPACE 2
QEAB9000 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC QUERY RETURN CODE .... FOR ........',       X
               COMP=YES,PREFIX=QEAB,LET=E,TIME=YES,NUM=03,             X
               SUB=(DEC,(R6),CHARA,MUSNAM),ROUT=PRC
         RELCC D6                      RELEASE THE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEAB9100 DS    0H
         WTOPC TEXT='NO CLASS .. PATHS TO ........, NO CONNECTION MADE X
               FOR ........',COMP=YES,PREFIX=QEAB,LET=E,NUM=04,        X
               SUB=(CHARA,MUSPCL,CHARA,EBW020,CHARA,MUSNAM),TIME=YES,  X
               ROUT=PRC
         RELCC D6                      RELEASE THE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC
         SPACE 2
QEAB9200 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC CONNECT RETURN CODE .... FOR ........',     X
               COMP=YES,PREFIX=QEAB,LET=E,TIME=YES,NUM=05,             X
               SUB=(DEC,(R6),CHARA,MUSNAM),ROUT=PRC
         RELCC D6                      RELEASE THE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
         LTORG
         FINIS QEAB
         END   ,
