         PRINT NOGEN
         BEGIN NAME=QEAC,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEAC                                       *
* RELATED MODULES. QEA0-QEA6, QEAA-QEAB, QEAD-QEAF            *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF TEST DRIVER'S PATH ACTIVATION EXIT    *
*                  (PAX)                                      *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS IS THE MPIF TEST DRIVER'S PATH ACTIVATION  *
*             EXIT.  IT IS INVOKED FOR A USER WHEN A PATH IS  *
*             ACTIVATED TO A SYSTEM THE USER PREVIOUSLY       *
*             QUERIED.  IT WILL QUERY THE OTHER SYSTEM TO SEE *
*             IF THERE ARE ANY USERS TO CONNECT WITH.  IF SO, *
*             A CONNECTION WILL BE ESTABLISHED.               *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBW020 - DESTINATION SYSTEM SAVE AREA *
*                       EBW060 - TOD CLOCK SAVE AREA          *
*                       EBW070 - TOD CLOCK SAVE AREA          *
*                       EBX000 - MPIFC PARAMETER LIST         *
*                       CE1CR4 - QUERY REPLY AREA             *
*                                                             *
* REGISTER-CONVENTIONS..R1 - DCTMGT BASE REGISTER             *
*                       R2 - SD0RV  BASE REGISTER             *
*                       R4 - DCTMUS BASE REGISTER             *
*                       R5 - DCTMUP BASE REGISTER             *
*                       R6 - DCTMUS BASE REGISTER, SUFFIX=A   *
*                       R7 - NUMBER OF PATHS WITH THE CORRECT *
*                            CLASS TO THE OTHER SYSTEM.       *
*                                                             *
***************************************************************
*                                                             *
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - THIS EXIT CAN BE INITIATED ON ANY         *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * NONE                 *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * R1 - PAX PARAMETER   * NONE                 *
*               *      LIST            *                      *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  MGT - MPIF GLOBAL TABLE                  *
*                    @ISSDRV (DRIVER GLOBALS)                 *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. EXITC                                         *
*     -ERROR... SERRC EEA005 - TOD CLOCK ERROR                *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* SINCE PATH START-UP CAN COMPLETE ASYNCHRONOUSLY TO THIS EXIT
* BEING KICKED OFF, PROCESSING WILL BE DELAYED.  THIS WILL GIVE
* START-UP PROCESSING TIME TO COMPLETE.
***************************************************************
         SPACE 2
         STCK  EBW060                  SAVE THE CURRENT TIME
         BNZ   QEAC1100                BRANCH IF COULDN'T SAVE TIME
         L     R15,EBW060              LOAD THE CURRENT TIME
         AL    R15,=F'6'               ADD 6 SECONDS TO CURRENT TIME
         ST    R15,EBW060              STORE THIS NEW "GOAL" TIME
         SPACE 2
QEAC1000 DS    0H
         DLAYC                         DELAY PROCESSING
         STCK  EBW070                  STORE NEW CURRENT TIME
         BNZ   QEAC1100                BRANCH IF COULDN'T SAVE TIME
         CLC   EBW070(4),EBW060        CURRENT TIME > "GOAL" TIME?
         BNH   QEAC1000                NO  - BRANCH, DELAY AGAIN
         B     QEAC2000                YES - BRANCH AND CONTINUE
         SPACE 2
QEAC1100 DS    0H
         LA    R0,EEA5MSG              WILL APPEND MESSAGE TO DUMP
         SERRC E,EEA005,MSG,CP,ECB     CLOCK ERROR - DUMP AND EXIT
EEA5MSG  DS    0D
         DC    AL1(ENDEEA5-BEGEEA5)
BEGEEA5  DC    C'ERROR STORING TOD CLOCK'
ENDEEA5  EQU   *
         EJECT
***************************************************************
* DO SOME PRELIMINARY PROCESSING, ESTABLISH DSECTS, AND LOAD
* BASE REGISTERS.  THE UITOK RETURNED IN THE EXIT PARAMETER
* LIST IS THE ADDRESS OF THIS USER'S SECONDARY BLOCK.  THE
* SECONDARY BLOCK WILL BE LOCKED FOR THIS ENTIRE SEGMENT.
***************************************************************
         SPACE  2
QEAC2000 DS     0H
         DCTMGT REG=R1                 MPIF GLOBAL TABLE (MGT) DSECT
         DCTMUP REG=R5,LTYPE=ALL       MPIFC PARAMETER DSECTS
         DCTMUS REG=R4                 DRIVER SECONDARY BLOCK DSECT
         DCTMUS REG=R6,SUFFIX=A        DSECT CONNECTION SLOTS WITHIN --
*                                         THE SECONDARY BLOCK
         SPACE  2
         LR     R5,R1                  LOAD PAX PARAMETER LIST ADDRESS
         MVC    EBW020(8),PAESNAME     SAVE OTHER SYSTEM'S NAME IN ECB
         L      R4,PAEUIT              LOAD SECONDARY BLOCK BASE REG.
         CORHC  (R4)                   LOCK THE SECONDARY BLOCK
         SPACE  2
         SD0RV  REG=R2                 DRIVER @ISSDRV DSECT
         GLOBZ  REGR=R14               GET GLOBAL 1 ADDRESS
         L      R2,@ISSDRV             LOAD @ISSDRV BASE REGISTER
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE OUTPUT MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         EJECT
***************************************************************
* THE NAME OF THE SYSTEM ON THE OTHER END OF THE NEW PATH IS
* COMPARED TO THE NAME OF THIS SYSTEM.  IF THEY ARE THE SAME,
* AND THIS IS A DEV=MTOM USER, A CONNECTION WILL BE MADE OVER
* THE NEW PATH.
*
* IF THE NAMES ARE DIFFERENT, THE USER HAS DEV=CTOC, AND THIS
* SYSTEM HAS THE LARGER ALPHABETIC NAME, A CONNECTION WILL BE
* MADE OVER THE NEW PATH.  THIS SYSTEM MUST HAVE THE 'LARGER'
* NAME SO ONLY ONE USER ON EITHER END OF THE PATH WILL ISSUE
* THE CONNECT REQUEST.
***************************************************************
         SPACE 2
         CINFC R,CMMPIF,F,REG=R1,BSS=YES    LOAD MGT BASE REGISTER
         SPACE 2
         CLI   MUSDEV,DEVCTOC          IS THIS USER A DEV = CTOC USER?
         BNE   QEAC3000                NO - BRANCH, THIS IS MTOM USER
         CLC   MGTMSSN(8),EBW020       THIS SYSTEM > OTHER SYSTEM?
         BH    QEAC4000                YES - GO MAKE CONNECTIONS
         CORUC (R4)                    NO  - UNLOCK THE SECONDARY BLOCK
         EXITC                         DO NOT MAKE CONNECTIONS
         SPACE 2
QEAC3000 DS    0H                      TRY AND MAKE MTOM CONNECTION
         CLC   MGTMSSN(8),EBW020       SAME SYSTEM NAMES?
         BE    QEAC4000                YES - GO MAKE CONNECTION
         CORUC (R4)                    NO  - UNLOCK THE SECONDARY BLOCK
         EXITC                         DO NOT MAKE CONNECTION
         EJECT
***************************************************************
* QUERY THE OTHER SYSTEM TO SEE IF THERE IS A USER TO CONNECT
* WITH.
*
* THE REPLY AREA WILL RECEIVE A LIST OF ALL USER'S WITH THE
* SPECIFIED NAME.  THEREFORE, IT MUST BE SEARCHED TO SEE IF THE
* USER IS IDENTIFIED IN THE SYSTEM AT THE OTHER END OF THE
* ACTIVATED PATH.  IF THE USER IS NOT IN THE OTHER SYSTEM, NO
* CONNECTIONS ARE MADE.
***************************************************************
         SPACE 2
QEAC4000 DS    0H
         LA    R5,EBX000               PARAMETER LIST ADDRESS
         GETCC D4,L2                   BLOCK FOR QUERY REPLY AREA
         XC    0(QURPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVI   QUOPTS1,QUP1UNL         QUERY OF USERS IN ALL SYSTEMS
         MVC   QUIDTOK,MUSITK          PUT USER'S IDTOK IN PARAM. LIST
         MVC   QUCLASS(1),MUSPCL       PUT PATH CLASS IN PARAM. LIST
         MVC   QURUNAM,MUSNAM          NAME OF REQUESTING USER
         MVC   QUDUNAM,MUSNAM          NAME OF DESTINATION USER
         MVI   QUDSNAM,C'*'            QUERY ALL SYSTEMS IN COMPLEX
         LH    R14,CE1CC4              LOAD SIZE OF REPLY AREA
         ST    R14,QUFBAL              PUT THE SIZE IN PARAMETER LIST
         MVC   QUFBAA,CE1CR4           REPLY AREA ADDRESS IN PARAM LIST
         MPIFC RTYPE=QUERY,REG=R5      ISSUE THE QUERY
         LTR   R15,R15                 WAS THE QUERY SUCCESSFUL?
         BNZ   QEAC9000                NO  - BRANCH, ISSUE ERROR MSG
         SPACE 2
         L     R5,CE1CR4               REPLY AREA BASE REGISTER
         LH    R7,QRACT                NUMBER OF ELEMENTS IN REPLY AREA
QEAC4100 DS    0H
         CLC   QRUSYS(8),EBW020        IS THIS SYSTEM AT END OF PATH?
         BE    QEAC5000                YES - BRANCH AND CONTINUE
         LA    R5,QRALEN1(,R5)         NO  - BUMP TO NEXT ENTRY IN LIST
         BCT   R7,QEAC4100             BRANCH IF MORE USERS TO CHECK
         RELCC D4                      RELEASE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         NO USER IN OTHER SYSTEM - EXIT
         EJECT
***************************************************************
* WE HAVE DETERMINED THERE IS A USER IN THE OTHER SYSTEM WE
* WANT TO CONNECT WITH.  HOWEVER, WE MAY ALREADY BE CONNECTED
* OVER THE PATH THAT WAS JUST ACTIVATED.  FOR EXAMPLE, AFTER
* AN ECHO CHECK FAILURE, BOTH THE DUX AND PAX ARE INVOKED.  IF
* THE DUX RECEIVES CONTROL FIRST, IT WILL CONNECT OVER THE
* PATH, AND THIS SEGMENT WILL NOT HAVE TO DO ANYTHING.
*
* A MPIFC QUERY WILL BE ISSUED TO FIND OUT WHAT PATHS ARE
* AVAILABLE TO THE OTHER SYSTEM.  THIS LIST WILL BE SEARCHED TO
* SEE WHICH HAVE THE CORRECT PATH CLASS.  A COUNT OF THESE
* PATHS WILL BE STORED IN R7.
*
* NEXT, THE USER'S SECONDARY BLOCK WILL BE SEARCHED, AND R7
* WILL DECREMENTED ONCE FOR EACH CONNECTION ALREADY ESTABLISHED
* TO THE OTHER SYSTEM.  NOTE, FOR A MTOM USER BOTH 'ENDS' OF
* THE CONNECTION ARE IN THE SECONDARY BLOCK, SO R7 WILL BE
* DECREMENTED ONCE FOR EVERY TWO CONNECTIONS.
*
* IF R7 FINISHES WITH A NON-ZERO VALUE, MEANING THERE ARE PATHS
* TO THE OTHER SYSTEM WHICH ARE NOT CONNECTED, THEN A CONNECT
* REQUEST WILL BE ISSUED.
***************************************************************
         SPACE 2
QEAC5000 DS    0H
         LA    R5,EBX000               MPIFC PARAMETER LIST ADDRESS
         XC    0(QURPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVI   QUOPTS1,QUP1PTHL        WILL QUERY FOR LIST PATHS
         MVC   QUIDTOK(4),MUSITK       INDICATE USER'S IDTOK
         MVC   QUCLASS(1),MUSPCL       INDICATE THE PATH CLASS
         MVC   QURUNAM(8),MUSNAM       NAME OF REQUESTING USER
         MVC   QUDSNAM(8),EBW020       NAME OF DESTINATION SYSTEM
         LH    R14,CE1CC4              LOAD SIZE OF REPLY AREA
         ST    R14,QUFBAL              PUT THE SIZE IN PARAMETER LIST
         MVC   QUFBAA,CE1CR4           REPLY AREA ADDRESS IN PARAM LIST
         MPIFC RTYPE=QUERY,REG=R5      ISSUE THE MPIFC QUERY
         LTR   R15,R15                 WAS THE QUERY SUCCESSFUL?
         BNZ   QEAC9000                NO  - BRANCH, ISSUE ERROR MSG
         SPACE 2
         L     R5,CE1CR4               LOAD REPLY AREA BASE REGISTER
         LH    R15,QRACT               LOAD NUMBER OF ALL AVAIL. PATHS
         SR    R7,R7                   CLEAR A REGISTER TO COUNT WITH
         SPACE 2
* SEARCH THE REPLY AREA AND COUNT THE NUMBER OF PATHS TO THE OTHER
* SYSTEM WITH THE CORRECT PATH CLASS.
QEAC5100 DS    0H
         CLC   QRPCLSI(1),MUSPCL       DO THE PATH CLASSES MATCH?
         BNE   QEAC5200                NO  - BRANCH, CHECK NEXT PATH
         LA    R7,1(,R7)               INCREMENT COUNT OF MATCHED PATHS
QEAC5200 DS    0H
         LA    R5,QRALEN5(,R5)         POINT TO NEXT PATH IN LIST
         BCT   R15,QEAC5100            LOOP IF MORE PATHS TO CHECK
         SPACE 2
* R7 CONTAINS THE NUMBER OF AVAILABLE PATHS TO THE OTHER SYSTEM.
* SEARCH THE SECONDARY BLOCK AND COUNT THE NUMBER OF CONNECTIONS THE
* USER ALREADY HAS TO THE OTHER SYSTEM.
         LH    R14,MUSNCN              CURRENT NUMBER OF CONNECTIONS
         LTR   R14,R14                 USER HAVE ANY CONNECTIONS?
         BZ    QEAC6000                NO  - NO NEED TO SEARCH BLOCK
         SPACE 2
         LA    R6,MUSHLN(,R4)          POINT TO FIRST CONNECT SLOT
         LA    R14,MUSMAX              MAXIMUM NUMBER CONNECT SLOTS
         SR    R15,R15                 CLEAR A REGISTER TO COUNT WITH
         SPACE 2
* THIS CONNECTION'S SYSTEM EQUAL TO SYSTEM WANT TO CONNECT TO?
QEAC5300 DS    0H
         TM    MUSINDA,MUSUSE          IS THIS CONNECTION SLOT IN USE?
         BZ    QEAC5400                NO  - BRANCH AND CONTINUE SEARCH
         CLC   MUSSYSA(8),EBW020       SYSTEM NAMES EQUAL?
         BNE   QEAC5400                NO  - BRANCH AND CONTINUE SEARCH
         LA    R15,1(,R15)             YES - FOUND ONE MORE CONNECTION
QEAC5400 DS    0H
         LA    R6,MUSCLN(,R6)          POINT TO NEXT CONNECTION SLOT
         BCT   R14,QEAC5300            BRANCH IF MORE SLOTS TO CHECK
         SPACE 2
* IF THIS A MTOM USER, R7 WILL BE DECREMENTED ONCE FOR EVERY 2 ACTIVE
* CONNECTIONS FOUND IN THE SECONDARY BLOCK.
         CLI   MUSDEV,DEVCTOC          IS THIS A CTOC TYPE USER?
         BE    QEAC5500                YES  - NO NEED TO ADJUST R15
         SRL   R15,2                   MTOM - ADJUST NUMBER CONNECTIONS
         SPACE 2
* ADJUST R7 SO IT CONTAINS THE NUMBER OF AVAILABLE PATHS TO THE OTHER
* SYSTEM THE USER IS NOT ALREADY CONNECTED OVER.  IF R7 IS NOW GREATER
* THAN ZERO, A CONNECT WILL BE ISSUED TO THE OTHER SYSTEM.  IF R7 IS
* ZERO THEN EXIT BECAUSE THERE ARE NO PATHS TO CONNECT OVER.
QEAC5500 DS    0H
         SR    R7,R15                  ADJUST NUMBER OF PATHS
         BP    QEAC6000                BRANCH IF PATH TO CONNECT OVER
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         RELCC D4                      RELEASE THE QUERY REPLY AREA
         EXITC                         NO PATHS TO CONNECT OVER
         EJECT
***************************************************************
* THE USER'S SECONDARY BLOCK HAS TO BE SEARCHED TO FIND AN
* EMPTY CONNECT SLOT.  IF THE CONNECTION WILL CAUSE THE USER TO
* EXCEED IT'S MAXIMUM NUMBER OF CONNECTIONS, THEN AN ERROR
* MESSAGE IS PRINTED, AND THE CONNECTION IS NOT ESTABLISHED.
***************************************************************
         SPACE 2
QEAC6000 DS    0H
         LA    R14,MUSMAX              LOAD MAXIMUM NUMBER CONNECTIONS
         CH    R14,MUSNCN              WILL EXCEED MAX NUM CONNECTIONS?
         BH    QEAC6100                NO  - BRANCH AND CONTINUE
         WTOPC TEXT='CONNECT REQUEST SUPPRESSED FOR ........ TO SYSTEM X
               ........',COMP=YES,SUB=(CHARA,MUSNAM,CHARA,EBW020),     X
               PREFIX=QEAC,NUM=01,LET=E,ROUT=PRC
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         RELCC D4                      RELEASE THE QUERY REPLY AREA
         EXITC                         CANNOT DO ANY CONNECTING
         SPACE 2
QEAC6100 DS    0H
         LA    R6,MUSHLN(,R4)          POINT TO THE FIRST CONNECT SLOT
QEAC6200 DS    0H                      SEARCH FOR FREE CONNECT SLOT
         TM    MUSINDA,MUSUSE          IS THIS CONNECT SLOT IN USE?
         BZ    QEAC7000                NO  - USE THIS CONNECT SLOT
         LA    R6,MUSCLN(,R6)          YES - BUMP TO THE NEXT SLOT
         B     QEAC6200                TRY AND USE THE NEXT SLOT
         EJECT
***************************************************************
* THE MPIFC PARAMETER LIST WILL BE BUILT IN EBX000, AND THE
* MPIFC CONNECT REQUEST WILL BE ISSUED.  THE UCTOK WILL BE THE
* ADDRESS OF THE CONNECTION SLOT.  THE UCTOK WILL ALSO BE
* PASSED IN THE USER DATA AREA (CNDATA).  THIS ALLOWS MTOM
* CONNECTIONS TO HAVE 'MATCHED' CONNECTION SLOTS LINKED
* TOGETHER (MTOM USERS HAVE 2 CONNECTION SLOTS FOR EACH
* CONNECTION).  THE LINK IS NECESSARY SO OTHER SEGMENTS DO NOT
* PERFORM THE SAME SERVICE TWICE ON A CONNECTION.  FOR EXAMPLE,
* QEA3 USES THE LINK TO KEEP FROM SENDING THE SAME MESSAGES
* TWICE OVER THE SAME CONNECTION.
***************************************************************
         SPACE 2
QEAC7000 DS    0H
         XC    0(MUSCLN,R6),0(R6)      CLEAR THE CONNECTION SLOT
         LA    R5,EBX000               POINT TO MPIFC PARAMETER LIST
         XC    0(CNRPLEN,R5),0(R5)     CLEAR THE PARAMETER LIST
         MVC   CNPCLASS,MUSPCB         COPY THE CONNECTION'S PATH CLASS
         MVI   CNOPTS1,CNMPATH         INDICATE USING MULTI-PATHING
         MVC   CNIDTOK,MUSITK          COPY THE USER'S IDTOK
         MVC   CNOUNAM,MUSNAM          COPY THE USER'S NAME
         ST    R6,CNUCTOK              INDICATE THE CONNECTION'S CONTOK
         MVC   CNDUNAM,MUSNAM          SAVE THE DESTINATION USER NAME
         MVC   CNDSNAM,EBW020          COPY THE DESTINATION SYSTEM NAME
         MVC   CNDRXAD,QEACDRX         INDICATE THE DATA RECEIVED EXIT
         MVC   CNMS,=AL4(4096)         MAX. DATA THIS USER CAN SEND
         MVC   CNMR,=AL4(4096)         MAX. DATA THIS USER CAN RECEIVE
         SR    R1,R1                   CLEAR ADDRESS REGISTER
         LA    R1,CNUCTOK              LOAD PTR TO THIS SIDE'S UCTOK
         ST    R1,CNDATA               UCTOK TO OTHER SIDE CONNECTION
         MVC   CNDATAL,=AL4(4)         LENGTH OF DATA IN CNDATA
         SPACE 2
         MPIFC RTYPE=CONNECT,REG=R5    ISSUE THE MPIFC CONNECT
         LTR   R15,R15                 WAS THE CONNECTION SUCCESSFUL?
         BNZ   QEAC9100                NO  - GO ISSUE AN ERROR MESSAGE
         SPACE 2
         OI    MUSINDA,MUSUSE          INDICATE CONNECTION SLOT IN USE
         OI    MUSINDA,MUSINC          CONNECTION IS INCOMPLETE
         MVC   MUSSYSA,EBW020          SAVE SYSTEM NAME IN SEC. BLOCK
         SPACE 2
         LH    R6,MUSNCN               LOAD NUMBER USER'S CONNECTIONS
         LA    R6,1(,R6)               INCREMENT THAT NUMBER
         STH   R6,MUSNCN               SAVE NEW NUMBER OF CONNECTIONS
         SPACE 2
         LH    R6,MUSICON              NUMBER OF INCOMPLETE CONNECTIONS
         LA    R6,1(,R6)               INCREMENT THAT NUMBER
         STH   R6,MUSICON              NEW NUMBER OF INCOMPLETE CONN.
         SPACE 2
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAC7100                YES - DON'T PRINT MESSAGE
         WTOPC TEXT='SUCCESSFUL CONNECT REQUEST FOR ........ TO SYSTEM X
               ........',COMP=YES,SUB=(CHARA,MUSNAM,CHARA,EBW020),     X
               PREFIX=QEAC,NUM=02,LET=I,ROUT=PRC
QEAC7100 DS    0H
         RELCC D4                      RELEASE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEACDRX  SWISC QEAA,IS=R0,TYPE=ENTER   DATA RECEIVED EXIT VECTOR
         EJECT
***************************************************************
* ISSUE BAD MPIFC RETURN CODE ERROR MESSAGES.
***************************************************************
         SPACE 2
QEAC9000 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC QUERY RETURN CODE .... FOR ........',       X
               COMP=YES,PREFIX=QEAC,LET=E,TIME=YES,NUM=03,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,MUSNAM)
         RELCC D4                      RELEASE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEAC9100 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC CONNECT RETURN CODE .... FOR ........',     X
               COMP=YES,PREFIX=QEAC,LET=E,TIME=YES,NUM=04,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,MUSNAM)
         RELCC D4                      RELEASE QUERY REPLY AREA
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
         LTORG
         FINIS QEAC
         END   ,
