         PRINT NOGEN
         BEGIN NAME=QEAD,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEAD                                       *
* RELATED MODULES. QEA0-QEA6, QEAA-QEAC, QEAE-QEAF            *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF TEST DRIVER'S ERROR EXIT (ERX)        *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS IS THE MPIF TEST DRIVER'S ERROR EXIT.  IT  *
*             IS INVOKED WHENEVER MPIF DETECTS AN ERROR HAS   *
*             OCCURRED FOR A USER.  THIS SEGMENT WILL TAKE    *
*             ACTION ACCORDING TO THE TYPE OF ERROR THAT      *
*             OCCURRED.                                       *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBCM01 - SYSTEM TEST INDICATORS       *
*                       EBX000 - MPIFC DISCONNECT PARAMETER   *
*                                LIST                         *
*                                                             *
* REGISTER-CONVENTIONS..R1  - SD0RV  BASE REGISTER            *
*                       R2  - PI1DT  BASE REGISTER            *
*                       R4  - DCTMUS BASE REGISTER            *
*                       R5  - DCTMUP BASE REGISTER            *
*                       R6  - DCTMUS BASE REGISTER, SUFFIX=A  *
*                       R15 - DCTMUS BASE REGISTER, SUFFIX=B  *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - THIS EXIT CAN BE INITIATED ON ANY         *
*                   I-STREAM, EVEN THOUGH THE DRIVER IS       *
*                   STRUCTURED TO BE I-STREAM SHARED.         *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * NONE                 *
* DATA LEVELS.. * D0 - MPIF DATA 1     * NONE                 *
*               * D1 - MPIF DATA 2     *                      *
*               * (ONLY IF MPIF UNABLE *                      *
*               * TO DELIVER THE       *                      *
*               * MESSAGE)             *                      *
* REGISTERS.... * R1 - ERX PARAMETER   * NONE                 *
*               *      LIST            *                      *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  @ISSDRV - DRIVER GLOBAL AREA             *
*                                                             *
* EXIT-NORMAL.. EXITC                                         *
*     -ERROR... NONE                                          *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* LOAD THE BASE REGISTERS.  THE UITOK VALUE IS THE ADDRESS OF
* THIS USER'S SECONDARY BLOCK.  THE UCTOK VALUE IS THE ADDRESS
* OF A CONNECTION SLOT WITHIN THE SECONDARY BLOCK.  THIS
* CONNECTION IS THE ONE THAT IS IN "ERROR".  THE SECONDARY
* BLOCK WILL BE LOCKED FOR THIS ENTIRE SEGMENT.
***************************************************************
         SPACE  2
         DCTMUP REG=R5,LTYPE=ALL       MPIFC PARAMETER LISTS
         DCTMUS REG=R4                 DRIVER SECONDARY BLOCK DSECT
         DCTMUS REG=R6,SUFFIX=A        DSECT CONNECTION SLOTS WITHIN --
         DCTMUS REG=R15,SUFFIX=B       --- THE SECONDARY BLOCK
         SD0RV  REG=R1                 DRIVER @ISSDRV DSECT
         GLOBZ  REGR=R14               LOAD GLOBAL 1 ADDRESS
         SPACE  2
         LR     R5,R1                  ERX PARAMETER LIST ADDRESS
         L      R1,@ISSDRV             LOAD @ISSDRV BASE REGISTER
         MVC    EBCM01(1),@SYSTEST     SAVE SYSTEM TEST INDICATOR
         MVC    EBROUT(3),@EBROUT      INDICATE WHERE ROUTE OUTPUT MSGS
         MVC    CE1CPD(1),@CE1CPD      CPU ID NEEDED FOR ROUTING ALSO
         L      R4,EREUIT              USER'S UITOK = SEC. BLOCK ADDR.
         L      R6,EREUCT              LOAD THE CONNECTION SLOT ADDRESS
         CORHC (R4)                    LOCK THE SECONDARY BLOCK
         EJECT
***************************************************************
* OBTAIN ADDRESSABILITY TO THE PROCESSOR ID TABLE, WHICH WILL
* BE USED TO GET THIS PROCESSOR'S CPU ID.  THE ID WILL THEN BE
* PLACED IN THIS ECB'S 'CE1CPD' FIELD BEFORE ISSUING CERTAIN
* WTOPC'S.  THIS WILL PREVENT WTOPC FROM ROUTING THESE MESSAGES
* TO A POTENTIALLY INACTIVE PROCESSOR, CAUSING OPR-34C DUMPS.
***************************************************************
         SPACE 2
         PI1DT REG=R2                     PROCESSOR ID TABLE DSECT
         CINFC R,CMMPID,F,REG=R2,BSS=YES  GET ADDRESS OF THE PID
         EJECT
***************************************************************
* IF THE ERROR EXIT WAS KICKED OFF BECAUSE MPIF WAS UNABLE TO
* DELIVER A MESSAGE, PRINT AN ERROR MESSAGE AND EXIT (NO
* CONNECTIONS HAVE BEEN LOST).
***************************************************************
         SPACE 2
QEAD1000 DS    0H
         CLC   ERECV,=AL2(ERECRAB)     MESSAGE RETURNED BY MPIF?
         BNE   QEAD2000                NO  - BRANCH AND CONTINUE
         SPACE 2
         MVC   CE1CPD(1),PI1CPUI       ENSURE ROUTING TO OWN PROCESSOR
         WTOPC TEXT='MPIF UNABLE TO SEND MESSAGE FOR USER ........',   X
               PREFIX=QEAD,NUM=01,LET=E,ROUT=PRC,COMP=YES,             X
               SUB=(CHARA,MUSNAM),EBROUT=NO
         SPACE 2
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC
         EJECT
***************************************************************
* IF THE ERROR EXIT WAS KICKED OFF DUE TO A WARNING, ISSUE A
* MPIFC DISCONNECT TO END THE CONNECTION.  IF THIS IS A MTOM
* USER THE ERROR EXIT WILL BE KICKED OFF FOR BOTH ENDS OF THE
* CONNECTION.  TO PREVENT BOTH ECB'S FROM ISSUING A DISCONNECT,
* THE FIRST WILL SET A BIT IN THE CONNECT SLOT OF THE OTHER,
* TO INDICATE THE OPERATION IS COMPLETE.  SINCE THE ERROR EXIT
* WILL KICKED OFF WHEN THE DISCONNECT IS ISSUED THE MUSERX
* INDICATOR WILL ALSO BE SET FOR A MTOM USER.  THIS WILL
* PREVENT THE 'CONNECTION LOST' MESSAGE FROM BEING PRINTED
* AFTER THE DISCONNECT.
*
* IF THE CONNECTION IS INCOMPLETE, THE DISCONNECT WILL NOT BE
* ISSUED.  THE CONNECTION WILL EITHER BE COMPLETED OR
* TERMINATED USING THE CONNECTION COMPLETE EXIT (QEA6).  ALL
* INDICATORS AND COUNTERS WILL BE CLEANED UP THERE.
***************************************************************
         SPACE 2
QEAD2000 DS    0H
         TM    ERECODE+1,ERXEWARN      ERX DUE TO A WARNING?
         BZ    QEAD3000                NO  - BRANCH AND CONTINUE
         SPACE 2
         TM    MUSINDA,MUSINC          IS THIS CONNECTION COMPLETE?
         BNZ   QEAD2400                NO  - GO EXIT THIS SEGMENT
         SPACE 2
         CLI   MUSDEV,DEVCTOC          IS THIS A DEVICE = CTOC USER?
         BE    QEAD2200                YES - CAN ISSUE THE DISCONNECT
         TM    MUSINDA,MUSDIS          DISCONNECT ALREADY ISSUED?
         BZ    QEAD2100                NO  - GO ISSUE THE DISCONNECT
         NI    MUSINDA,X'FF'-MUSDIS    YES - CLEAR DISCONNECT INDICATOR
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEAD2100 DS    0H                      SET FLAGS IN OTHER CONNECT SLOT
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSINDB,MUSDIS          INDICATE DISCONNECT COMPLETED
         OI    MUSINDB,MUSERX          INDICATE ERX ALREADY KICKED OFF
         SPACE 2
QEAD2200 DS    0H
         LA    R5,EBX000               LOAD DISCONNECT PARAMETER BASE
         XC    EBX000(DSRPLEN),EBX000  CLEAR THE PARAMETER LIST
         MVC   DSIDTOK,MUSITK          USER'S ID TOKEN TO PARAM. LIST
         MVC   DSCONTOK,MUSCTKA        THIS CONNECTION'S CONTOK
         MVC   DSRUNAM(8),MUSNAM       REQUESTING USER'S NAME TO LIST
         MVC   DSDUNAM(8),MUSNAM       DESTINATION USER'S NAME TO LIST
         MPIFC RTYPE=DISCONNECT,REG=R5 ISSUE THE DISCONNECT REQUEST
         SPACE 2
         LTR   R15,R15                 MPIFC COMPLETE SUCCESSFULLY?
         BNZ   QEAD2300                NO  - GO ISSUE ERROR MESSAGE
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAD4000                YES - DON'T PRINT MESSAGE
         WTOPC TEXT='SUCCESSFUL DISCONNECT FOR ........ TO SYSTEM .....X
               ...',COMP=YES,SUB=(CHARA,MUSNAM,CHARA,MUSSYSA),LET=W,   X
               PREFIX=QEAD,NUM=02,ROUT=PRC
         B     QEAD4000                GO DO SOME CLEAN UP
         SPACE 2
QEAD2300 DS    0H
         LR    R6,R15                  LOAD MPIFC RETURN CODE
         WTOPC TEXT='MPIFC DISCONNECT RETURN CODE .... FOR ........',  X
               COMP=YES,PREFIX=QEAD,LET=E,TIME=YES,NUM=03,ROUT=PRC,    X
               SUB=(DEC,(R6),CHARA,MUSNAM)
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         SPACE 2
QEAD2400 DS    0H                      CONNECTION IS INCOMPLETE
         WTOPC TEXT='DISCONNECT FOR ........ TO SYSTEM ........ SUPPRESX
               SED',COMP=YES,SUB=(CHARA,MUSNAM,CHARA,MUSSYSA),         X
               PREFIX=QEAD,NUM=06,LET=W,ROUT=PRC
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
***************************************************************
* ERROR IS DUE TO A LOST CONNECTION.  IF THIS A MTOM USER THE
* 'CONNECTION LOST' MESSAGE WILL BE MADE ONLY IF THE MESSAGE
* HASN'T BEEN PRINTED PREVIOUSLY.  THIS IS DONE BECAUSE THE ERX
* WILL BE KICKED OFF FOR EACH SIDE OF THE MTOM CONNECTION.
***************************************************************
         SPACE 2
QEAD3000 DS    0H
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAD4000                YES - DON'T PRINT MESSAGE
         SPACE 2
         CLI   MUSDEV,DEVCTOC          IS THIS A DEVICE = CTOC USER?
         BE    QEAD3200                YES - CAN REPORT ERROR
         TM    MUSINDA,MUSERX          MESSAGE ALREADY BEEN PRINTED?
         BZ    QEAD3100                NO  - BRANCH AND PRINT MESSAGE
         NI    MUSINDA,X'FF'-MUSERX    YES - CLEAR INDICATOR
         B     QEAD4000                BRANCH, GO DO SOME CLEAN UP
         SPACE 2
QEAD3100 DS    0H
         L     R15,MUSOCSA             UCTOK OF OTHER SIDE CONNECTION
         OI    MUSINDB,MUSERX          IND. TO OTHER SIDE MSG PRINTED
         SPACE 2
QEAD3200 DS    0H
         MVC   CE1CPD(1),PI1CPUI       ENSURE ROUTING TO OWN PROCESSOR
         WTOPC TEXT='CONNECTION LOST TO SYSTEM ........ FOR USER ......X
               ..',COMP=YES,PREFIX=QEAD,LET=W,ROUT=PRC,EBROUT=NO,      X
               TIME=YES,NUM=04,SUB=(CHARA,MUSSYSA,CHARA,MUSNAM)
         EJECT
***************************************************************
* THIS ROUTINE WILL UPDATE THE SECONDARY BLOCK'S NUMBER OF
* CONNECTIONS AND MARK THE CONNECTION SLOT FREE.  IF THE USER
* IS NO LONGER CONNECTED A WARNING WILL BE PRINTED.
***************************************************************
         SPACE 2
QEAD4000 DS    0H
         LH    R15,MUSNCN              NUMBER OF USER'S CONNECTIONS
         BCTR  R15,0                   DECREMENT THE NUMBER CONNECTIONS
         STH   R15,MUSNCN              SAVE THE NEW NUMBER CONNECTIONS
         NI    MUSINDA,X'FF'-MUSUSE    MARK CONNECTION SLOT FREE
         SPACE 2
         LTR   R15,R15                 USER STILL HAVE A CONNECTION?
         BNZ   QEAD4100                YES - DON'T PRINT WARNING MSG
         TM    EBCM01,SYSRUN           IS THIS A SYSTEM TEST RUN?
         BO    QEAD4100                YES - DON'T PRINT WARNING MSG
         MVC   CE1CPD(1),PI1CPUI       ENSURE ROUTING TO OWN PROCESSOR
         WTOPC TEXT='........ IS NO LONGER CONNECTED TO ANOTHER USER', X
               COMP=YES,PREFIX=QEAD,LET=W,TIME=YES,NUM=05,ROUT=PRC,    X
               SUB=(CHARA,MUSNAM),EBROUT=NO
         SPACE 2
QEAD4100 DS    0H
         CORUC (R4)                    UNLOCK THE SECONDARY BLOCK
         EXITC                         EXIT THIS SEGMENT
         EJECT
         LTORG
         FINIS QEAD
         END   ,
