         PRINT NOGEN
         BEGIN NAME=QEAE,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          Copyright IBM Corp. 2006, 2010                             *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
* D16936  12/10/2007  ZTEST MPIF SETUP driver needs to be updated     *
*                        to set up MPIF conn                          *
* F20508  05/14/2010  Updates required due to z/TPF APAR PJ37752      *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEAE                                       *
* RELATED MODULES. QEA0-QEA6, QEAA-QEAD, QEAF                 *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF COMPLEX SETUP                         *
* LEVEL........... VERSION 5  MODIFICATION 1                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS RESPONSIBLE FOR ISSUING "ZMPIF" *
*             COMMANDS TO DEFINE THE MPIF COMPLEX, SYSTEM,    *
*             CLASSES, DEVICES, AND PATHS. THE SYSTEM AND     *
*             PATH PARAMETERS WILL BE BASED ON THIS           *
*             PROCESSOR'S CPU ID.                             *
*                                                             *
*             QEAE IS INVOKED BY QEA0 WHEN A "ZTEST MPIF      *
*             SETUP" MESSAGE IS ENTERED. ENTRY TO QEAE IS     *
*             BY AN ENTNC FROM QEA0                           *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   DEPENDENCY.... NONE                                       *
*   ALLOCATION.... 4K, CORE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBW002   - PROCESSOR INDEX            *
*                       EBW003   - MESSAGE NUMBER             *
*                                                             *
* REGISTER-CONVENTIONS..R2 - PI1DT BASE REGISTER              *
*                       R3 - KEYPOINT "M" BASE REGISTER       *
*                       R4 - MI0MI DSECT BASE REGISTER        *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... NONE                                        *
*                                                             *
* RESTRICTIONS... - MPIF AND SDPS (LOOSELY-COUPLED) MUST BE   *
*                   INSTALLED                                 *
*                 - 'SETUP' CAN ONLY BE ISSUED FROM THE BSS   *
*                 - 'SETUP' PROCESSING CANNOT ACCESS ANY      *
*                   GLOBAL FIELDS SINCE IT CAN RUN IN RESTART *
*                   BEFORE GLOBALS ARE LOADED.                *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * NONE                 * NONE                 *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * NONE                 * NONE                 *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  CNOU - TO ISSUE 'ZSIPC A P 10' MESSAGE   *
*                    CBE0 - TO ISSUE 'ZMPIF' MESSAGES         *
*                    CVFN - TO ISSUE 'ZRIPL' MESSAGE          *
*                                                             *
*      DATA AREAS..  PROCESSOR ID TABLE (PIDT)                *
*                                                             *
* EXIT-NORMAL.. ENTDC TO CVFN FOR ZRIPL                       *
*     -ERROR... NONE                                          *
*                                                             *
***************************************************************
         EJECT
***********************************************************************
* D A T A   M A C R O S                                               *
***********************************************************************
         CZ1CP ,                       CONTROL PROGRAM EQUATES          F20508
**********************************************************************
* ENTRY WAS FROM QEA0 TO BEGIN THE SETUP PROCESS:                    *
*                                                                    *
*   - IF THIS IS NOT THE BSS SUBSYSTEM, OR IF IS THIS IS NOT AN SDPS *
*     (LOOSELY-COUPLED) SYSTEM, ISSUE AN ERROR MESSAGE AND EXIT      *
*   - PRINT AN OUTPUT MESSAGE THAT THE SETUP HAS STARTED             *
**********************************************************************
         SYSTC SBSDPS,IFOFF=QEAENOLC   BRANCH IF NOT A L/C SYSTEM
         CLC   CE1SSU(2),=X'FF00'      IS THIS THE BSS SUBSYSTEM?
         BNE   QEAESUB                 NO - BRANCH AND ISSUE ERROR
*
         LA    R6,QEAEMSG1
         BAS   R7,QEAEWTOPC
**************************************************************
* ESTABLISH BASE REGISTERS TO KEYPOINT "M" AND MESSAGE BLOCK *
**************************************************************
         MK0CK REG=R3
         CINFC R,CMMCTKM,F,REG=R3,BSS=YES  GET ADDRESS OF KEYPOINT M
*
         MI0MI REG=R4                  INPUT MESSAGE DSECT
         GETCC D9,L0                   GET CORE BLOCK FOR MESSAGES
         L     R4,CE1CR9               SET THE INPUT MESSAGE BASE REG
**********************************************************************
*   - GO ISSUE THE FIRST ZMPIF MESSAGE                               *
**********************************************************************
         XC    EBW002(2),EBW002        Clear Proc and Msg Indexes
         B     QEAEQEAE                GO ISSUE THE FIRST MESSAGE
*
QEAENOLC DS    0H
         LA    R6,QEAEMSG2
         BAS   R7,QEAEWTOPC
         EXITC
*
QEAESUB  DS    0H
         LA    R6,QEAEMSG3
         BAS   R7,QEAEWTOPC
         EXITC
         EJECT
************************************************************
* CONTINUE THE SETUP PROCESS                               *
* EBW003 CONTAINS THE NUMBER OF THE LAST MESSAGE COMPLETED *
************************************************************
         SPACE 2
QEAEQEAE DS    0H
         PI1DT REG=R2                      PROCESSOR ID TABLE DSECT
         CINFC R,CMMPID,F,REG=R2,BSS=YES   GET ADDRESS OF THE PID
         XR    R15,R15                 Clear R15
         IC    R15,EBW003              LOAD MESSAGE NUMBER
         MH    R15,=H'4'               CREATE A BRANCH INDEX
         B     QEAETBLE(R15)           BRANCH BASED ON MESSAGE NUMBER
         SPACE 2
QEAETBLE DS    0H
         B     QEAEM00                 GO DO MESSAGE NUMBER 00
         B     QEAEM01                 GO DO MESSAGE NUMBER 01
         B     QEAEM02                 GO DO MESSAGE NUMBER 02
         B     QEAEM03                 GO DO MESSAGE NUMBER 03
         B     QEAEM04                 GO DO MESSAGE NUMBER 04
         B     QEAEM05                 GO DO MESSAGE NUMBER 05
         B     QEAEM06                 GO DO MESSAGE NUMBER 06
         B     QEAEM07                 GO DO MESSAGE NUMBER 07
         B     QEAEM08                 GO DO MESSAGE NUMBER 08
         B     QEAEM09                 GO DO MESSAGE NUMBER 09
         B     QEAEM10                 GO DO MESSAGE NUMBER 10
         B     QEAEM11                 GO DO MESSAGE NUMBER 11
         EJECT
*********************************************************************
*        BUILD MESSAGE 00
*********************************************************************
         SPACE 2
QEAEM00  DS    0H
         MVC   MI0ACC+6(L'M00+1),M00   MOVE THE TEXT INTO MSG BLOCK
         MVI   EBW003,1                Do Msg 1 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 01
*
*        Do this message for the current processor only.
*
*********************************************************************
         SPACE 2
QEAEM01  DS    0H
         MVC   MI0ACC+6(L'M01+1),M01   MOVE TEXT INTO MESSAGE BLOCK
         MVC   MI0ACC+M01PP1(1),PI1CPUI Put processor ID into base
         MVI   EBW003,2                Do Msg 2 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 02
*********************************************************************
         SPACE 2
QEAEM02  DS    0H
         MVC   MI0ACC+6(L'M02+1),M02   MOVE THE TEXT INTO MSG BLOCK
         MVI   EBW003,3                Do Msg 3 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 03
*********************************************************************
         SPACE 2
QEAEM03  DS    0H
         MVC   MI0ACC+6(L'M03+1),M03   MOVE THE TEXT INTO MSG BLOCK
         MVI   EBW003,4                Do Msg 4 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 04
*********************************************************************
         SPACE 2
QEAEM04  DS    0H
         MVC   MI0ACC+6(L'M04+1),M04   MOVE THE TEXT INTO MSG BLOCK
         MVI   EBW003,5                Do Msg 5 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 05
*********************************************************************
         SPACE 2
QEAEM05  DS    0H
         MVC   MI0ACC+6(L'M05+1),M05   MOVE THE TEXT INTO MSG BLOCK
         MVI   EBW003,6                Do Msg 6 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 06
*
*        Do this message for the current processor only.
*
*********************************************************************
         SPACE 2
QEAEM06  DS    0H
         MVC   MI0ACC+6(L'M06+1),M06   MOVE TEXT INTO MESSAGE BLOCK
         MVC   MI0ACC+M06PP1(1),PI1CPUI Put processor ID into Msg
         MVC   MI0ACC+M06PP2(1),PI1CPUI Put processor ID into Msg
         MVI   EBW003,7                Do Msg 07 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 07
*
*        Do this message for the current processor only.
*
*********************************************************************
         SPACE 2
QEAEM07  DS    0H
         MVC   MI0ACC+6(L'M07+1),M07   MOVE TEXT INTO MESSAGE BLOCK
         MVC   MI0ACC+M07PP1(1),PI1CPUI Put processor ID into Msg
         MVC   MI0ACC+M07PP2(1),PI1CPUI Put processor ID into Msg
         MVI   EBW003,8                Do Msg 8 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 08
*
*        Note for Messages 8 and 9:
*
*        These messages constitute a loop.  Do Msg 8 and then 9
*        for proc 0, then do Msg 8 and 9 for proc 1, and so
*        on.  That's why message 9 loads message number 8 as the
*        next message.
*
*        Do messages 8 and 9 for all processors except the
*        current processor.
*
*********************************************************************
         SPACE 2
QEAEM08  DS    0H
         XR    R5,R5                   Clear R5 for IC
         IC    R5,EBW002               Put Processor Index into R5
         CH    R5,PI1CIDN              Is R5=idx of current processor?
         BNE   QEAEM08B                No, continue with this msg
         SPACE 1
         LA    R5,1(,R5)               Yes, increment to next proc.
         CH    R5,PI1NOCP              Have we done all processors?
         BL    QEAEM08A                No, save new index and continue
         XC    EBW002,EBW002           Yes, zero proc index and
         B     QEAEM10                 go to message 10
         SPACE 1
QEAEM08A DS    0H
         STC   R5,EBW002
QEAEM08B DS    0H
         MH    R5,=Y(PI1LNG)           Multiply by PI1DT Sec II length
         AR    R2,R5                   Add to PI1DT Base
         SPACE 1
         MVC   MI0ACC+6(L'M08+1),M08   MOVE TEXT INTO MESSAGE BLOCK
         MVC   MI0ACC+M08PP1(1),PI1SPT Put processor ID into Msg
         MVC   MI0ACC+M08PP2(1),PI1SPT Put processor ID into Msg
         MVI   EBW003,9                Do Msg 9 Next
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 09
*
*        Note for Messages 8 and 9:
*
*        These messages constitute a loop.  Do Msg 8 and then 9
*        for proc 0, then do Msg 8 and 9 for proc 1, and so
*        on.  That's why message 9 loads message number 8 as the
*        next message.
*
*********************************************************************
         SPACE 2
QEAEM09  DS    0H
         XR    R5,R5                   Clear R5 for IC
         IC    R5,EBW002               Put Processor Index into R5
         LR    R6,R5                   Put Index into R6 for later MH
         SPACE 1
         LA    R5,1(,R5)
         CH    R5,PI1NOCP              Are there more procs to do?
         BNL   QEAEM009A               No, setup for Msg 10
         STC   R5,EBW002               Yes, store new processor index
         MVI   EBW003,8                Go and do Msg 08 for next proc
         B     QEAEM009B
         SPACE 1
QEAEM009A DS   0H
         XC    EBW002,EBW002           Clear Processor Index
         MVI   EBW003,10               Do Msg 10 Next
         SPACE 1
QEAEM009B DS   0H
         MH    R6,=Y(PI1LNG)           Multiply by PI1DT Sec II length
         AR    R2,R6                   Add to PI1DT Base
         SPACE 1
         MVC   MI0ACC+6(L'M09+1),M09   MOVE TEXT INTO MESSAGE BLOCK
         MVC   MI0ACC+M09PP1(1),PI1SPT Put processor ID into Msg
         MVC   MI0ACC+M09PP2(1),PI1SPT Put processor ID into Msg
         B     QEAECRES                GO ISSUE MESSAGE AND CRESC
         EJECT
*********************************************************************
*        BUILD MESSAGE 10
*********************************************************************
         SPACE 2
QEAEM10  DS    0H
         MVC   MI0ACC(L'M10+1),M10     MOVE MESSAGE INTO BLOCK
         MVC   MI0CCT(2),=AL2(L'M10+1) SET UP LENGTH IN MESSAGE BLOCK
         MVI   EBW003,11               Do Msg 11 Next
         SR    R1,R1                   TIMEOUT
         SR    R6,R6                   RTNLST
         LA    R7,128                  LENGTH
         CRESC CNOU,DATA=(R4,R7),IS=SAME,RTNLST=(R6),                  X
               WAIT=YES,TIMEOUT=(R1)
         B     QEAEQEAE
         EJECT
*********************************************************************
*        BUILD MESSAGE 11 - THE SETUP IS COMPLETE, ISSUE THE ZRIPL.
*********************************************************************
         SPACE 2
QEAEM11  DS    0H
         MVC   MI0ACC(L'M11+1),M11     MOVE TEXT INTO MESSAGE BLOCK
         MVC   MI0CCT(2),=AL2(L'M11+1) SET UP LENGTH OF THE MESSAGE
         LR    R1,R4                   SET UP R1 FOR CVFN               F20508
         FLIPC D0,D9                   PLACE MESSAGE ON D0 FOR CVFN     F20508
         ENTDC CVFN                    GO ISSUE THE ZRIPL               F20508
         EJECT
***********************************************************************
*        FINISH BUILDING THE "ZMPIF" MESSAGE,                         *
*        AND CALL CBE0 TO PROCESS IT                                  *
*                                                                     *
*        ANY PAIR OF DOLLAR SIGNS ($$) FOUND IN THE ZMPIF MESSAGE     *
*        WILL BE REPLACED BY THE FIRST TWO VSN CHARACTERS OF THE      *
*        ONLINE DASD OF THIS SYSTEM.  THIS IS TO ALLOW MULTIPLE TPF   *
*        COMPLEXES TO CO-EXIST WITHOUT THEM TRYING TO ESTABLISH PATHS *
*        TO EACH OTHER.                                               *
***********************************************************************
         SPACE 2
QEAECRES DS    0H
         MVC   MI0ACC(6),=C'ZMPIF '    FINISH BUILDING ZMPIF MESSAGE
         LA    R14,MI0ACC+6            POINT TO REST OF MESSAGE TEXT
         #DO   UNTIL=(CLI,0(R14),EQ,#EOM)       EXIT IF END OF MESSAGE
           #IF   (CLC,0(2,R14),EQ,=CL2'$$')     POINTING TO $$ ?
             MVC   0(2,R14),MK0AVSN             YES - REPLACE WITH VSN
           #EIF
           LA    R14,1(,R14)           POINT TO NEXT MESSAGE CHARACTER
         #EDO
         SR    R1,R1                   TIMEOUT
         SR    R6,R6                   RTNLST
         LA    R7,128                  LENGTH
         CRESC CBE0,DATA=(R4,R7),IS=SAME,RTNLST=(R6),                  X
               WAIT=YES,TIMEOUT=(R1)
         B     QEAEQEAE
         EJECT
***************************************************************
* SUBROUTINE TO ISSUE A WTOPC                                 *
***************************************************************
QEAEWTOPC DS   0H
         DCTMSG
         USING DCTMSG,R6
         WTOPC TEXTA=MSGLEN,LETA=MSGLET,NUMA=MSGNUM,PREFIX=QEAE
         DROP  R6                   DCTMSG
         BR    R7                   Return to caller
         EJECT 1
**************************************************************
* THE FOLLOWING IS A LIST OF MESSAGES THAT WILL BE ISSUED.   *
*                                                            *
* PLEASE NOTE THE FOLLOWING:                                 *
*  - THE "ZMPIF" PORTION OF THE MESSAGES IS ADDED BY THE     *
*    "QEAECRES" ROUTINE.                                     *
*                                                            *
*  - THE ORDER OF THIS LIST SHOULD NOT BE ALTERED, AND       *
*    PLEASE REMEMBER THE FOLLOWING WHEN ADDING OR DELETING   *
*    MESSAGES:                                               *
*                                                            *
*     * MESSAGE NUMBERS ARE USED AS A TABLE INDEX.           *
*     * PROCESSING FOR EACH MESSAGE DIFFERS DEPENDING ON     *
*       THE TYPE OF MESSAGE.                                 *
*     * FOR PROCESSOR UNIQUE MESSAGES, QEAE ASSUMES THE      *
*       LENGTH OF THE MESSAGE IS EQUAL FOR EACH CPU.         *
*                                                            *
*  - The Processor Position values are zero-based and start  *
*    at the beginning of the string + 6 (for the ZMPIF part) *
**************************************************************
         SPACE 2
M00    DC C'SET C,N-32,NU-100,C-80,P-81,S-82',AL1(#EOM)
*
M01    DC C'SET S,N-CPUx.$$,NC-90,NU-24,NP-90,ND-90,Q-10',AL1(#EOM)
M01PP1 EQU 17      Processor Position 1 (where the 'x' is)
*
M02    DC C'SET CLASS,CODE-A,RB-60000,WB-60000,P-Y,B-Y',AL1(#EOM)
M03    DC C'SET CLASS,CODE-B,RB-4000,WB-4000,L-Y',AL1(#EOM)
*
M04    DC C'DEF DEV,N-$CTOC01,MOD-C,SDA-256/0900,CL-AB',AL1(#EOM)
M05    DC C'DEF DEV,N-$CTOC02,MOD-C,SDA-84/0100,CL-AB',AL1(#EOM)
*
M06    DC C'DEF P,N-CPUx.A1,D-MTOM,S-CPUy.$$,C-A',AL1(#EOM)
M06PP1 EQU 17      Processor Position 1 (where the 'x' is)
M06PP2 EQU 34      Processor Position 2 (where the 'y' is)
*
M07    DC C'DEF P,N-CPUx.B1,D-MTOM,S-CPUy.$$,C-B',AL1(#EOM)
M07PP1 EQU 17      Processor Position 1 (where the 'x' is)
M07PP2 EQU 34      Processor Position 2 (where the 'y' is)
*
M08    DC C'DEF P,N-CPUx.A1,S-CPUy.$$,C-A',AL1(#EOM)
M08PP1 EQU 17      Processor Position 1 (where the 'x' is)
M08PP2 EQU 27      Processor Position 2 (where the 'y' is)
*
M09    DC C'DEF P,N-CPUx.B1,S-CPUy.$$,C-B',AL1(#EOM)
M09PP1 EQU 17      Processor Position 1 (where the 'x' is)
M09PP2 EQU 27      Processor Position 2 (where the 'y' is)
*
M10    DC C'ZSIPC A P 10',AL1(#EOM)
*
M11    DC C'ZRIPL',AL1(#EOM)
*
**********************************************************
* MESSAGES
**********************************************************
QEAEMSG1 GENMSG TEXT='MPIF SETUP BEGINS - RE-IPL WILL OCCUR WITHIN 15  X
               SECONDS',NUM=1,LET=W
QEAEMSG2 GENMSG TEXT='SETUP IS NOT ALLOWED ON A BASE-ONLY SYSTEM',     X
               NUM=2,LET=E
QEAEMSG3 GENMSG TEXT='SETUP IS RESTRICTED TO THE BSS',                 X
               NUM=3,LET=E
         EJECT
         LTORG
         FINIS QEAE
         END   ,
