         PRINT NOGEN
         BEGIN NAME=QEAF,IBM=YES
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
***************************************************************
*                                                             *
* MODULE NAME..... QEAF                                       *
* RELATED MODULES. QEA0-QEA6, QEAA-QEAE                       *
* DOCUMENT NAME... z/TPF MPIF DRIVER USER'S GUIDE             *
* DESCRIPTION..... MPIF DRIVER STATUS DISPLAY SEGMENT         *
* LEVEL........... VERSION 2  MODIFICATION 2                  *
*                                                             *
* FUNCTION... THIS SEGMENT IS RESPONSIBLE FOR HANDLING STATUS *
*             REQUESTS.  IF "ZTEST MPIF STATUS" IS ENTERED,   *
*             A DETAILED STATUS REPORT WILL BE GENERATED FOR  *
*             THIS SUBSYSTEM-USER.  IF "ZTEST MPIF STATUS     *
*             ALL" IS ENTERED, SUMMARY STATUS IS DISPLAYED    *
*             FOR ALL SUBSYSTEM-USERS.                        *
*                                                             *
* MODULE ATTRIBUTES..                                         *
*   TYPE.......... 'E' (ECB CONTROLLED)                       *
*   PROCESSOR..... SHARED, SUBSYSTEM UNIQUE                   *
*   ALLOCATION.... 4K, FILE-RESIDENT                          *
*   ENVIRONMENT... DRIVER                                     *
*                                                             *
* ECB-CONVENTIONS.......EBW000 - REPORT TYPE INDICATOR        *
*                       EBW040 - WTOPC TEXT SAVE AREA         *
*                       EBW044 - WTOPC TEXT SAVE AREA         *
*                       EBX010 - WTOPC TEXT SAVE AREA         *
*                       EBX014 - WTOPC TEXT SAVE AREA         *
*                       EBX020 - SYSTEM TEST PARAMETER AREA   *
*                                                             *
* REGISTER-CONVENTIONS..R1 - SD0RV BASE REGISTER              *
*                       R2 - DCTMPR BASE REGISTER             *
*                       R4 - GLOBAL 1 BASE REGISTER           *
*                       R5 - MS0AT BASE REGISTER              *
*                       R6 - MS0UT BASE REGISTER              *
*                       R7 - DCTMUS BASE REGISTER             *
*                                                             *
***************************************************************
* INTERFACE REQUIREMENTS:                                     *
*                                                             *
* DEPENDENCIES... QEA0 MUST HAVE PARSED AND ERROR CHECKED THE *
*                 COMMAND.  IT MUST ALSO HAVE SET UP THE      *
*                 EBW000 WORK AREA TO INDICATE A DETAILED OR  *
*                 SUMMARY REPORT WILL BE PRINTED.             *
*                                                             *
* RESTRICTIONS... - MPIF MUST BE INSTALLED.                   *
*                 - THE DRIVER MUST BE ALLOWED TO RUN IN ANY  *
*                   SUBSYSTEM-USER.                           *
*                 - "STATUS" COMMANDS CAN ONLY BE ENTERED ON  *
*                   THE MAIN I-STREAM (THIS ALSO IMPLIES      *
*                   THAT LOCKING IS NOT REQUIRED WHILE        *
*                   PROCESSING DRIVER RECORDS).               *
*                                                             *
*  ECB          * INPUT..              * OUTPUT..             *
* --------------*----------------------*----------------------*
* WORK AREA.... * EBW000 = X'00'       * NONE                 *
*               *   (DETAIL REPORT)    *                      *
*               * EBW000 = X'FF'       *                      *
*               *   (SUMMARY REPORT)   *                      *
* DATA LEVELS.. * NONE                 * NONE                 *
* REGISTERS.... * NONE                 * NONE                 *
*                                                             *
***************************************************************
*                                                             *
* EXTERNAL-REFERENCES..                                       *
*      ROUTINES....  NONE                                     *
*      DATA AREAS..  MS0AT (SUBSYSTEM ATTRIBUTE TABLE)        *
*                    MS0UT (SUBSYSTEM-USER ATTRIBUTE TABLE)   *
*                    @ISSDRV (DRIVER GLOBALS)                 *
*                        - @MPRADD  : DRIVER PRIMARY RECORD   *
*                                     ADDRESS                 *
*                        - @SYSLEV  : LEVEL OF SYSTEM TEST    *
*                                     ACTIVITY                *
*                        - @SYSINT  : FREQUENCY OF SYSTEM     *
*                                     TEST ACTIVITY           *
*                        - @SYSTEST : SYSTEM TEST ACTIVITY    *
*                                     INDICATORS              *
*                        - @EBROUT  : EBROUT SAVE AREA        *
*                        - @CE1CPD  : CPU ID SAVE AREA        *
*                                                             *
* EXIT-NORMAL.. BACKC                                         *
*     -ERROR... SERRC EEA007 - COULD NOT DECODE SYSTEM TEST   *
*                              INDICATORS                     *
*                                                             *
***************************************************************
         EJECT
***************************************************************
* ESTABLISH DSECTS, LOAD BASE REGISTERS, AND DETERMINE WHETHER
* THE USER WANTS A SUMMARY OR DETAILED REPORT.
***************************************************************
         SPACE  2
         EJECT
         DCTMPR REG=R2                 DRIVER PRIMARY RECORD DSECT
         DCTMUS REG=R7                 DRIVER SECONDARY RECORD'S DSECT
         SPACE  2
         MS0AT  REG=R5                 SUBSYSTEM ATTRIBUTE TABLE
         MS0UT  REG=R6                 SUBSYSTEM-USER ATTRIBUTE TABLE
         SPACE  2
         SD0RV  REG=R1                 DRIVER INFORMATION AREA DSECT
         GLOBZ  REGR=R4                GET GLOBAL 1 ADDRESS
         L      R1,@ISSDRV             SET @ISSDRV BASE REGISTER
         SPACE  2
         CLI    EBW000,X'00'           THE USER WANT A DETAIL REPORT?
         BNE    QEAF2000               NO  - GO GIVE SUMMARY REPORT
         EJECT
***************************************************************
* ENTRY WAS DUE TO "ZTEST MPIF STATUS" MESSAGE.  A DETAILED
* REPORT WILL BE PRINTED FOR THE DRIVER IN THIS SUBSYSTEM-USER.
***************************************************************
         SPACE 2
QEAF1000 DS    0H
         WTOPC TEXTA=MESSAGE1,TIME=YES,ENDOFM=NO,                      X
               PREFIX=QEAF,NUM=01,LET=I,CHAIN=YES
         SPACE 2
* DETERMINE THE STATE (STARTED, STARTING, STOPPING, OR STOPPED) OF THE
* THE DRIVER, AND PRINT AN APPROPRIATE MESSAGE.
         TM    @SYSTEST,SYSRUN         SYSTEM TEST LEVEL RUNNING?
         BO    QEAF1050                YES - BRANCH, CONTINUE CHECKING
         LA    R2,=CL8' STOPPED'       NO  - MSG IS "DRIVER STOPPED"
         B     QEAF1200                BRANCH TO ISSUE MESSAGE
         SPACE 2
QEAF1050 DS    0H
         TM    @SYSTEST,STARTING       IS THE DRIVER STARTING?
         BZ    QEAF1100                NO  - BRANCH, CONTINUE CHECKING
         LA    R2,=CL8'STARTING'       YES - MSG IS "DRIVER STARTING"
         B     QEAF1200                BRANCH TO ISSUE MESSAGE
         SPACE 2
QEAF1100 DS    0H
         TM    @SYSTEST,STARTED        IS THE DRIVER STARTED?
         BZ    QEAF1150                NO  - BRANCH, CONTINUE CHECKING
         LA    R2,=CL8' STARTED'       YES - MSG IS "DRIVER STARTED"
         B     QEAF1200                BRANCH TO ISSUE MESSAGE
         SPACE 2
QEAF1150 DS    0H
         TM    @SYSTEST,STOPPING       IS THE DRIVER STOPPING?
         BZ    QEAF9000                NO  - BRANCH TO ISSUE SERRC
         LA    R2,=CL8'STOPPING'       YES - MSG IS "DRIVER STOPPING"
         SPACE 2
QEAF1200 DS    0H
         WTOPC TEXT='  CONTINUOUS MPIF ACTIVITY: ........',            X
               HEADER=NO,TIME=NO,CHAIN=YES,ENDOFM=NO,SUB=(CHARA,(R2))
         EJECT
* NEXT, IF THE DRIVER IS STARTING OR STARTED, DISPLAY THE LEVEL AND
* INTERVAL PARAMETERS OF THE SYSTEM TEST RUN.  IF A VALUE IS CURRENTLY
* SET AT THE DEFAULT, IT WILL BE INDICATED IN THE MESSAGE.
         TM    @SYSTEST,STARTING+STARTED    IS THE DRIVER STARTING/ED?
         BZ    QEAF1450                     NO - SKIP THIS DISPLAY
         SPACE 2
         LA    R15,LEVDEF              LOAD THE DEFAULT LEVEL VALUE
         CH    R15,@SYSLEV             DIFFERENT FROM ACTUAL VALUE?
         BNE   QEAF1250                YES - GO SUPPRESS "DEFAULT"
         LA    R5,=CL9'(default)'      NO  - WILL PRINT WORD "DEFAULT"
         B     QEAF1300                GO PRINT THE LEVEL PARAMETER
QEAF1250 DS    0H
         LA    R5,=CL9' '              WILL NOT PRINT WORD "DEFAULT"
QEAF1300 DS    0H
         LH    R3,@SYSLEV              LOAD SYSTEM TEST LEVEL
         WTOPC TEXT='  ACTIVITY LEVEL          : ........  .........', X
               HEADER=NO,SUB=(DEC,(R3),CHARA,(R5)),                    X
               CHAIN=YES,ENDOFM=NO,TIME=NO
         SPACE 2
         LA    R15,INTDEF              LOAD THE DEFAULT INTERVAL VALUE
         CH    R15,@SYSINT             DIFFERENT FROM ACTUAL VALUE?
         BNE   QEAF1350                YES - GO SUPPRESS "DEFAULT"
         LA    R5,=CL9'(default)'      NO  - WILL PRINT WORD "DEFAULT"
         B     QEAF1400                GO PRINT THE INTERVAL PARAMETER
QEAF1350 DS    0H
         LA    R5,=CL9' '              WILL NOT PRINT WORD "DEFAULT"
QEAF1400 DS    0H
         LH    R3,@SYSINT              LOAD SYSTEM TEST INTERVAL
         WTOPC TEXT='  ACTIVITY INTERVAL       : ........  .........', X
               HEADER=NO,SUB=(DEC,(R3),CHARA,(R5)),                    X
               CHAIN=YES,ENDOFM=NO,TIME=NO
         EJECT
* NEXT, REPORT THE PRIMARY BLOCK ADDRESS.  IF THE BLOCK DOESN'T EXIST,
* AN APPROPRIATE MESSAGE WILL BE PRINTED, AND THE REPORT WILL BE
* TERMINATED.
QEAF1450 DS    0H
         OC    @MPRADD(4),@MPRADD      DOES A PRIMARY BLOCK EXIST YET?
         BNZ   QEAF1500                YES - GO REPORT IT'S ADDRESS
         WTOPC TEXT='  NO PRIMARY BLOCK OR MPIF USERS',                X
               HEADER=NO,TIME=NO,CHAIN=YES,ENDOFM=NO
         B     QEAF1850                GO FINISH THE STATUS MESSAGE
         SPACE 2
QEAF1500 DS    0H
         WTOPC TEXT='  PRIMARY BLOCK ADDRESS   : ........',HEADER=NO,  X
               TIME=NO,SUB=(HEXA,@MPRADD),CHAIN=YES,ENDOFM=NO
         WTOPC TEXT=' ',HEADER=NO,TIME=NO,CHAIN=YES,ENDOFM=NO
         EJECT
* NEXT, SEARCH THE PRIMARY BLOCK TO SEE IF THERE ARE ANY USERS
* IDENTIFIED TO THE SYSTEM.  IF NOT, ISSUE A MESSAGE SAYING SUCH,
* AND TERMINATE THE REPORT.
         L     R2,@MPRADD              PRIMARY BLOCK BASE REGISTER
         LA    R3,MPRMAX               MAXIMUM NUMBER OF USERS
         SPACE 2
QEAF1550 DS    0H
         MONTC
         OC    MPRNAM(8),MPRNAM        IS THERE A USER IN THIS SLOT?
         LMONC
         BNZ   QEAF1600                YES - CAN CONTINUE PROCESSING
         LA    R2,MPRLEN(,R2)          LOOK AT NEXT ENTRY IN TABLE
         BCT   R3,QEAF1550             BRANCH IF MORE SLOTS TO CHECK
         SPACE 2
         WTOPC TEXT='  NO IDENTIFIED MPIF USERS',                      X
               HEADER=NO,CHAIN=YES,ENDOFM=NO,TIME=NO
         B     QEAF1850                GO FINISH THE STATUS MESSAGE
         EJECT
* PRINT A TABLE HEADER, AND THEN ONE LINE OF INFORMATION FOR EACH USER
* IN THE SYSTEM.  AN '*' WILL BE PRINTED NEXT TO THE NUMBER OF
* CONNECTIONS IF THE USER HAS ONE OR MORE 'INCOMPLETE' CONNECTION.
QEAF1600 DS    0H
         WTOPC TEXT='  USER        CL    DEVICE    CONN     MESSAGES SEX
               NT   ACKNOWLEDGED',HEADER=NO,TIME=NO,ENDOFM=NO,CHAIN=YES
         L     R2,@MPRADD              PRIMARY BLOCK BASE REGISTER
         LA    R3,MPRMAX               MAXIMUM NUMBER OF USERS
         SPACE 2
QEAF1650 DS    0H
         MONTC
         OC    MPRNAM(8),MPRNAM        IS THERE A USER IN THIS SLOT?
         LMONC
         BZ    QEAF1800                NO  - GO LOOK AT NEXT SLOT
         L     R7,MPRMUS               USER'S SECONDARY BLOCK ADDRESS
         LH    R5,MUSNCN               LOAD NUMBER CONNECTIONS
         SPACE 2
         CLI   MUSDEV,DEVCTOC          IS THIS A CTOC TYPE USER?
         BNE   QEAF1700                NO  - GO REPORT MTOM TYPE USER
         MVC   EBW040(4),=C'CTOC'      SET DEVICE = CTOC FOR WTOPC
         B     QEAF1750                GO CONTINUE BUILDING MESSAGE
QEAF1700 DS    0H
         MVC   EBW040(4),=C'MTOM'      SET DEVICE = MTOM FOR WTOPC
         SPACE 2
QEAF1750 DS    0H
         MVC   EBW044(2),=C'  '        DEFAULT IS NO INCOMPLETE CONN.
         OC    MUSICON(2),MUSICON      ANY INCOMPLETE CONNECTIONS?
         BZ    QEAF1775                NO  - CAN GO ISSUE WTOPC
         MVC   EBW044(2),=C'* '        FLAG INCOMPLETE CONNECTIONS
         SPACE 2
QEAF1775 DS    0H
         WTOPC TEXT='  ........    ..     ....      .. ..  ..........  X
                  ..........',HEADER=NO,TIME=NO,CHAIN=YES,ENDOFM=NO,   X
               SUB=(CHARA,MUSNAM,CHARA,MUSPCL,CHARA,EBW040,DEC,(R5),   X
               CHARA,EBW044,DECA,MUSMSN,DECA,MUSARC)
         SPACE 2
QEAF1800 DS    0H
         LA    R2,MPRLEN(,R2)          LOOK AT NEXT ENTRY IN TABLE
         BCT   R3,QEAF1650             BRANCH IF MORE SLOTS TO CHECK
         EJECT
* THE REPORT IS COMPLETE. PRINT TRAILING LINES AND EXIT.
QEAF1850 DS    0H
         WTOPC TEXT=' ',HEADER=NO,TIME=NO,CHAIN=YES,ENDOFM=NO
         WTOPC TEXT='END OF DISPLAY',HEADER=NO,CHAIN=YES,ENDOFM=YES
         BACKC                         END OF DETAIL REPORT
         EJECT
***************************************************************
* THIS ROUTINE WILL PRINT A SUMMARY STATUS TABLE, TELLING
* WHETHER THE DRIVER IS STARTED, STARTING, STOPPED, OR STOPPING
* IN EACH SUBSYSTEM-USER.
*
* THE UATBC WILL SET UP ADDRESSABILITY TO THE MS0UT, AND WILL
* PLACE THE NUMBER OF SSU IN THE SS INTO R2.  IF THE UATBC TAKES
* THE NOTAVL BRANCH, IT MEANS THE SS IS DORMANT, AND WILL NOT
* BE REPORTED ON.
***************************************************************
         SPACE 2
QEAF2000 DS    0H
         WTOPC TEXTA=MESSAGE2,TIME=YES,ENDOFM=NO,                      X
               PREFIX=QEAF,NUM=02,LET=I,CHAIN=YES
         SPACE 2
         CINFC R,CMMSSAT,F,REG=R5      SET UP ADDRESSABILITY TO MS0AT
         L     R7,MA0CNT               LOAD NUMBER OF SUBSYSTEMS
         SPACE 2
QEAF2100 DS    0H                      LOOP ONCE FOR EACH SUBSYSTEM
         MVC   EBX010(4),MA0NAM        SAVE SS NAME FOR LATER WTOPC
         L     R6,MA0NAM               LOAD THE NAME OF SUBSYSTEM
         UATBC IDLOC=(N,SSI,R6),NOTAVL=QEAF2700,COUNT=R2
         SPACE 2
QEAF2200 DS    0H                      LOOP ONCE FOR EACH SSU
         MVC   EBX014(4),MU0NAM        SAVE SSU NAME FOR LATER WTOPC
         L     R4,MU0GLA               SET GLOBAL BASE ADDRESS FOR SSU
         L     R1,@ISSDRV              SET BASE ADDRESS FOR @ISSDRV
         SPACE 2
         TM    @SYSTEST,SYSRUN         SYSTEM TEST LEVEL RUNNING?
         BO    QEAF2300                YES - BRANCH, CONTINUE CHECKING
         LA    R3,=CL8'STOPPED'        NO  - INDICATE DRIVER STOPPED
         B     QEAF2600                BRANCH TO ISSUE MESSAGE
         SPACE 2
QEAF2300 DS    0H
         TM    @SYSTEST,STARTING       IS THE DRIVER STARTING?
         BZ    QEAF2400                NO  - BRANCH, CONTINUE CHECKING
         LA    R3,=CL8'STARTING'       YES - INDICATE DRIVER STARTING
         B     QEAF2600                BRANCH TO ISSUE MESSAGE
         SPACE 2
QEAF2400 DS    0H
         TM    @SYSTEST,STARTED        IS THE DRIVER STARTED?
         BZ    QEAF2500                NO  - BRANCH, CONTINUE CHECKING
         LA    R3,=CL8'STARTED'        YES - INDICATE DRIVER STARTED
         B     QEAF2600                BRANCH TO ISSUE MESSAGE
         SPACE 2
QEAF2500 DS    0H
         TM    @SYSTEST,STOPPING       IS THE DRIVER STOPPING?
         BZ    QEAF9000                NO  - BRANCH TO ISSUE SERRC
         LA    R3,=CL8'STOPPING'       YES - INDICATE DRIVER STOPPING
         SPACE 2
QEAF2600 DS    0H
         WTOPC TEXT='  ....    ....     ........',ENDOFM=NO,CHAIN=YES, X
               SUB=(CHARA,EBX010,CHARA,EBX014,CHARA,(R3)),HEADER=NO
         SPACE 2
         MVC   EBX010(4),=CL4' '       CLEAR SS NAME FOR NEXT MESSAGE
         AHI   R6,MU0LEN               BUMP TO NEXT SSU ENTRY           64ML3221
         BCT   R2,QEAF2200             BRANCH IF MORE SSU IN THIS SS
         SPACE 2
QEAF2700 DS    0H
         LA    R5,SSATSZ(,R5)          BUMP TO NEXT SS ENTRY
         BCT   R7,QEAF2100             BRANCH IF MORE SS TO DO
         SPACE 2
         WTOPC TEXT=' ',HEADER=NO,TIME=NO,CHAIN=YES,ENDOFM=NO
         WTOPC TEXT='END OF DISPLAY',HEADER=NO,CHAIN=YES,ENDOFM=YES
         BACKC                         END OF SUMMARY REPORT
         EJECT
***************************************************************
* THE FOLLOWING ROUTINE WILL ISSUE A SERRC BECAUSE THE SYSTEM
* TEST INDICATORS HAVE AN UNKNOWN VALUE.
***************************************************************
         SPACE 2
QEAF9000 DS    0H
         LA    R0,EEA7MSG              WILL APPEND MESSAGE TO DUMP
         SERRC E,EEA007,MSG,OP,ECB     DUMP, CAN'T DECODE INDICATORS
EEA7MSG  DS    0D
         DC    AL1(ENDEEA7-BEGEEA7)
BEGEEA7  DC    C'CAN NOT DECODE CONTINUOUS ACTIVITY INDICATORS FOR A '
         DC    C'STATUS REQUEST'
ENDEEA7  EQU   *
         EJECT
***************************************************************
* THE FOLLOWING ARE WTOPC MESSAGE TEXTS.
***************************************************************
         SPACE 2
MESSAGE1 DS    0F
         DC    AL1(END1-BEGIN1)
BEGIN1   DC    C'MPIF DRIVER DETAIL STATUS',AL1(#CAR)
END1     EQU   *
         SPACE 2
MESSAGE2 DS    0F
         DC    AL1(END2-BEGIN2)
BEGIN2   DC    C'MPIF DRIVER SUMMARY STATUS',AL1(#CAR)
END2     EQU   *
         EJECT
         LTORG
         FINIS QEAF
         END   ,
